// crt-boot.c

#include "exec.h"
#include "F205/undocumented.h"
#include "F205/iwdg.h"
#include "F205/rcc.h"

extern int  data_start; // initialisierte statische & globale Daten im RAM
extern int  data_end;   // Ende derselben
extern int  text_end;   // die initialen Werte dazu im Flash
//                      // direkt im Anschluss an das Programm
extern int  bss_start;  // mit 0 initialisierte statische & globale Daten
extern int  bss_end;    // Ende des Datenbereichs

extern int  stack;
extern int  tos;         // (diese 7 werden im Linker Script definiert)

uint32_t volatile timestamp0, timestamp1, timestamp2;


void __attribute__ ((naked)) // es funktioniert sogar mit prolog
crt_boot (void)
{
   uint32_t  reset_flags;
   uint32_t  ts0;
   int      *flash, *ram;

// der IWDG muss hier gestartet werden, der Start per Option Byte lohnt sich
// nicht oder knnte sogar nachteilig sein. Damit er hier gestartet werden
// kann muss genau 1 Wort in der Vektortabelle und die ersten 2K des Flash
// in Ordnung sein.

// so aufwendig initialisiert werden. Prescaler und RLR
// haben zwar definierte Reset-Werte, aber bei welchem Reset gelten die?

   IWDG->KR = IWDG_START;
   while (IWDG->SR_PVU || IWDG->SR_RVU) { }
   IWDG->KR = IWDG_UNLOCK;
   IWDG->PR = IWDG_PR_DIV_128;
   IWDG->RLR = 4095;

   reset_flags = RCC->CSR >> 24;

   DEBUG->TRCENA = 1;
   DWT->CYCCNT = 0;
   DWT->CYCCNT_EN = 1;
// DWT->CTRL_SLEEPEVTENA = 1; // leider unbenutzbar, luft alle 16us ber
   ts0 = DWT->CYCCNT;

// RAM Initialisierung
// Die dauert zwar ziemlich lange, aber in der Zwischenzeit kann sich die
// Betriebsspannung stabilisieren. Die Ausgangspins sind ja schon aktiv.

// FIXME  Hier wird erstmal das gesamte RAM gelscht.
   for (ram = (int *)0x2001C000; ram < (int *)0x20020000; ram++) {
      *ram = 0;
   }
   for (flash = &text_end, ram = &data_start; ram < &data_end; /* */ ) {
      *ram++ = *flash++;
   }
   // for (ram = &_bss_start; ram < &_bss_end; ram++) {
   //    *ram = 0;
   // }
   for (ram = &stack; ram < &tos; ram++) {
      *ram = 0x42424242;
   }

// Hier koennte man den Process Stack Pointer initialisieren und
// per CONTROL-Register auf ihn umschalten. In dem Register wird
// ggf. auch die Floating Point Unit konfiguriert.
//
// Das bringt aber nicht viel, wenn man kein Betriebssystem hat.

// Programmstart

   RCC->CSR_RMVF = 1;

   timestamp0 = ts0;
   timestamp1 = DWT->CYCCNT;
   gpio_init ();
   timestamp2 = DWT->CYCCNT;
   led (42);

   main (reset_flags);
   while (1);
}

