// gpio-init.c

#include <stddef.h>
#include "F205/gpio.h"
#include "F205/rcc.h"
#include "asm.h"
#include "exec.h"
#include "gpio-init.h"

typedef enum port_letter_enum {
   PORT_UNDEFINDED, A, B, C, D, E, F, G, H, I
} port_letter_enum;

#define PIN(port,bit,signal,mode,type,speed,pull,data) \
{{{AF_P##port##bit##_##mode, TYPE_##type, data, SPEED_##speed, PULL_##pull,0}}},

const gpio_init_struct  padlist[] = {
#include "mps1401.h"
   {{{ 0 }}}
};


void
gpio_init (void)
{
   gpio_struct            *gpio;
   gpio_init_struct const *pad;
   int                     port, bit;
   uint32_t                af[2], mode, type, data, speed, pull;

   RCC->AHB1ENR |= (  1 << AHB1_GPIOA) | (1 << AHB1_GPIOB)
                   | (1 << AHB1_GPIOC) | (1 << AHB1_GPIOD)
                   | (1 << AHB1_GPIOE) | (1 << AHB1_GPIOF)
                   | (1 << AHB1_GPIOG) | (1 << AHB1_GPIOH)
                   | (1 << AHB1_GPIOI);
   RCC->AHB2ENR = 0;
   RCC->AHB3ENR = 0;
//RCC->APB2ENR |= (1 << APB2_USART1) | (1 << APB2_USART6);
   DSB ();

   for (pad = padlist; pad->i.function != 0; pad++) {
      switch (pad->o.rccport) {
      case 1:
         RCC->APB1ENR |= 1 << pad->o.rccbit;
         break;
      case 2:
         RCC->APB2ENR |= 1 << pad->o.rccbit;
         break;
      case 3:
         RCC->APB1ENR |= (1 << APB1_CAN1) | (1 << APB1_CAN2);
         break;
      }
   }
   DSB ();

   for (port = 'A' - '@'; port <= 'I' - '@'; port++) {
      af[0] = 0;
      af[1] = 0;
      mode = 0;
      type = 0;
      data = 0;
      speed = 0;
      pull = 0xaaaaaaaa;  // alle Pull-Down einschalten
//    lock = 0;
      for (pad = padlist; pad->i.function != 0; pad++) {
         if (pad->o.port != port) { continue; }
         bit = pad->o.pin;
         pull &= ~ (3 << (bit * 2));
         if (pad->o.pin_mode == GM_NC) {
            pull |= 2 << (bit * 2);
         } else {
            af[bit >> 3] |= pad->o.af << ((bit & 7) * 4);
//          lock  |= pad->o.lock      << (bit * 1);
            mode  |= pad->o.pin_mode  << (bit * 2);
            type  |= pad->o.otype     << (bit * 1);
            data  |= pad->o.odata     << (bit * 1);
            speed |= pad->o.ospeed    << (bit * 2);
            pull  |= pad->o.pupd      << (bit * 2);
         }
      }
      gpio = GPIOA + (port - 1);
      gpio->OTYPER  = type;
      gpio->OSPEEDR = speed;
      gpio->PUPDR   = pull;
      gpio->ODR     = data;
      gpio->AFRL    = af[0];
      gpio->AFRH    = af[1];
      gpio->MODER   = mode;
//    if (lock != 0) {
//       gpio->LCKR  = lock | 65536;
//       gpio->LCKR  = lock;
//       gpio->LCKR  = lock | 65536;
//    }
   }
   return;
}

