// imghdr-boot.c

#include <stddef.h>
#include "image-header.h"

extern isr_type   crt_boot, crash;
extern uint32_t   tos, text_start;

const image_struct __attribute__ ((section (".image_header")))  header = {
   &tos,                                    // 00  0 Initial SP (linker script)
   .crt_boot          = crt_boot,           // 04  1 Initial PC "Reset-Handler"
   .css_nmi           = crash,              // 08  2 NMI, Clock security system
   .hard_fault        = crash,              // 0c  3 Hard fault (double fault)
   .name              = "boot",             // 10
   .version_string    = "V00.38",           // 18
   .verno             = 0,                  // 20
   .revno             = 38,                 // 21
   .image_type        = IT_BOOTLOADER,      // 23
   .source_date_epoch = 0,                  // 24
   .syslib_version    = 0,                  // 28
   .shm_version       = 0,                  // 2c
   .end               = 0,                  // 30
   .crc               = (uint32_t) &header, // 34
   .syslog_text       = NULL,               // 38
   .const_start       = NULL,               // 3c  (systick)
   .ram_start         = 0,                  // 40  (window watchdog)
   .bss_end           = 0,                  // 44  (PVD)
   .text_start        = &header,            // 48  (RTC)
   .start_delay       = 0,                  // 4c  (RTC wakeup)
   .privilege         = 1,                  // 4e
   .prio              = 3                   // 4f
};

// &tos,                  //  00   0  Initial SP, defined in linker script
// crt0,                  //  04   1  Initial PC = Reset-"Handler"
// crash,                 //  08   2  CSS Clock security system (NMI)
// crash,                 //  0c   3  Hard fault (double fault)
// crash,                 //  10   4  Memory protection
// crash,                 //  14   5  Bus fault
// crash,                 //  18   6  Usage fault
// 0,                     //  1c   7  -
// 0,                     //  20   8  -
// 0,                     //  24   9  -
// 0,                     //  28  10  -
// 0,                     //  29      -
// 0,                     //  30      -
// crash,                 //  2c  11  SVC instruction
// crash,                 //  30  12  Debug monitor
// NULL,                  //  34  13  -                 syslog text
// crash,                 //  38  14  Pendable request for system service
// crash,                 //  3c  15  System tick timer
  
// crash,                 //  40  16   0  Window watchdog interrupt
// crash,                 //  44  17   1  PVD thru EXTI line detection
// crash,                 //  48  18   2  RTC Tamper & TimeStamp (EXTI)
// crash,                 //  4c  19   3  RTC Wakeup (EXTI)

// crash,                 //  50  20   4  Flash global interrupt
// crash,                 //  54  21   5  RCC
// crash,                 //  58  22   6  EXTI0
// crash,                 //  5c  23   7  EXTI1
// crash,                 //  60  24   8  EXTI2
// crash,                 //  64  25   9  EXTI3
// crash,                 //  68  26  10  EXTI4
// crash,                 //  6c  27  11  DMA1 Stream 0
// crash,                 //  70  28  12  DMA1 Stream 1
// crash,                 //  74  29  13  DMA1 Stream 2
// crash,                 //  78  30  14  DMA1 Stream 3
// crash,                 //  7c  31  15  DMA1 Stream 4
// crash,                 //  80  32  16  DMA1 Stream 5
// crash,                 //  84  33  17  DMA1 Stream 6
// crash,                 //  88  34  18  ADC1, ADC2 and ADC3
// crash,                 //  8c  35  19  CAN1 TX
// crash,                 //  90  36  20  CAN1 RX0
// crash,                 //  94  37  21  CAN1 RX1
// crash,                 //  98  38  22  CAN1 SCE
// crash,                 //  9c  39  23  EXTI Line [9:5]
// crash,                 //  a0  40  24  TIM1 Break, TIM9
// crash,                 //  a4  41  25  TIM1 Update, TIM10
// crash,                 //  a8  42  26  TIM1 Trigger and Commutation, TIM11
// crash,                 //  ac  43  27  TIM1 Capture Compare
// crash,                 //  b0  44  28  TIM2
// crash,                 //  b4  45  29  TIM3
// crash,                 //  b8  46  30  TIM4
// crash,                 //  bc  47  31  I2C1 event
// crash,                 //  c0  48  32  I2C1 error
// crash,                 //  c4  49  33  I2C2 event
// crash,                 //  c8  50  34  I2C2 error
// crash,                 //  cc  51  35  SPI1
// crash,                 //  d0  52  36  SPI2
// crash,                 //  d4  53  37  USART1
// crash,                 //  d8  54  38  USART2
// crash,                 //  dc  55  39  USART3
// crash,                 //  e0  56  40  EXTI Line [15:10]
// crash,                 //  e4  57  41  RTC Alarm (EXTI)
// crash,                 //  e8  58  42  USB OTG FS Wakeup (EXTI)
// crash,                 //  ec  59  43  TIM8 Break, TIM12
// crash,                 //  f0  60  44  TIM8 Update, TIM13
// crash,                 //  f4  61  45  TIM8 Trigger and Commutation, TIM14
// crash,                 //  f8  62  46  TIM8 Capture Compare
// crash,                 //  fc  63  47  DMA1 Stream 7
// crash,                 // 100  64  48  FSMC
// crash,                 // 104  65  49  SDIO
// crash,                 // 108  66  50  TIM5
// crash,                 // 10c  67  51  SPI3
// crash,                 // 110  68  52  UART4
// crash,                 // 114  69  53  UART5
// crash,                 // 118  70  54  TIM6, DAC1 & DAC2 underrun error
// crash,                 // 11c  71  55  TIM7
// crash,                 // 120  72  56  DMA2 Stream 0
// crash,                 // 124  73  57  DMA2 Stream 1
// crash,                 // 128  74  58  DMA2 Stream 2
// crash,                 // 12c  75  59  DMA2 Stream 3
// crash,                 // 130  76  60  DMA2 Stream 4
// crash,                 // 134  77  61  Ethernet
// crash,                 // 138  78  62  Ethernet Wakeup (EXTI)
// crash,                 // 13c  79  63  CAN2 TX
// crash,                 // 140  80  64  CAN2 RX0
// crash,                 // 144  81  65  CAN2 RX1
// crash,                 // 148  82  66  CAN2 SCE
// crash,                 // 14c  83  67  USB OTG FS
// crash,                 // 150  84  68  DMA2 Stream 5
// crash,                 // 154  85  69  DMA2 Stream 6
// crash,                 // 158  86  70  DMA2 Stream 7
// crash,                 // 15c  87  71  USART6
// crash,                 // 160  88  72  I2C3 event
// crash,                 // 164  89  73  I2C3 error
// crash,                 // 168  90  74  USB OTG HS End Point 1 Out
// crash,                 // 16c  91  75  USB OTG HS End Point 1 In
// crash,                 // 170  92  76  USB OTG HS Wakeup (EXTI)
// crash,                 // 174  93  77  USB OTG HS
// crash,                 // 178  94  78  DCMI
// crash,                 // 17c  95  79  CRYP crypto
// crash,                 // 180  96  80  Hash and RNG
// crash,                 // 184  97  81
// crash,                 // 188  98  82
// crash,                 // 18c  99  83
// crash,                 // 190 100  84
// crash,                 // 194 101  85
// crash,                 // 198 102  86
// crash,                 // 19c 103  87
// crash,                 // 1a0 104  88
// crash,                 // 1a4 105  89
// crash,                 // 1a8 106  90
// crash,                 // 1ac 107  91
// crash,                 // 1b0 108  92
// crash,                 // 1b4 109  93
// crash,                 // 1b8 110  94
// crash,                 // 1bc 111  95
// crash,                 // 1c0 112  96
// crash,                 // 1c4 113  97
// crash,                 // 1c8 114  98
// crash,                 // 1cc 115  99
// crash,                 // 1d0 116 100
// crash,                 // 1d4 117 101
// crash,                 // 1d8 118 102
// crash,                 // 1dc 119 103
// crash,                 // 1e0 120 104
// crash,                 // 1e4 121 105
// crash,                 // 1e8 122 106
// crash,                 // 1ec 123 107
// crash,                 // 1f0 124 108
// crash,                 // 1f4 125 109
// crash,                 // 1f8 126 110
// crash                  // 1fc 127 111

