// DS2484.h
#pragma once

// DS2484 -- I2C/1-Wire Master

static const int I2C_DS2484 = 24;  // i2c Adressen haben 7 Bit und enthalten
                                   // kein Read/Write-Bit. Es gibt keine
                                   // getrennten Lese- und Schreib-Adressen.
                                   // Die beste Schreibweise wre 0b0011000,
                                   // aber das erlaubt C11 nicht.
typedef enum DS2484_command_enum {
  DEVICE_RESET     = 0xF0,  //  525 us
  SET_READ_POINTER = 0xE1,  //    0
  WRITE_DEV_CONFIG = 0xD2,  //    0
  ADJUST_W1_PORT   = 0xC3,  //    0
  W1_RESET         = 0xB4,  // 1200 us
  W1_SINGLE_BIT    = 0x87,  //   75 us
  W1_WRITE_BYTE    = 0xA5,  //  600 us
  W1_READ_BYTE     = 0x96,  //  600 us
  W1_TRIPLET       = 0x78   //  225 us
} DS2484_command_enum;

typedef enum DS2484_read_pointer_code_enum {
  DEVICE_CONFIG = 0xC3,
  STATUS        = 0xF0,
  READ_DATA     = 0xE1,
  PORT_CONFIG   = 0xB4
} DS2484_read_pointer_code_enum;

typedef enum DS2484_device_config_bit_enum {
  APU = 1,  // active pullup
  PDN = 2,  // 1-wire power down
  SPU = 4,  // strong pullup
  W1S = 8   // 1-wire speed
} DS2484_device_config_bit_enum;

typedef enum DS2484_device_config_enum {
  IDLE_CONFIG = APU | ((~APU & 0xF) << 4),
  SPU_CONFIG  = APU | SPU | ((~(APU | SPU) & 0xF) << 4),
  POWER_DOWN  = PDN | ((~PDN & 0xF) << 4)
} DS2484_device_config_enum;

typedef enum DS2484_status_bit_enum {
  W1B =   1,  // busy
  PPD =   2,  // presence pulse detect
  SD  =   4,  // short detected
  LL  =   8,  // logic level
  RST =  16,  // device reset
  SBR =  32,  // single bit result
  TSB =  64,  // triplet second bit
  DIR = 128  // branch direction taken
} DS2484_status_bit_enum;

// DS2431 -- 1024 Bit EEPROM

typedef enum one_wire_commands_enum {
  READ_ROM          = 0x33,
  MATCH_ROM         = 0x55,
  SEARCH_ROM        = 0xF0,
  SKIP_ROM          = 0xCC,
  RESUME            = 0xA5,
// memory commands
  WRITE_SCRATCHPAD  = 0x0F,
  READ_SCRATCHPAD   = 0xAA,
  COPY_SCRATCHPAD   = 0x55,
  READ_MEMORY       = 0xF0
} one_wire_commands_enum;

