// adc.h
#pragma once
#include <stdint.h>

typedef enum adc_resolution_enum {
  RES_12_BIT,
  RES_10_BIT,
  RES_8_BIT,
  RES_6_BIT
} adc_resolution_enum;

typedef enum adc_sampling_enum {
  SMP_3,
  SMP_15,
  SMP_28,
  SMP_56,
  SMP_84,
  SMP_112,
  SMP_144,
  SMP_480
} adc_sampling_enum;

typedef enum adc_prescaler_enum {
  APB2_DIV_2,
  APB2_DIV_4,
  APB2_DIV_6,
  APB2_DIV_8
} adc_prescaler_enum;

typedef volatile struct /* adc_struct */ {  // word access
// 00 Status register (ADC_SR)
  uint32_t  SR_AWD       :  1; // Analog watchdog flag
  uint32_t  SR_EOC       :  1; // Regular channel end of conversion
  uint32_t  SR_JEOC      :  1; // Injected channel end of conversion
  uint32_t  SR_JSTRT     :  1; // Injected channel start flag
  uint32_t  SR_STRT      :  1; // Regular channel start flag
  uint32_t  SR_OVR       :  1; // Overrun
  uint32_t               : 26;
// 04 Control register 1 (ADC_CR1)
  uint32_t  CR1_AWDCH    :  5; // Analog watchdog channel select bits
  uint32_t  CR1_EOCIE    :  1; // Interrupt enable for EOC
  uint32_t  CR1_AWDIE    :  1; // Analog watchdog interrupt enable
  uint32_t  CR1_JEOCIE   :  1; // Interrupt enable for injected channels
  uint32_t  CR1_SCAN     :  1; // Scan mode
  uint32_t  CR1_AWDSGL   :  1; // Enable the watchdog on a single
                               //   channel in scan mode
  uint32_t  CR1_JAUTO    :  1; // Automatic injected group conversion
  uint32_t  CR1_DISCEN   :  1; // Discontinuous mode on regular channels
  uint32_t  CR1_JDISCEN  :  1; // Discontinuous mode on injected channels
  uint32_t  CR1_DISCNUM  :  3; // Discontinuous mode channel count
  uint32_t               :  6;
  uint32_t  CR1_JAWDEN   :  1; // Analog watchdog enable on injected channels
  uint32_t  CR1_AWDEN    :  1; // Analog watchdog enable on regular channels
  uint32_t  CR1_RES      :  2; // Resolution
  uint32_t  CR1_OVRIE    :  1; // Overrun interrupt enable
  uint32_t               :  5;
// 08 Control register 2 (ADC_CR2)
  uint32_t  CR2_ADON     :  1; // A/D Converter ON / OFF
  uint32_t  CR2_CONT     :  1; // Continuous conversion
  uint32_t               :  6;
  uint32_t  CR2_DMA      :  1; // Direct memory access mode (for single ADC)
  uint32_t  CR2_DDS      :  1; // DMA disable selection (for single ADC mode)
  uint32_t  CR2_EOCS     :  1; // End of conversion selection
  uint32_t  CR2_ALIGN    :  1; // Data alignment
  uint32_t               :  4;
  uint32_t  CR2_JEXTSEL  :  4; // External event select for injected group
  uint32_t  CR2_JEXTEN   :  2; // External trigger enable for injected channels
  uint32_t  CR2_JSWSTART :  1; // Start conversion of injected channels
  uint32_t               :  1;
  uint32_t  CR2_EXTSEL   :  4; // External event select for regular group
  uint32_t  CR2_EXTEN    :  2; // External trigger enable for regular channels
  uint32_t  CR2_SWSTART  :  1; // Start conversion of regular channels
  uint32_t               :  1;
// 0C Sample time register 1 (ADC_SMPR1)
  uint32_t  SMPR1;
#if 0
  uint32_t  SMPR1_SMP10  :  3; // Channel 10 sampling time selection
  uint32_t  SMPR1_SMP11  :  3; // Channel 11 sampling time selection
  uint32_t  SMPR1_SMP12  :  3; // Channel 12 sampling time selection
  uint32_t  SMPR1_SMP13  :  3; // Channel 13 sampling time selection
  uint32_t  SMPR1_SMP14  :  3; // Channel 14 sampling time selection
  uint32_t  SMPR1_SMP15  :  3; // Channel 15 sampling time selection
  uint32_t  SMPR1_SMP16  :  3; // Channel 16 sampling time selection
  uint32_t  SMPR1_SMP17  :  3; // Channel 17 sampling time selection
  uint32_t  SMPR1_SMP18  :  3; // Channel 18 sampling time selection
  uint32_t               :  5;
#endif
// 10 Sample time register 2 (ADC_SMPR2)
  uint32_t  SMPR2;
#if 0
  uint32_t  SMPR2_SMP0   :  3; // Channel 0 sampling time selection
  uint32_t  SMPR2_SMP1   :  3; // Channel 1 sampling time selection
  uint32_t  SMPR2_SMP2   :  3; // Channel 2 sampling time selection
  uint32_t  SMPR2_SMP3   :  3; // Channel 3 sampling time selection
  uint32_t  SMPR2_SMP4   :  3; // Channel 4 sampling time selection
  uint32_t  SMPR2_SMP5   :  3; // Channel 5 sampling time selection
  uint32_t  SMPR2_SMP6   :  3; // Channel 6 sampling time selection
  uint32_t  SMPR2_SMP7   :  3; // Channel 7 sampling time selection
  uint32_t  SMPR2_SMP8   ;  3; // Channel 8 sampling time selection
  uint32_t  SMPR2_SMP9   :  3; // Channel 9 sampling time selection
  uint32_t               :  2;
#endif
// 14 injected channel data offset register x (ADC_JOFRx) (x=1..4)
  uint32_t  JOFR1;
// 18
  uint32_t  JOFR2;
// 1C
  uint32_t  JOFR3;
// 20
  uint32_t  JOFR4;
// 24 Watchdog higher threshold register (ADC_HTR)
  uint32_t  HTR;
// 28 Watchdog lower threshold register (ADC_LTR)
  uint32_t  LTR;
// 2C Regular sequence register 1 (ADC_SQR1)
  uint32_t  SQR1_SQ13    :  5; // 13th conversion in regular sequence
  uint32_t  SQR1_SQ14    :  5; // 14th conversion in regular sequence
  uint32_t  SQR1_SQ15    :  5; // 15th conversion in regular sequence
  uint32_t  SQR1_SQ16    :  5; // 16th conversion in regular sequence
  uint32_t  SQR1_L       :  4; // Regular channel sequence length
  uint32_t               :  8;
// 30 Regular sequence register 2 (ADC_SQR2)
  uint32_t  SQR2_SQ7     :  5; // 7th conversion in regular sequence
  uint32_t  SQR2_SQ8     :  5; // 8th conversion in regular sequence
  uint32_t  SQR2_SQ9     :  5; // 9th conversion in regular sequence
  uint32_t  SQR2_SQ10    :  5; // 10th conversion in regular sequence
  uint32_t  SQR2_SQ11    :  5; // 11th conversion in regular sequence
  uint32_t  SQR2_SQ12    :  5; // 12th conversion in regular sequence
  uint32_t               :  2;
// 34 Regular sequence register 3 (ADC_SQR3)
  uint32_t  SQR3_SQ1     :  5; // 1st conversion in regular sequence
  uint32_t  SQR3_SQ2     :  5; // 2nd conversion in regular sequence
  uint32_t  SQR3_SQ3     :  5; // 3rd conversion in regular sequence
  uint32_t  SQR3_SQ4     :  5; // 4st conversion in regular sequence
  uint32_t  SQR3_SQ5     :  5; // 5st conversion in regular sequence
  uint32_t  SQR3_SQ6     :  5; // 6st conversion in regular sequence
  uint32_t               :  2;
// 38 Injected sequence register (ADC_JSQR)
  uint32_t  JSQR;
#if 0
  uint32_t  JSQ1         :  5; // 1st conversion in injected sequence
                               //   (when JL[1:0]=3, see note RM0033)
  uint32_t  JSQ2         :  5; // 1st conversion in injected sequence
  uint32_t  JSQ3         :  5; // 1st conversion in injected sequence
  uint32_t  JSQ4         :  5; // 1st conversion in injected sequence
  uint32_t  JL           :  2; // Injected sequence length
  uint32_t               : 10;
#endif
// 3C Injected data register x (ADC_JDRx) (x= 1..4)
  uint32_t  JDR1;
// 40
  uint32_t  JDR2;
// 44
  uint32_t  JDR3;
// 48
  uint32_t  JDR4;
// 4C Regular data register (ADC_DR)
  uint32_t  DR;
} adc_struct;

_Static_assert (sizeof(adc_struct) == 80, " adc_struct: bad size");


typedef volatile struct /* adc_common_struct */ {  // word access
// 00 Common status register (ADC_CSR)
  uint32_t  CSR_AWD1    :  1; // Analog watchdog flag of ADC1
  uint32_t  CSR_EOC1    :  1; // End of conversion of ADC1
  uint32_t  CSR_JEOC1   :  1; // Injected channel end of conversion of ADC1
  uint32_t  CSR_JSTRT1  :  1; // Injected channel Start flag of ADC1
  uint32_t  CSR_STRT1   :  1; // Regular channel Start flag of ADC1
  uint32_t  CSR_OVR1    :  1; // Overrun flag of ADC1
  uint32_t              :  2;
  uint32_t  CSR_AWD2    :  1; // Analog watchdog flag of ADC1
  uint32_t  CSR_EOC2    :  1; // End of conversion of ADC1
  uint32_t  CSR_JEOC2   :  1; // Injected channel end of conversion of ADC1
  uint32_t  CSR_JSTRT2  :  1; // Injected channel Start flag of ADC1
  uint32_t  CSR_STRT2   :  1; // Regular channel Start flag of ADC1
  uint32_t  CSR_OVR2    :  1; // Overrun flag of ADC1
  uint32_t              :  2;
  uint32_t  CSR_AWD3    :  1; // Analog watchdog flag of ADC1
  uint32_t  CSR_EOC3    :  1; // End of conversion of ADC1
  uint32_t  CSR_JEOC3   :  1; // Injected channel end of conversion of ADC1
  uint32_t  CSR_JSTRT3  :  1; // Injected channel Start flag of ADC1
  uint32_t  CSR_STRT3   :  1; // Regular channel Start flag of ADC1
  uint32_t  CSR_OVR3    :  1; // Overrun flag of ADC1
  uint32_t              :  2;
  uint32_t              :  8;
// 04 Common control register (ADC_CCR)
  uint32_t  CCR_MULTI   :  5; // Multi ADC mode selection
  uint32_t              :  3; //
  uint32_t  CCR_DELAY   :  4; // Delay between 2 sampling phases
  uint32_t              :  1; //
  uint32_t  CCR_DDS     :  1; // DMA disable selection (for multi-ADC mode)
  uint32_t  CCR_DMA     :  2; // Direct memory access mode for multi ADC mode
  uint32_t  CCR_ADCPRE  :  2; // ADC prescaler
  uint32_t              :  4; //
  uint32_t  CCR_VBATE   :  1; // VBAT enable
  uint32_t  CCR_TSVREFE :  1; // Temperature sensor and VREFINT enable
  uint32_t              :  8; //
// 08 Common regular data register for dual and triple modes
  uint32_t  CDR;
} adc_common_struct;

static adc_common_struct * const ADC  = (adc_common_struct *)0x40012300;
static adc_struct *        const ADC3 =        (adc_struct *)0x40012200;
static adc_struct *        const ADC2 =        (adc_struct *)0x40012100;
static adc_struct *        const ADC1 =        (adc_struct *)0x40012000;

