// dbgmcu.h

// RM0033 Reference manual F205, F215, F207, F217

#pragma once
#include <stdint.h>

typedef volatile struct /* dbgmcu_struct */ {  // word access
// 00 MCU device ID code (DBGMCU_IDCODE)
  uint32_t  IDCODE_DEV_ID          : 12; // Device identifier
                                         //   F20x = F21x = 0x411
  uint32_t  IDCODE_RSVD            :  4;
  uint32_t  IDCODE_REV_ID          : 16; // Revision identifier
// 04 Debug MCU configuration register (DBGMCU_CR)
  uint32_t  DBG_SLEEP              :  1; // Debug Sleep mode
  uint32_t  DBG_STOP               :  1; // Debug Stop mode
  uint32_t  DBG_STANDBY            :  1; // Debug Standby mode
  uint32_t                         :  2;
  uint32_t  TRACE_IOEN             :  1; // Trace pin assignment control
  uint32_t  TRACE_MODE             :  2; // Trace pin assignment control
  uint32_t                         : 24;
// 08 Debug MCU APB1 freeze register (DBGMCU_APB1_FZ)
  uint32_t  DBG_TIM2_STOP          :  1; // TIM2 counter stopped
                                         //   when core is halted
  uint32_t  DBG_TIM3_STOP          :  1; // TIM3
  uint32_t  DBG_TIM4_STOP          :  1; // TIM4
  uint32_t  DBG_TIM5_STOP          :  1; // TIM5
  uint32_t  DBG_TIM6_STOP          :  1; // TIM6
  uint32_t  DBG_TIM7_STOP          :  1; // TIM7
  uint32_t  DBG_TIM12_STOP         :  1; // TIM12
  uint32_t  DBG_TIM13_STOP         :  1; // TIM13
  uint32_t  DBG_TIM14_STOP         :  1; // TIM14
  uint32_t                         :  1;
  uint32_t  DBG_RTC_STOP           :  1; // RTC stopped when Core is halted
  uint32_t  DBG_WWDG_STOP          :  1; // Window Watchdog stopped
                                         //   when Core is halted
  uint32_t  DBG_IWDG_STOP          :  1; // Independent watchdog stopped
                                         //   when core is halted
  uint32_t                         :  8;
  uint32_t  DBG_I2C1_SMBUS_TIMEOUT :  1; // SMBUS timeout mode stopped
                                         //   when Core is halted
  uint32_t  DBG_I2C2_SMBUS_TIMEOUT :  1; // SMBUS timeout mode stopped
                                         //   when Core is halted
  uint32_t  DBG_I2C3_SMBUS_TIMEOUT :  1; // SMBUS timeout mode stopped
                                         //   when Core is halted
  uint32_t                         :  1;
  uint32_t  DBG_CAN1_STOP          :  1; // Debug CAN2 stopped
                                         //   when Core is halted
  uint32_t  DBG_CAN2_STOP          :  1; // Debug CAN2 stopped
                                         //   when Core is halted
  uint32_t                         :  5;
// 0C Debug MCU APB2 Freeze register (DBGMCU_APB2_FZ)
  uint32_t  DBG_TIM1_STOP          :  1; // TIM1 counter stopped
                                         //   when core is halted
  uint32_t  DBG_TIM8_STOP          :  1; // TIM8 counter stopped
                                         //   when core is halted
  uint32_t                         : 14;
  uint32_t  DBG_TIM9_STOP          :  1; // TIM9 counter stopped
                                         //   when core is halted
  uint32_t  DBG_TIM10_STOP         :  1; // TIM10 counter stopped
                                         //   when core is halted
  uint32_t  DBG_TIM11_STOP         :  1; // TIM11 counter stopped
                                         //   when core is halted
  uint32_t                         : 13;
} dbgmcu_struct;

_Static_assert (sizeof(dbgmcu_struct) == 16, " dbgmcu_struct: bad size");

static dbgmcu_struct * const DBGMCU = (dbgmcu_struct *)0xE0042000;

