// flash.h
#pragma once
#include <stdint.h>

static const uint32_t  FLASH_KEY1 = 0x45670123;
static const uint32_t  FLASH_KEY2 = 0xCDEF89AB;
static const uint32_t  FLASH_OPTKEY1 = 0x08192A3B;
static const uint32_t  FLASH_OPTKEY2 = 0x4C5D6E7F;
static const uint32_t  FLASH_PSIZE_8_BIT  = 0;
static const uint32_t  FLASH_PSIZE_16_BIT = 1;
static const uint32_t  FLASH_PSIZE_32_BIT = 2;
static const uint32_t  FLASH_PSIZE_64_BIT = 3;

typedef volatile struct /* flash_struct */ {
// 00 register (FLASH_ACR)
  uint32_t  LATENCY     :  3;   // No. of CPU wait states
  uint32_t              :  5;
  uint32_t  PRFTEN      :  1;   // Prefetch enable
  uint32_t  ICEN        :  1;   // Instruction cache enable
  uint32_t  DCEN        :  1;   // Data cache enable
  uint32_t  ICRST       :  1;   // Instruction cache reset
  uint32_t  DCRST       :  1;   // Data cache reset
  uint32_t              : 19;
// 04 register (FLASH_KEYR)
  uint32_t  KEY;                // Flash key; word access
// 08 register (FLASH_OPTKEYR)
  uint32_t  OPTKEY;             // Flash option key; word access
// 0C Status register (FLASH_SR)
  uint32_t  EOP         :  1;   // End of operation
  uint32_t  OPERR       :  1;   // Operation error
  uint32_t              :  2;
  uint32_t  WRPERR      :  1;   // Write protection error
  uint32_t  PGAERR      :  1;   // Programming alignment error
  uint32_t  PGPERR      :  1;   // Programming parallelism error
  uint32_t  PGSERR      :  1;   // Programming sequence error
  uint32_t              :  8;
  uint32_t  BSY         :  1;   // Busy
  uint32_t              : 15;
// 10 Control register (FLASH_CR)
  uint32_t  PG          :  1;   // Programming
  uint32_t  SER         :  1;   // Sector erase
  uint32_t  MER         :  1;   // Mass erase
  uint32_t  SNB         :  4;   // Sector number (0..11)
  uint32_t              :  1;
  uint32_t  PSIZE       :  2;   // Program size (x8, x16, x32, x64)
  uint32_t              :  6;
  uint32_t  STRT        :  1;   // Start: triggers an erase operation
  uint32_t              :  7;
  uint32_t  EOPIE       :  1;   // End of operation interrupt enable
  uint32_t  ERRIE       :  1;   // Error interrupt enable
  uint32_t              :  5;
  uint32_t  LOCK        :  1;   // FLASH_CR is locked; Reset: 1
// 14 Control register (FLASH_OPTCR)
  uint32_t  OPTLOCK     :  1;   // FLASH_OPTCR is locked; Reset: 1
  uint32_t  OPTSTRT     :  1;   // Option start; Reset: 0
  uint32_t  BOR_LEV     :  2;   // Brown out reset level; Reset: 3
                                //   0: Level3 = 2.75 .. 2.97V
                                //   1: Level2 = 2.44 .. 2.63V
                                //   2: Level1 = 2.13 .. 2.33V
                                //   3: Level0 = 1.60 .. 1.80V = power on reset
  uint32_t              :  1;
  uint32_t  WDG_SW      :  1;   // Reset: 1
  uint32_t  nRST_STOP   :  1;   // Reset: 1
  uint32_t  nRST_STDBY  :  1;   // Reset: 1
  uint32_t  RDP         :  8;   // Read protext; Reset: 0xAA
  uint32_t  nWRP        : 12;   // Not write protect; Reset: 0xFFF
  uint32_t              :  4;
} flash_struct;

_Static_assert (sizeof(flash_struct) == 24, " flash_struct: bad size");

static flash_struct * const FLASH = (flash_struct *)0x40023C00;

