// iwdg.h
#pragma once
#include <stdint.h>

typedef enum iwdg_key_enum {
  IWDG_UNLOCK = 0x5555,
  IWDG_START  = 0xCCCC,
  IWDG_RELOAD = 0xAAAA
} iwdg_key_enum;

typedef enum iwdg_prescaler_enum {
//                  // period   max. timeout
//                  // @32kHz   @47 .. 17kHz
//
  IWDG_PR_DIV_4,    // 125 us   348 ..   964 ms
  IWDG_PR_DIV_8,    // 250 us   697 ..  1928 ms
  IWDG_PR_DIV_16,   // 500 us  1.39 ..  3.86 s
  IWDG_PR_DIV_32,   //  1 ms   2.78 ..  7.71 s
  IWDG_PR_DIV_64,   //  2 ms   5.57 .. 15.42 s
  IWDG_PR_DIV_128,  //  4 ms  11.15 .. 30.84 s
  IWDG_PR_DIV_256,  //  8 ms  22.30 .. 61.68 s
  IWDG_PR_DIV_256_TOO
} iwdg_prescaler_enum;

// Register updates take up to 5 LSI cycles (0.3ms @ 17kHz)

typedef volatile struct /* iwdg_struct */ {  // 16-, 32-Bit access
  // Key register
  uint32_t  KR;           // 16 Bit
  // Prescaler
  uint32_t  PR;           // 3 Bit
  // Reload register
  uint32_t  RLR;          // 12 Bit
  // Status register
  uint32_t  SR_PVU  :  1; // Watchdog prescaler value update
  uint32_t  SR_RVU  :  1; // Watchdog counter reload value update
  uint32_t          : 30;
} iwdg_struct;

static iwdg_struct * const IWDG = (iwdg_struct *)0x40003000;

