// mpu.h -- Memory protection unit MPU

// PM0056 Cortex-M3 programming manual

#pragma once
#include <stdint.h>

static const uint32_t  MPU_RBAR_VALID = 1 << 4;

static const uint32_t  MPU_RASR_ENABLE = 1 <<  0;
static const uint32_t  MPU_RASR_B      = 1 << 16;
static const uint32_t  MPU_RASR_C      = 1 << 17;
static const uint32_t  MPU_RASR_S      = 1 << 18;
static const uint32_t  MPU_RASR_TEX    = 0;
static const uint32_t  MPU_RASR_XN     = 1 << 28;

static const uint32_t  MPU_RASR_FLASH = 1 << 17;
static const uint32_t  MPU_RASR_SRAM  = 1 << 17 | 1 << 18;
static const uint32_t  MPU_RASR_PERI  = 1 << 16 | 1 << 18;

static const uint32_t  MPU_RO_RO    = ((6 << 24));
static const uint32_t  MPU_RO_RO_XN = ((6 << 24) | (1 << 28));
static const uint32_t  MPU_RO_NONE  = ((5 << 24));
static const uint32_t  MPU_RW_RW    = ((3 << 24) | (1 << 28));
static const uint32_t  MPU_RW_RO    = ((2 << 24) | (1 << 28));
static const uint32_t  MPU_RW_NONE  = ((1 << 24) | (1 << 28));
static const uint32_t  MPU_NONE     = ((0 << 24) | (1 << 28));


typedef volatile struct /* mpu_struct */ {  // ? access
// 00 Type register (MPU_TYPER)
  uint32_t  SEPARATE      :  1; // Separate flag (= 0)
  uint32_t                :  7;
  uint32_t  DREGION       :  8; // Number of MPU data regions (= 8)
  uint32_t  IREGION       :  8; // Number of MPU instruction regions (= 0)
  uint32_t                :  8;
// 04 Control
  uint32_t  CR_ENABLE     :  1; // Enables the MPU
  uint32_t  CR_HFNMIENA   :  1; // Enables the operation of MPU during
                                //   hard fault, NMI, and FAULTMASK handlers.
  uint32_t  CR_PRIVDEFENA :  1; // Enable priviliged software access
                                //   to default memory map.
  uint32_t                : 29;
// 08 Region number register (MPU_RNR)
  uint32_t  REGION        :  8; // MPU region
  uint32_t                : 24;
// 0C Region base address register (MPU_RBAR)
  uint32_t  RBAR;
    //      REGION      :  4; // MPU region field
    //      VALID       :  1; // MPU region number valid
    //      ADDR        : 27; // Region base address field
// 10 Region attribute and size register (MPU_RASR)
  uint32_t  RASR;
#if 0
  uint32_t  ENABLE      :  1; // Region enable bit.
  uint32_t  SIZE        :  5; // Size of the MPU protection region.
  uint32_t              :  2;
  uint32_t  SRD         :  8; // Subregion disable bits.
  uint32_t  B           :  1; // memory attribute
  uint32_t  C           :  1; // memory attribute
  uint32_t  S           :  1; // Shareable memory attribute
  uint32_t  TEX         :  3; // memory attribute
  uint32_t              :  2;
  uint32_t  AP          :  3; // Access permission
  uint32_t              :  1;
  uint32_t  XN          :  1; // Instruction access disable bit:
  uint32_t              :  3;
#endif
} mpu_struct;
_Static_assert (sizeof(mpu_struct) == 20, " mpu_struct: bad size");


// NVIC_ICTR  Interrupt Controller Type Register   0xE000E004;
// SCB_ACTLR  Auxiliary control register           0xE000E008;
// NVIC_STIR  Software trigger interrupt register  0xE000EF00;

//     itm_struct *  const ITM    =  (itm_struct *)0xE0000000;
//     dwt_struct *  const DWT    =  (dwt_struct *)0xE0001000;
//     fpb_struct *  const FPB    =  (fpb_struct *)0xE0002000;
//     stk_struct *  const STK    =  (stk_struct *)0xE000E010;
//     nvic_struct * const NVIC   = (nvic_struct *)0xE000E100;
//     scb_struct *  const SCB    =  (scb_struct *)0xE000ED00;
//     cpac_struct * const CPAC   = (cpac_struct *)0xE000ED88;
static mpu_struct *  const MPU    =  (mpu_struct *)0xE000ED90;
//     dcb_struct *  const DCB    =  (dcb_struct *)0xE000EDF0;
//     fpu_struct *  const FPU    =  (fpu_struct *)0xE000EF30;
//     dbg_struct *  const DBGMCU =  (dbg_struct *)0xE0042000;

