// nvic.h -- Nested vectored interrupt controller NVIC

// PM0056 Cortex-M3 programming manual

#pragma once
#include <stdint.h>

typedef volatile struct /* nvic_struct */ {  // word access, IP[x] byte access
// 000 Interrupt set enable
  uint32_t  ISER[3];
  uint32_t  rsvd_000[29];
// 080 Interrupt clear enable
  uint32_t  ICER[3];
  uint32_t  rsvd_080[29];
// 100 Interrupt set pending
  uint32_t  ISPR[3];
  uint32_t  rsvd_100[29];
// 180 Interrupt clear pending
  uint32_t  ICPR[3];
  uint32_t  rsvd_180[29];
// 200 Interrupt active
  uint32_t  IABR[3];
  uint32_t  rsvd_200[29];
// 300 Interrupt priority
  uint8_t   IP[81];
  uint8_t   rsvd_300[3];
} nvic_struct;
_Static_assert (sizeof(nvic_struct) == 724, " nvic_struct: bad size");


// NVIC_ICTR  Interrupt Controller Type Register   0xE000E004;
// SCB_ACTLR  Auxiliary control register           0xE000E008;
// NVIC_STIR  Software trigger interrupt register  0xE000EF00;

//     itm_struct *  const ITM    =  (itm_struct *)0xE0000000;
//     dwt_struct *  const DWT    =  (dwt_struct *)0xE0001000;
//     fpb_struct *  const FPB    =  (fpb_struct *)0xE0002000;
//     stk_struct *  const STK    =  (stk_struct *)0xE000E010;
static nvic_struct * const NVIC   = (nvic_struct *)0xE000E100;
//     scb_struct *  const SCB    =  (scb_struct *)0xE000ED00;
//     cpac_struct * const CPAC   = (cpac_struct *)0xE000ED88;
//     mpu_struct *  const MPU    =  (mpu_struct *)0xE000ED90;
//     dcb_struct *  const DCB    =  (dcb_struct *)0xE000EDF0;
//     fpu_struct *  const FPU    =  (fpu_struct *)0xE000EF30;
//     dbg_struct *  const DBGMCU =  (dbg_struct *)0xE0042000;

