// pwr.h
#pragma once
#include <stdint.h>

typedef enum pwr_pls_enum {
  PWR_PLS_2000mV,
  PWR_PLS_2100mV,
  PWR_PLS_2300mV,
  PWR_PLS_2500mV,
  PWR_PLS_2600mV,
  PWR_PLS_2700mV,
  PWR_PLS_2800mV,
  PWR_PLS_2900mV
} pwr_pls_enum;

typedef volatile struct /* pwr_struct */ {  // half/word access
  // 00 Power control register (PWR_CR)
  uint32_t  CR_LPDS     :  1; // Low-power deep sleep
  uint32_t  CR_PDDS     :  1; // Power down deepsleep
  uint32_t  CR_CWUF     :  1; // Clear wakeup flag
  uint32_t  CR_CSBF     :  1; // Clear standby flag
  uint32_t  CR_PVDE     :  1; // Power voltage detector enable
  uint32_t  CR_PLS      :  3; // PVD level selection
  uint32_t  CR_DBP      :  1; // Disable backup domain write protection
  uint32_t  CR_FPDS     :  1; // Flash power down in Stop mode
  uint32_t              : 22;
  // 04 Power control/status register (PWR_CSR)
  uint32_t  CSR_WUF     :  1; // Wakeup flag
  uint32_t  CSR_SBF     :  1; // Standby flag
  uint32_t  CSR_PVDO    :  1; // PVD output
  uint32_t  CSR_BRR     :  1; // Backup regulator ready
  uint32_t              :  4;
  uint32_t  CSR_EWUP    :  1; // Enable WKUP pin
  uint32_t  CSR_BRE     :  1; // Backup regulator enable
  uint32_t              : 22;
} pwr_struct;

_Static_assert (sizeof(pwr_struct) == 8, " pwr_struct: bad size");

static pwr_struct * const PWR = (pwr_struct *)0x40007000;

