// scb.h -- System Control Block SCB

// PM0056 Cortex-M3 programming manual

#pragma once
#include <stdint.h>

// static const uint32_t  AIRCR_VECTRESET_MAGIC     = 0x05FA0001;
// static const uint32_t  AIRCR_VECTCLRACTIVE_MAGIC = 0x05FA0002;
   static const uint32_t  AIRCR_SYSRESETREQ_MAGIC   = 0x05FA0004;
   static const uint32_t  AIRCR_PRIGROUP_16_0_MAGIC = 0x05FA0300;
   static const uint32_t  AIRCR_PRIGROUP_8_2_MAGIC  = 0x05FA0400;
   static const uint32_t  AIRCR_PRIGROUP_4_4_MAGIC  = 0x05FA0500;
   static const uint32_t  AIRCR_PRIGROUP_2_8_MAGIC  = 0x05FA0600;
   static const uint32_t  AIRCR_PRIGROUP_0_16_MAGIC = 0x05FA0700;
// static const uint32_t  AIRCR_ENDIANESS_MAGIC     = 0x05FA8000;


typedef volatile struct /* scb_struct */ {  // word access, SHPRx byte access
// 00 CPUID base register (CPUID)
  uint32_t  CPUID_Revision       :  4; // Revision number
  uint32_t  CPUID_PartNo         : 12; // Part number of the processor (=0xC23)
  uint32_t  CPUID_Constant       :  4; // Reads as 0xF
  uint32_t  CPUID_Variant        :  4; // Variant number
  uint32_t  CPUID_Implementer    :  8; // Implementer code (=0x41)
// 04 Interrupt control and state register (ICSR)
  uint32_t  ICSR_VECTACTIVE      :  9; // Active vector
  uint32_t                       :  2;
  uint32_t  ICSR_RETOBASE        :  1; // Return to base level
  uint32_t  ICSR_VECTPENDING     : 10; // Pending vector
  uint32_t  ICSR_ISRPENDING      :  1; // Interrupt pending flag,
                                       //   excl. NMI & Faults
  uint32_t                       :  1; // This bit is reserved for Debug use
                                       //   and reads-as-zero when not in Debug
  uint32_t                       :  1;
  uint32_t  ICSR_PENDSTCLR       :  1; // SysTick exception clear-pending bit
  uint32_t  ICSR_PENDSTSET       :  1; // SysTick exception set-pending bit
  uint32_t  ICSR_PENDSVCLR       :  1; // PendSV clear-pending bit
  uint32_t  ICSR_PENDSVSET       :  1; // PendSV set-pending bit
  uint32_t                       :  2;
  uint32_t  ICSR_NMIPENDSET      :  1; // NMI set-pending bit
// 08 Vector table offset register (VTOR)
  uint32_t  VTOR;
// 0C Application interrupt and reset control register (AIRCR)
  uint32_t  AIRCR;
#if 0
  uint32_t  VECTRESET            :  1; // Reserved for Debug use
  uint32_t  VECTCLRACTIVE        :  1; // Reserved for Debug use
  uint32_t  SYSRESETREQ          :  1; // System reset request
  uint32_t                       :  5;
  uint32_t  PRIGROUP             :  3; // Interrupt priority grouping field
  uint32_t                       :  4;
  uint32_t  ENDIANESS            :  1; // Data endianness bit (= 0)
  uint32_t  VECTKEY              : 16; // Register key (write 0x5FA for access)
#endif
// 10 System control register (SCR)
  uint32_t                       :  1;
  uint32_t  SCR_SLEEPONEXIT      :  1; // Enter (deep) sleep on ISR-return
  uint32_t  SCR_SLEEPDEEP        :  1; // Deep sleep
  uint32_t                       :  1;
  uint32_t  SCR_SEVEONPEND       :  1; // Send Event on Pending bit
  uint32_t                       : 27;
// 14 Configuration and control register (CCR)
  uint32_t  CCR_NONBASETHRDENA   :  1; // Processor can enter Thread mode from
                                       //   any level under the control of
                                       //   an EXC_RETURN
  uint32_t  CCR_USERSETMPEND     :  1; // unprivileged access to the STIR
  uint32_t                       :  1;
  uint32_t  CCR_UNALIGN_TRP      :  1; // Trap unaligned halfword and
                                       //   word accesses
  uint32_t  CCR_DIV_0_TRP        :  1; // Trap divide by 0
  uint32_t                       :  3;
  uint32_t  CCR_BFHFNMIGN        :  1; // Handlers running at priority -1 and -2
                                       //   ignore data bus faults caused by
                                       //   load and store instructions.
  uint32_t  CCR_STKALIGN         :  1; // 8-byte stack alignment
                                       //   on exception entry
  uint32_t                       : 22;
// 18 System handler priority registers (SHPR1)
// 1C System handler priority registers (SHPR2)
// 20 System handler priority registers (SHPR3)
  uint8_t   SHP__0_MM            :  8; // system handler 4, memory mgt fault
  uint8_t   SHP__1_BUS           :  8; // system handler 5, bus fault
  uint8_t   SHP__2_USAGE         :  8; // system handler 6, usage fault
  uint8_t                        :  8;
  uint8_t                        :  8;
  uint8_t                        :  8;
  uint8_t                        :  8;
  uint8_t   SHP__7_SVC           :  8; // system handler 11, SVCall
  uint8_t   SHP__8_DEBUG         :  8; // (ARM confidential)
  uint8_t                        :  8;
  uint8_t   SHP_10_PENDSV        :  8; // system handler 14, PendSV
  uint8_t   SHP_11_SYSTICK       :  8; // system handler 15, SysTick
// 24 System handler control and state register (SHCSR)
  uint32_t  SHCSR_MEMFAULTACT    :  1; // Memory management fault
                                       //   exception active
  uint32_t  SHCSR_BUSFAULTACT    :  1; // Bus fault exception active
  uint32_t                       :  1;
  uint32_t  SHCSR_USGFAULTACT    :  1; // Usage fault exception active
  uint32_t                       :  3;
  uint32_t  SHCSR_SVCALLACT      :  1; // SVC call active
  uint32_t  SHCSR_MONITORACT     :  1; // Debug monitor active
  uint32_t                       :  1;
  uint32_t  SHCSR_PENDSVACT      :  1; // PendSV exception active
  uint32_t  SHCSR_SYSTICKACT     :  1; // SysTick exception active
  uint32_t  SHCSR_USGFAULTPENDED :  1; // Usage fault exception pending
  uint32_t  SHCSR_MEMFAULTPENDED :  1; // Memory management fault
                                       //   exception pending
  uint32_t  SHCSR_BUSFAULTPENDED :  1; // Bus fault exception pending
  uint32_t  SHCSR_SVCALLPENDED   :  1; // SVC call pending
  uint32_t  SHCSR_MEMFAULTENA    :  1; // Memory management fault enable
  uint32_t  SHCSR_BUSFAULTENA    :  1; // Bus fault enable
  uint32_t  SHCSR_USGFAULTENA    :  1; // Usage fault enable
  uint32_t                       : 13;
// 28 Configurable fault status register (CFSR)
  uint32_t  CFSR;
#if 0
  uint32_t  IACCVIOL       :  1; // Instruction access violation flag
  uint32_t  DACCVIOL       :  1; // Data access violation flag
  uint32_t                 :  1;
  uint32_t  MUNSTKERR      :  1; // Memory manager fault on unstacking
                                 //   for a return from exception
  uint32_t  MSTKERR        :  1; // Memory manager fault on stacking
                                 //   for exception entry
  uint32_t                 :  2;
  uint32_t  MMARVALID      :  1; // Memory Management Fault Address Register
                                 //   (MMAR) valid flag
  uint32_t  IBUSERR        :  1; // Instruction bus error
  uint32_t  PRECISERR      :  1; // Precise data bus error
  uint32_t  IMPRECISERR    :  1; // Imprecise data bus error
  uint32_t  UNSTKERR       :  1; // Bus fault on unstacking for
                                 //   a return from exception
  uint32_t  STKERR         :  1; // Bus fault on stacking for exception entry
  uint32_t                 :  2;
  uint32_t  BFARVALID      :  1; // Bus Fault Address Register BFAR valid flag
  uint32_t  UNDEFINSTR     :  1; // Undefined instruction usage fault
  uint32_t  INVSTATE       :  1; // Invalid state usage fault
  uint32_t  INVPC          :  1; // Invalid PC load usage fault (by EXC_RETURN)
  uint32_t  NOCP           :  1; // No coprocessor usage fault
  uint32_t                 :  4;
  uint32_t  UNALIGNED      :  1; // Unaligned access usage fault
  uint32_t  DIVBYZERO      :  1; // Divide by zero usage fault
  uint32_t                 :  6;
#endif
// 2C Hard fault status register (HFSR)
  uint32_t  HFSR;
#if 0
  uint32_t                 :  1;
  uint32_t  VECTTBL        :  1; // Vector table hard fault
  uint32_t                 : 28;
  uint32_t  FORCED         :  1; // Forced hard fault
  uint32_t  DEBUG_VT       :  1; // Reserved for Debug use
#endif
// 30 Debug fault status (DFSR), undocumented
  uint32_t  DFSR;
// 34 Memory management fault address register (MMFAR)
  uint32_t  MMFAR;
// 38 Bus fault address register (BFAR)
  uint32_t  BFAR;
// 3C Auxiliary fault status register (AFSR) -- not implemented
// uint32_t  AFSR;
} scb_struct;
_Static_assert (sizeof(scb_struct) == 60, " scb_struct: bad size");


// NVIC_ICTR  Interrupt Controller Type Register   0xE000E004;
// SCB_ACTLR  Auxiliary control register           0xE000E008;
// NVIC_STIR  Software trigger interrupt register  0xE000EF00;

//     itm_struct *  const ITM    =  (itm_struct *)0xE0000000;
//     dwt_struct *  const DWT    =  (dwt_struct *)0xE0001000;
//     fpb_struct *  const FPB    =  (fpb_struct *)0xE0002000;
//     stk_struct *  const STK    =  (stk_struct *)0xE000E010;
//     nvic_struct * const NVIC   = (nvic_struct *)0xE000E100;
static scb_struct *  const SCB    =  (scb_struct *)0xE000ED00;
//     cpac_struct * const CPAC   = (cpac_struct *)0xE000ED88;
//     mpu_struct *  const MPU    =  (mpu_struct *)0xE000ED90;
//     dcb_struct *  const DCB    =  (dcb_struct *)0xE000EDF0;
//     fpu_struct *  const FPU    =  (fpu_struct *)0xE000EF30;
//     dbg_struct *  const DBGMCU =  (dbg_struct *)0xE0042000;

