// spi.h
#pragma once
#include <stdint.h>

typedef volatile struct /* spi_struct */ {  // 16-, 32-Bit access
// 00 SPI control register 1 (SPI_CR1)
  uint32_t  CR1_CPHA     :  1; // Clock phase
  uint32_t  CR1_CPOL     :  1; // Clock polarity
  uint32_t  CR1_MSTR     :  1; // Master selection
  uint32_t  CR1_BR       :  3; // Baud rate control
  uint32_t  CR1_SPE      :  1; // SPI enable
  uint32_t  CR1_LSBFIRST :  1; // Frame format
  uint32_t  CR1_CR1_SSI  :  1; // Internal slave select
  uint32_t  CR1_SSM      :  1; // Software slave management
  uint32_t  CR1_RXONLY   :  1; // Receive only
  uint32_t  CR1_DFF      :  1; // Data frame format; 0: 8-Bit, 1: 16-Bit
  uint32_t  CR1_CRCNEXT  :  1; // CRC transfer next
  uint32_t  CR1_CRCEN    :  1; // Hardware CRC calculation enable
  uint32_t  CR1_BIDIOE   :  1; // Output enable in bidirectional mode
  uint32_t  CR1_BIDIMODE :  1; // Bidirectional data mode enable
  uint32_t               : 16;
// 04 SPI control register 2 (SPI_CR2)
  uint32_t  CR2_RXDMAEN  :  1; // Rx buffer DMA enable
  uint32_t  CR2_TXDMAEN  :  1; // Tx buffer DMA enable
  uint32_t  CR2_SSOE     :  1; // SS output enable
  uint32_t               :  1;
  uint32_t  CR2_FRF      :  1; // Frame format; 0: Motorola mode, 1: TI mode
  uint32_t  CR2_ERRIE    :  1; // Error interrupt enable
  uint32_t  CR2_RXNEIE   :  1; // RX buffer not empty interrupt enable
  uint32_t  CR2_TXEIE    :  1; // Tx buffer empty interrupt enable
  uint32_t               : 24;
// 08 SPI status register (SPI_SR)
  uint32_t  SR_RXNE      :  1; // Receive buffer not empty
  uint32_t  SR_TXE       :  1; // Transmit buffer empty
  uint32_t  SR_CHSIDE    :  1; // Channel side
  uint32_t  SR_UDR       :  1; // Underrun flag
  uint32_t  SR_CRCERR    :  1; // CRC error flag
  uint32_t  SR_MODF      :  1; // Mode fault
  uint32_t  SR_OVR       :  1; // Overrun flag
  uint32_t  SR_BSY       :  1; // Busy flag
  uint32_t  SR_TIFRFE    :  1; // TI frame format error
  uint32_t               : 23;
// 0C SPI data register (SPI_DR)
  uint16_t  DR;
  uint16_t  rsvd_0E;
// 10 SPI CRC polynomial register (SPI_CRCPR)
  uint16_t  CRCPR;
  uint16_t  rsvd_12;
// 14 SPI RX CRC register (SPI_RXCRCR)
  uint16_t  RXCRCR;
  uint16_t  rsvd_16;
// 18 SPI TX CRC register (SPI_TXCRCR)
  uint16_t  TXCRCR;
  uint16_t  rsvd_1A;
// 1C SPI_I2S configuration register (SPI_I2SCFGR)
  uint32_t  I2SCFGR_CHLEN   :  1; // Channel length (bits per audio channel)
  uint32_t  I2SCFGR_DATLEN  :  2; // Data length to be transferred
  uint32_t  I2SCFGR_CKPOL   :  1; // Steady state clock polarity
  uint32_t  I2SCFGR_I2SSTD  :  2; // I2S standard selection
  uint32_t                  :  1;
  uint32_t  I2SCFGR_PCMSYNC :  1; // PCM frame synchronization
  uint32_t  I2SCFGR_I2SCFG  :  2; // I2S configuration mode
  uint32_t  I2SCFGR_I2SE    :  1; // I2S Enable
  uint32_t  I2SCFGR_I2SMOD  :  1: // I2S mode selection
  uint32_t                  : 20;
// 20 SPI_I2S prescaler register (SPI_I2SPR)
  uint32_t  I2SDIV          :  8; // I2S Linear prescaler
  uint32_t  ODD             :  1; // Odd factor for the prescaler
  uint32_t  MCKOE           :  1; // Master clock output enable
  uint32_t                  : 22;
} spi_struct;

_Static_assert (sizeof(spi_struct) == 36, " spi_struct: bad size");

static spi_struct * const SPI1 = (spi_struct *)0x40013000;
static spi_struct * const SPI2 = (spi_struct *)0x40003800;
static spi_struct * const SPI3 = (spi_struct *)0x40003C00;

