// stk.h -- SysTick timer STK

// PM0056 Cortex-M3 programming manual

#pragma once
#include <stdint.h>

typedef volatile struct /* stk_struct */ {  // word access
// 00 SysTick control and status register (STK_CTRL)
  uint32_t  CTRL_ENABLE      :  1; // Counter enable
  uint32_t  CTRL_TICKINT     :  1; // SysTick exception request enable
  uint32_t  CTRL_CLKSOURCE   :  1; // Clock source
                                   //   0: AHB/8, 1: Processor clock AHB
  uint32_t                   : 13;
  uint32_t  CTRL_COUNTFLAG   :  1; // Returns 1 if timer counted to 0 since
                                   //   last time this was read.
  uint32_t                   : 15;
// 04 SysTick reload value register (STK_LOAD)
  uint32_t  RELOAD;
// 08 SysTick current value register (STK_VAL)
  uint32_t  CURRENT;
// 0C SysTick calibration value register (STK_CALIB)
  uint32_t  CALIB_TENMS      : 24; // Calibration value (see Reference Manual)
  uint32_t                   :  6;
  uint32_t  CALIB_SKEW       :  1; // SKEW flag (= 1; TENMS is not known)
  uint32_t  CALIB_NOREF      :  1; // NOREF flag (= 0; HCLK/8 is provided)
} stk_struct;
_Static_assert (sizeof(stk_struct) == 16, " stk_struct: bad size");


// NVIC_ICTR  Interrupt Controller Type Register   0xE000E004;
// SCB_ACTLR  Auxiliary control register           0xE000E008;
// NVIC_STIR  Software trigger interrupt register  0xE000EF00;

//     itm_struct *  const ITM    =  (itm_struct *)0xE0000000;
//     dwt_struct *  const DWT    =  (dwt_struct *)0xE0001000;
//     fpb_struct *  const FPB    =  (fpb_struct *)0xE0002000;
static stk_struct *  const STK    =  (stk_struct *)0xE000E010;
//     nvic_struct * const NVIC   = (nvic_struct *)0xE000E100;
//     scb_struct *  const SCB    =  (scb_struct *)0xE000ED00;
//     cpac_struct * const CPAC   = (cpac_struct *)0xE000ED88;
//     mpu_struct *  const MPU    =  (mpu_struct *)0xE000ED90;
//     dcb_struct *  const DCB    =  (dcb_struct *)0xE000EDF0;
//     fpu_struct *  const FPU    =  (fpu_struct *)0xE000EF30;
//     dbg_struct *  const DBGMCU =  (dbg_struct *)0xE0042000;

