// syscfg.h
#pragma once
#include <stdint.h>

typedef enum mem_mode_enum {
  MAIN_FLASH,
  SYSTEM_FLASH,
  FSMC,
  SRAM
} moe_mode_enum;

typedef enum exti_port_enum {
  PORTA,
  PORTB,
  PORTC,
  PORTD,
  PORTE,
  PORTF,
  PORTG,
  PORTH,
  PORTI
} exti_port_enum;

typedef volatile struct /* syscfg_struct */ {  // word access
// 00 Memory remap register (SYSCFG_MEMRMP)
  uint32_t  MEMRMP_MEM_MODE     :  2; // Memory mapping selection
  uint32_t                      : 30;
// 04 Peripheral mode configuration register (SYSCFG_PMC)
  uint32_t                      : 23;
  uint32_t  MEMRMP_MII_RMII_SEL :  1; // Ethernet PHY interface selection
  uint32_t                      :  8;
// 08 External interrupt configuration register 1 (SYSCFG_EXTICR1)
  uint32_t  EXTICR1_EXTI0       :  4;
  uint32_t  EXTICR1_EXTI1       :  4;
  uint32_t  EXTICR1_EXTI2       :  4;
  uint32_t  EXTICR1_EXTI3       :  4;
  uint32_t                      : 16;
// 0C External interrupt configuration register 2 (SYSCFG_EXTICR2)
  uint32_t  EXTICR2_EXTI4       :  4;
  uint32_t  EXTICR2_EXTI5       :  4;
  uint32_t  EXTICR2_EXTI6       :  4;
  uint32_t  EXTICR2_EXTI7       :  4;
  uint32_t                      : 16;
// 10 External interrupt configuration register 3 (SYSCFG_EXTICR3)
  uint32_t  EXTICR3_EXTI8       :  4;
  uint32_t  EXTICR3_EXTI9       :  4;
  uint32_t  EXTICR3_EXTI10      :  4;
  uint32_t  EXTICR3_EXTI11      :  4;
  uint32_t                      : 16;
// 14 External interrupt configuration register 4 (SYSCFG_EXTICR4)
  uint32_t  EXTICR4_EXTI12      :  4;
  uint32_t  EXTICR4_EXTI13      :  4;
  uint32_t  EXTICR4_EXTI14      :  4;
  uint32_t  EXTICR4_EXTI15      :  4;
  uint32_t                      : 16;
// 18
  uint32_t  rsvd_18;
// 1C
  uint32_t  rsvd_1c;
// 20 Compensation cell control register (SYSCFG_CMPCR)
  uint32_t  CMP_PD              :  1; // Compensation cell
                                      //   0: power-down, 1: enabled
  uint32_t                      :  7;
  uint32_t  READY               :  1; // Compensation cell ready flag
  uint32_t                      : 23;
} syscfg_struct;

_Static_assert (sizeof(syscfg_struct) == 36, " syscfg_struct: bad size");

static syscfg_struct * const SYSCFG = (syscfg_struct *)0x40013800;

