// system-memory.h

// PM0059 STM32F205/215, STM32F207/217 Flash programming manual

#pragma once
#include <stdint.h>

typedef volatile struct /* sysmem_struct */ {  // access ???
// 7800
  uint8_t   OTP_FLASH[512];
// 7A00
  uint8_t   OTP_LOCK[16];       // 7a00
// 7A10 Unique Device ID
  uint16_t  U_ID_Y;
  uint16_t  U_ID_X;
  uint8_t   U_ID_WAFER;
  uint8_t   U_ID_LOT[7];
  uint32_t  rsvd_a;
// 7A20 Flash size
  uint32_t                    : 16;
  uint32_t  F_SIZE            : 16;
  uint32_t  rsvd_c[64-9];
// 7B00
  uint32_t  rsvd_d[64];
// 7C00 ROP & user option Bytes (ROP & USER)
  uint32_t                    :  2;
  uint32_t  OPTION_BOR_LEV    :  2; // BOR reset Level 3: off, 0: 3.6V?!
  uint32_t                    :  1;
  uint32_t  OPTION_WDG_SW     :  1; // 0: Hardware watchdog
                                    // 1: Software watchdog
  uint32_t  OPTION_nRST_STOP  :  1; // Reset generated when entering Stop mode
  uint32_t  OPTION_nRST_STDBY :  1; // Reset generated when entering Standby
  uint32_t  OPTION_ROP        :  8; // Read protection option byte
  uint32_t                    : 16;
  uint32_t                    : 32;
// 7C08 Write protections (nWRP)
  uint32_t  OPTION_nWRP       : 12; // 0: Write protection active on sector i
  uint32_t                    : 20;
  uint32_t                    : 32;
} sysmem_struct;

_Static_assert (sizeof(sysmem_struct) == 1024+16, " sys_mem_struct: bad size");

static sysmem_struct * const SYSMEM = (sysmem_struct *)0x1fff7800;

