// tim.h
#pragma once
#include <stdint.h>

// General-purpose timers (TIM2 to TIM5)

typedef volatile struct /* tim2345_struct */ {  // see RM0033 for access modes
// 00 Control register 1 (TIMx_CR1)
  uint32_t  CR1_CEN     :  1; // Counter enable
  uint32_t  CR1_UDIS    :  1; // Update disable
  uint32_t  CR1_URS     :  1; // Update request source
  uint32_t  CR1_OPM     :  1; // One-pulse mode
  uint32_t  CR1_DIR     :  1; // Direction
  uint32_t  CR1_CMS     :  2; // Center-aligned mode selection
  uint32_t  CR1_ARPE    :  1; // Auto-reload preload enable
  uint32_t  CR1_CKD     :  2; // Clock division
  uint32_t              :  6;
  uint32_t              : 16;
// 04 Control register 2 (TIMx_CR2)
  uint32_t              :  3;
  uint32_t  CR2_CCDS    :  1; // Capture/compare DMA selection
  uint32_t  CR2_MMS     :  3; // Master mode selection
  uint32_t  CR2_TI1S    :  1; // TI1 selection
  uint32_t              : 24;
// 08 Slave mode control register (TIMx_SMCR)
  uint32_t  SMCR_SMS    :  3; // Slave mode selection
  uint32_t              :  1;
  uint32_t  SMCR_TS     :  3; // Trigger selection
  uint32_t  SMCR_MSM    :  1; // Master/Slave mode
  uint32_t  SMCR_ETF    :  4; // External trigger filter
  uint32_t  SMCR_ETPS   :  2; // External trigger prescaler
  uint32_t  SMCR_ECE    :  1; // External clock enable
  uint32_t  SMCR_ETP    :  1; // External trigger polarity
  uint32_t              : 16;
// 0C DMA/Interrupt enable register (TIMx_DIER)
  uint32_t  DIER_UIE    :  1; // Update interrupt enable
  uint32_t  DIER_CC1IE  :  1; // Capture/Compare 1 interrupt enable
  uint32_t  DIER_CC2IE  :  1; // Capture/Compare 2 interrupt enable
  uint32_t  DIER_CC3IE  :  1; // Capture/Compare 3 interrupt enable
  uint32_t  DIER_CC4IE  :  1; // Capture/Compare 4 interrupt enable
  uint32_t              :  1;
  uint32_t  DIER_TIE    :  1; // Trigger interrupt enable
  uint32_t              :  1;
  uint32_t  DIER_UDE    :  1; // Update DMA request enable
  uint32_t  DIER_CC1DE  :  1; // Capture/Compare 1 DMA request enable
  uint32_t  DIER_CC2DE  :  1; // Capture/Compare 2 DMA request enable
  uint32_t  DIER_CC3DE  :  1; // Capture/Compare 3 DMA request enable
  uint32_t  DIER_CC4DE  :  1; // Capture/Compare 4 DMA request enable
  uint32_t              :  1;
  uint32_t  DIER_TDE    :  1; // Trigger DMA request enable
  uint32_t              :  1;
  uint32_t              : 16;
// 10 Status register (TIMx_SR)
  uint32_t  SR_UIF      :  1; // Update interrupt flag
  uint32_t  SR_CC1IF    :  1; // Capture/compare 1 interrupt flag
  uint32_t  SR_CC2IF    :  1; // Capture/Compare 2 interrupt flag
  uint32_t  SR_CC3IF    :  1; // Capture/Compare 3 interrupt flag
  uint32_t  SR_CC4IF    :  1; // Capture/Compare 4 interrupt flag
  uint32_t              :  1;
  uint32_t  SR_TIF      :  1; // Trigger interrupt flag
  uint32_t              :  2;
  uint32_t  SR_CC1OF    :  1; // Capture/Compare 1 overcapture flag
  uint32_t  SR_CC2OF    :  1; // Capture/compare 2 overcapture flag
  uint32_t  SR_CC3OF    :  1; // Capture/Compare 3 overcapture flag
  uint32_t  SR_CC4OF    :  1; // Capture/Compare 4 overcapture flag
  uint32_t              :  3;
  uint32_t              : 16;
// 14 Event generation register (TIMx_EGR)
  uint32_t  EGR_UG      :  1; // Update generation
  uint32_t  EGR_CC1G    :  1; // Capture/compare 1 generation
  uint32_t  EGR_CC2G    :  1; // Capture/compare 2 generation
  uint32_t  EGR_CC3G    :  1; // Capture/compare 3 generation
  uint32_t  EGR_CC4G    :  1; // Capture/compare 4 generation
  uint32_t              :  1;
  uint32_t  EGR_TG      :  1; // Trigger generation
  uint32_t              :  9;
  uint32_t              : 16;
// 18 Capture/compare mode register 1 (TIMx_CCMR1)
  union {
    uint32_t    CCMR1;
    struct {
      uint32_t  CC1S    :  2; // Capture/Compare 1 selection
      uint32_t  IC1PSC  :  2; // Input capture 1 prescaler
      uint32_t  IC1F    :  4; // Input capture 1 filter
      uint32_t  CC2S    :  2; // Capture/compare 2 selection
      uint32_t  IC2PSC  :  2; // Input capture 2 prescaler
      uint32_t  IC2F    :  4; // Input capture 2 filter
      uint32_t          : 16;
    } CCMR1_IC;
    struct {
      uint32_t  CC1S    :  2; // Capture/Compare 1 selection
      uint32_t  OC1FE   :  1; // Output compare 1 fast enable
      uint32_t  OC1PE   :  1; // Output compare 1 preload enable
      uint32_t  OC1M    :  3; // Output compare 1 mode
      uint32_t  OC1CE   :  1; // Output compare 1 clear enable
      uint32_t  CC2S    :  2; // Capture/Compare 2 selection
      uint32_t  OC2FE   :  1; // Output compare 2 fast enable
      uint32_t  OC2PE   :  1; // Output compare 2 preload enable
      uint32_t  OC2M    :  3; // Output compare 2 mode
      uint32_t  OC2CE   :  1; // Output compare 2 clear enable
      uint32_t          : 16;
    } CCMR1_OC;
  };
// 1C Capture/compare mode register 2 (TIMx_CCMR2)
  union {
    uint32_t    CCMR2;
    struct {
      uint32_t  CC3S    :  2; // Capture/Compare 3 selection
      uint32_t  IC3PSC  :  2; // Input capture 3 prescaler
      uint32_t  IC3F    :  4; // Input capture 3 filter
      uint32_t  CC4S    :  2; // Capture/compare 4 selection
      uint32_t  IC4PSC  :  2; // Input capture 4 prescaler
      uint32_t  IC4F    :  4; // Input capture 4 filter
      uint32_t          : 16;
    } CCMR2_IC;
    struct {
      uint32_t  CC3S    :  2; // Capture/Compare 3 selection
      uint32_t  OC3FE   :  1; // Output compare 3 fast enable
      uint32_t  OC3PE   :  1; // Output compare 3 preload enable
      uint32_t  OC3M    :  3; // Output compare 3 mode
      uint32_t  OC3CE   :  1; // Output compare 3 clear enable
      uint32_t  CC4S    :  2; // Capture/Compare 4 selection
      uint32_t  OC4FE   :  1; // Output compare 4 fast enable
      uint32_t  OC4PE   :  1; // Output compare 4 preload enable
      uint32_t  OC4M    :  3; // Output compare 4 mode
      uint32_t  OC4CE   :  1; // Output compare 4 clear enable
      uint32_t          : 16;
    } CCMR2_OC;
  };
// 20 Capture/compare enable register (TIMx_CCER)
  uint32_t  CCER_CC1E   :  1; // Capture/Compare 1 output enable
  uint32_t  CCER_CC1P   :  1; // Capture/Compare 1 output polarity
  uint32_t              :  1;
  uint32_t  CCER_CC1NP  :  1; // Capture/Compare 1 output polarity
  uint32_t  CCER_CC2E   :  1; // Capture/Compare 2 output enable
  uint32_t  CCER_CC2P   :  1; // Capture/Compare 2 output polarity
  uint32_t              :  1;
  uint32_t  CCER_CC2NP  :  1; // Capture/Compare 2 output polarity
  uint32_t  CCER_CC3E   :  1; // Capture/Compare 3 output enable
  uint32_t  CCER_CC3P   :  1; // Capture/Compare 3 output polarity
  uint32_t              :  1;
  uint32_t  CCER_CC3NP  :  1; // Capture/Compare 3 output polarity
  uint32_t  CCER_CC4E   :  1; // Capture/Compare 4 output enable
  uint32_t  CCER_CC4P   :  1; // Capture/Compare 4 output polarity
  uint32_t              :  1;
  uint32_t  CCER_CC4NP  :  1; // Capture/Compare 4 output polarity
  uint32_t              : 16;
// 24 Counter (TIMx_CNT)
  uint32_t  CNT;
// 28 Prescaler (TIMx_PSC)
  uint32_t  PSC;
// 2C Auto-reload register (TIMx_ARR)
  uint32_t  ARR;
// 30
  uint32_t  rsvd_30;
// 34 Capture/compare register 1 (TIMx_CCR1)
  uint32_t  CCR1;
// 38 Capture/compare register 2 (TIMx_CCR2)
  uint32_t  CCR2;
// 3C Capture/compare register 3 (TIMx_CCR3)
  uint32_t  CCR3;
// 40 Capture/compare register 4 (TIMx_CCR4)
  uint32_t  CCR4;
// 44
  uint32_t  rsvd_44;
// 48 DMA control register (TIMx_DCR)
  uint32_t  DCR_DBA     :  5; // DMA base address
  uint32_t              :  3;
  uint32_t  DCR_DBL     :  5; // DMA burst length
  uint32_t              :  3;
  uint32_t              : 16;
// 4C DMA address for full transfer (TIMx_DMAR)
  uint32_t  DMAR;
// 50 Option register (TIM2_OR, TIM5_OR)
  uint32_t              :  6;
  uint32_t  OR_TI4_RMP  :  2; // Timer Input 4 remap (TIM5)
  uint32_t              :  2;
  uint32_t  OR_ITR1_RMP :  2; // Internal trigger 1 remap (TIM2)
  uint32_t              :  4;
  uint32_t              : 16;
} tim2345_struct;

_Static_assert (sizeof(tim2345_struct) == 84, " tim2345_struct: bad size");

static tim2345_struct * const TIM2 = (tim2345_struct *)0x40000000;
static tim2345_struct * const TIM3 = (tim2345_struct *)0x40000400;
static tim2345_struct * const TIM4 = (tim2345_struct *)0x40000800;
static tim2345_struct * const TIM5 = (tim2345_struct *)0x40000C00;

