// tim67.h
#pragma once
#include <stdint.h>

typedef enum tim67_master_mode_enum {
  RESET,  // The UG bit from the TIMx_EGR register is used as a trigger output
  ENABLE, // The Counter enable signal, CNT_EN, is used as a trigger output
  UPDATE  // The update event is selected as a trigger output
} tim67_master_mode_enum;

typedef volatile struct /* tim67_struct */ {  // ! half-word access
// 00 Control register 1 (TIMx_CR1)
  uint16_t  CR1_CEN     :  1; // Counter enable
  uint16_t  CR1_UDIS    :  1; // Update disable
  uint16_t  CR1_URS     :  1; // Update request source
  uint16_t  CR1_OPM     :  1; // One-pulse mode
  uint16_t              :  3;
  uint16_t  CR1_ARPE    :  1; // Auto-reload preload enable
  uint16_t              :  8;
  uint16_t  rsvd_02;
// 04 Control register 2 (TIMx_CR2)
  uint16_t              :  4;
  uint16_t  CR2_MMS     :  3; // Master mode selection
  uint16_t              :  9;
  uint16_t  rsvd_06;
// 08
  uint16_t  rsvd_08;
  uint16_t  rsvd_0A;
// 0C DMA/Interrupt enable register (TIMx_DIER)
  uint16_t  DIER_UIE    :  1; // Update interrupt enable
  uint16_t              :  7;
  uint16_t  DIER_UDE    :  1; // Update DMA request enable
  uint16_t              :  7;
  uint16_t  rsvd_0E;
// 10 Status register (TIMx_SR)
  uint16_t  SR_UIF      :  1; // Update interrupt flag
  uint16_t              : 15;
  uint16_t  rsvd_12;
// 14 Event generation register (TIMx_EGR)
  uint16_t  EGR_UG      :  1; // Update generation
  uint16_t              : 15;
  uint16_t  rsvd_16;
// 18
  uint16_t  rsvd_18;
  uint16_t  rsvd_1A;
// 1C
  uint16_t  rsvd_1C;
  uint16_t  rsvd_1E;
// 20
  uint16_t  rsvd_20;
  uint16_t  rsvd_22;
// 24 Counter (TIMx_CNT)
  uint16_t  CNT;
  uint16_t  rsvd_26;
// 28 Prescaler (TIMx_PSC)
  uint16_t  PSC;
  uint16_t  rsvd_2A;
// 2C Auto-reload register (TIMx_ARR)
  uint16_t  ARR;
  uint16_t  rsvd_2E;
} tim67_struct;

_Static_assert (sizeof(tim67_struct) == 48, " tim67_struct: bad size");

static tim67_struct * const TIM6 = (tim67_struct *)0x40001000;
static tim67_struct * const TIM7 = (tim67_struct *)0x40001400;

