// uart.h
#pragma once
#include <stdint.h>

// Universal Synchronous Asynchronous Receiver Transmitter

typedef volatile struct /* usart_struct */ {  // word access
  // Status
  uint32_t SR_PE        : 1; //
  uint32_t SR_FE        : 1; //
  uint32_t SR_NF        : 1; //
  uint32_t SR_ORE       : 1; //
  uint32_t SR_IDLE      : 1; //
  uint32_t SR_RXNE      : 1; //
  uint32_t SR_TC        : 1; //
  uint32_t SR_TXE       : 1; //
  uint32_t SR_LBD       : 1; // LIN break detection flag
  uint32_t SR_CTS       : 1; // CTS change flag
  uint32_t              :22;
  // Data register
  uint32_t DR;
  // Baud rate
  uint32_t BRR;
  // Control register 1
  uint32_t CR1_SBK      : 1; // Send break
  uint32_t CR1_RWU      : 1; // Receiver wakeup, 1: mute
  uint32_t CR1_RE       : 1; //
  uint32_t CR1_TE       : 1; //
  uint32_t CR1_IDLEIE   : 1; //
  uint32_t CR1_RXNEIE   : 1; //
  uint32_t CR1_TCIE     : 1; //
  uint32_t CR1_TXEIE    : 1; //
  uint32_t CR1_PEIE     : 1; //
  uint32_t CR1_PS       : 1; // Parity selection, 1: odd
  uint32_t CR1_PCE      : 1; // Parity control enable
  uint32_t CR1_WAKE     : 1; // Wakeup method
  uint32_t CR1_M        : 1; //
  uint32_t CR1_UE       : 1; //
  uint32_t              : 1; //
  uint32_t CR1_OVER8    : 1; // Oversampling by 8
  uint32_t              :16; //
  // Control register 2
  uint32_t CR2_ADD      : 4; // Address of the USART node
  uint32_t              : 1; //
  uint32_t CR2_LBDL     : 1; //
  uint32_t CR2_LBDIE    : 1; //
  uint32_t              : 1; //
  uint32_t CR2_LBCL     : 1; // Last bit clock pulse (on SCLK output)
  uint32_t CR2_CPHA     : 1; //
  uint32_t CR2_CPOL     : 1; //
  uint32_t CR2_CLKEN    : 1; //
  uint32_t CR2_STOP     : 2; // 0->1 stop bit  1->0.5  2->2  3->1.5
  uint32_t CR2_LINEN    : 1; //
  uint32_t              :17; //
  // Control register 3
  uint32_t CR3_EIE      : 1; // Error interrupt enable
  uint32_t CR3_IREN     : 1; // IrDA mode enable
  uint32_t CR3_IRLP     : 1; // IrDA low-power
  uint32_t CR3_HDSEL    : 1; // Half-duplex selection
  uint32_t CR3_NACK     : 1; // Smartcard NACK enable
  uint32_t CR3_SCEN     : 1; // Smartcard mode enable
  uint32_t CR3_DMAR     : 1; // DMA enable rx
  uint32_t CR3_DMAT     : 1; // DMA enable tx
  uint32_t CR3_RTSE     : 1; // RTS hardware flow control enable
  uint32_t CR3_CTSE     : 1; // CTS hardware flow control enable
  uint32_t CR3_CTSIE    : 1; //
  uint32_t CR3_ONEBIT   : 1; // One sample bit method enable
  uint32_t              :20; //
  // Guard time and prescaler
  uint32_t GTPR_PSC     : 8; // IrDA prescaler
  uint32_t GTPR_GT      : 8; // Smartcard guard time
  uint32_t              :16; //
} usart_struct;

static usart_struct * const USART1 = (usart_struct *)0x40011000;
static usart_struct * const USART2 = (usart_struct *)0x40004400;
static usart_struct * const USART3 = (usart_struct *)0x40004800;
static usart_struct * const USART6 = (usart_struct *)0x40011400;


// Universal Asynchronous Receiver Transmitter

typedef volatile struct /* uart_struct */ {  // word access
  // Status
  uint32_t SR_PE        : 1; //
  uint32_t SR_FE        : 1; //
  uint32_t SR_NF        : 1; //
  uint32_t SR_ORE       : 1; //
  uint32_t SR_IDLE      : 1; //
  uint32_t SR_RXNE      : 1; //
  uint32_t SR_TC        : 1; //
  uint32_t SR_TXE       : 1; //
  uint32_t SR_LBD       : 1; // LIN break detection flag
  uint32_t              : 1; // CTS change flag
  uint32_t              :22;
  // Data register
  uint32_t DR;
  // Baud rate
  uint32_t BRR;
  // Control register 1
  uint32_t CR1_SBK      : 1; // Send break
  uint32_t CR1_RWU      : 1; // Receiver wakeup, 1: mute
  uint32_t CR1_RE       : 1; //
  uint32_t CR1_TE       : 1; //
  uint32_t CR1_IDLEIE   : 1; //
  uint32_t CR1_RXNEIE   : 1; //
  uint32_t CR1_TCIE     : 1; //
  uint32_t CR1_TXEIE    : 1; //
  uint32_t CR1_PEIE     : 1; //
  uint32_t CR1_PS       : 1; // Parity selection, 1: odd
  uint32_t CR1_PCE      : 1; // Parity control enable
  uint32_t CR1_WAKE     : 1; // Wakeup method
  uint32_t CR1_M        : 1; //
  uint32_t CR1_UE       : 1; //
  uint32_t              : 1; //
  uint32_t CR1_OVER8    : 1; // Oversampling by 8
  uint32_t              :16; //
  // Control register 2
  uint32_t CR2_ADD      : 4; // Address of the USART node
  uint32_t              : 1; //
  uint32_t CR2_LBDL     : 1; //
  uint32_t CR2_LBDIE    : 1; //
  uint32_t              : 1; //
  uint32_t              : 1; // Last bit clock pulse (on SCLK output)
  uint32_t              : 1; //
  uint32_t              : 1; //
  uint32_t              : 1; //
  uint32_t              : 1; //
  uint32_t CR2_STOP2    : 1; // 0: 1, 1: 2
  uint32_t CR2_LINEN    : 1; //
  uint32_t              :17; //
  // Control register 3
  uint32_t CR3_EIE      : 1; // Error interrupt enable
  uint32_t CR3_IREN     : 1; // IrDA mode enable
  uint32_t CR3_IRLP     : 1; // IrDA low-power
  uint32_t CR3_HDSEL    : 1; // Half-duplex selection
  uint32_t              : 1; // Smartcard NACK enable
  uint32_t              : 1; // Smartcard mode enable
  uint32_t CR3_DMAR     : 1; // DMA enable rx
  uint32_t CR3_DMAT     : 1; // DMA enable tx
  uint32_t              : 1; // RTS hardware flow control enable
  uint32_t              : 1; // CTS hardware flow control enable
  uint32_t              : 1; //
  uint32_t CR3_ONEBIT   : 1; // One sample bit method enable
  uint32_t              :20; //
  // (Guard time and prescaler)
  uint32_t rsvd;
} uart_struct;

static uart_struct * const UART1 = (uart_struct *)0x40011000;
static uart_struct * const UART2 = (uart_struct *)0x40004400;
static uart_struct * const UART3 = (uart_struct *)0x40004800;
static uart_struct * const UART4 = (uart_struct *)0x40004C00;
static uart_struct * const UART5 = (uart_struct *)0x40005000;
static uart_struct * const UART6 = (uart_struct *)0x40011400;

