// PCF212x.h
#pragma once

// I2C RTC PCF2129T/2, PCA2129T/Q900/2, PCF2127T/2(?)

static const int I2C_PCF212x = 81; // i2c Adressen haben 7 Bit und enthalten
                                   // kein Read/Write-Bit. Es gibt keine
                                   // getrennten Lese- und Schreib-Adressen.
                                   // Die beste Schreibweise wre 0b1010001,
                                   // aber das erlaubt C11 nicht.

static const int OSF = 128; // Oscillator stop flag

typedef enum PCF212x_register_enum {
  CR1 = 0,
  CR2,
  CR3,
  SECONDS,              // OBACHT: Bit 7 = OSF
  MINUTES,
  HOURS,
  DAYS,
  WEEKDAYS,             // 0 = sunday
  MONTHS,               // 1 = january
  YEARS,
  SECOND_ALARM,
  MINUTE_ALARM,
  HOUR_ALARM,
  DAY_ALARM,
  WEEKDAY_ALARM,
  CLKOUT_CTL,
  WD_TIM_CTL,
  WD_TIM_VAL,
  TIMESTP_CTL,
  SEC_TIMESTP,
  MIN_TIMESTP,
  HOUR_TIMESTP,
  DAY_TIMESTP,
  MONTH_TIMESTP,
  YEAR_TIMESTP,
  AGING_OFFSET,
  INTERNAL_0x1A,        // PCF2127T: RAM address MSB
  INTERNAL_0x1B         //           RAM address LSB
                        //           RAM_wrt_cmd
                        //           RAM_rd_cmd
} PCF212x_register_enum;

typedef enum PCF212x_CR1_enum {
  EXT_TEST   = 128,             // external clock test mode
  STOP       =  32,             // reset divider chain
  TSF1       =  16,             // timestamp flag 1 (input @VDD/2)
  POR_OVRD   =   8,             // power on reset override
  AMPM       =   4,             // select AM/PM mode
  MI         =   2,             // minute interrupt enable
  SI         =   1              // second interrupt enable
} PCF212x_cr1_enum;

typedef enum PCF212x_CR2_enum {
  MSF        = 128,             // minute or second interrupt flag
  WDTF       =  64,             // watchdog flag
  TSF2       =  32,             // timestamp flag 2 (input @GND)
  AF         =  16,             // alarm flag
  TSIE       =   4,             // timestamp interrupt enable
  AIE        =   2              // alarm interrupt enable
} PCF212x_cr2_enum;

typedef enum PCF212x_CR3_enum {
  PM_OFF     = (7 << 5),        // only one VDD supply, VBAT @ GND
  PM_DIR     = (4 << 5),        // direct switching @ VDD < VBAT
  PM_DIR_BLD = (3 << 5),        // direct, with battery low detect
  PM_STD     = (1 << 5),        // standard switching @ VDD < 2.5V
  PM_STD_BLD = (0 << 5),        // standard, with battery low detect
  BTSE       =  16,             // battery switch-over timestamp enable
  BF         =   8,             // battery switch-over flag
  BLF        =   4,             // battery low flag
  BIE        =   2,             // battery switch-over interrupt enable
  BLIE       =   1              // battery low interrupt enable
} PCF212x_cr3_enum;

typedef enum PCF212x_CLKOUT_enum {
  TCR_4MIN   = (0 << 6),        // temperature measurement period
  TCR_2MIN   = (1 << 6),
  TCR_1MIN   = (2 << 6),
  TCR_30S    = (3 << 6),
  OTPR       = 32,              // Calibration EEPROM refresh
  COF_32768  =  0,              // CLKOUT frequency
  COF_16384  =  1,
  COF_8192   =  2,
  COF_4096   =  3,
  COF_2048   =  4,
  COF_1024   =  5,
  COF_1Hz    =  6,
  COF_HIGH_Z =  7
} PCF212x_CLKOUT_enum;

typedef enum PCF212x_WD_TIM_CTL_enum {
  WD_CD      = 128,             // watchdog timer enabled
  TI_TP      =  32,             // select pulsed interrupt signal
  TF_4096    =   0,             // timer clock 4096 Hz
  TF_64      =   1,             // timer clock 64 Hz
  TF_1       =   2,             // timer clock 1 Hz
  TF_1MIN    =   3              // timer clock 1/60 Hz
} PCF212x_WD_TIM_CTL_enum;

//  AE_S = 128  // second alarm enable
//  AE_M = 128  // minute alarm enable
//  AE_H = 128  // hour alarm enable
//  AE_D = 128  // day alarm enable
//  AE_W = 128  // weekday alatm enable

