// asm.h
#pragma once
#include <stdint.h>

// http://www.ethernut.de/en/documents/arm-inline-asm.html

typedef enum CONTROL_enum {
   PRIVILEGED_USE_MSP,     // to be used with
   NOT_PRIVILEGED_USE_MSP, //
   PRIVILEGED_USE_PSP,     // set_CONTROL(), get_CONTROL()
   NOT_PRIVILEGED_USE_PSP  //
} CONTROL_enum;


static inline int
CLZ (uint32_t input)
{
uint32_t  result;

   __asm__ (" clz   %0, %1" : "=r" (result) : "r" (input));
   return result;
}


static inline int
CTZ (uint32_t input)
{
uint32_t  result, tupni;

   __asm__ (" rbit  %0, %2\n"
            " clz   %1, %0" : "=r" (tupni), "=r" (result) : "r" (input));
   return result;
}


static inline void
CPSID_I (void)
{
   __asm__ (" cpsid  i");
}


static inline void
CPSIE_I (void)
{
   __asm__ (" cpsie  i");
}


static inline void
DSB (void)
{
   __asm__ (" dsb");
}


static inline void
ISB (void)
{
   __asm__ (" isb");
}


static inline int
get_BASEPRI (void)
{
int result;

   __asm__ volatile (" mrs  %0, basepri" : "=r" (result) );
   return result & 3;
}


static inline int
get_CONTROL (void)
{
int result;

   __asm__ volatile (" mrs  %0, control" : "=r" (result) );
   return result & 3;
}


static inline int
get_FAULTMASK (void)
{
int result;

   __asm__ volatile (" mrs  %0, faultmask" : "=r" (result) );
   return result & 3;
}


static inline int
get_LR (void)
{
int result;

   __asm__ volatile (" mov  %0, lr" : "=r" (result) );
   return result;
}


static inline uint32_t
get_MSP (void)
{
uint32_t  result = 0;

   __asm__ volatile (" mrs  %0, msp" : "=r" (result) );
   return result;
}


static inline int
get_PRIMASK (void)
{
int result;

   __asm__ volatile (" mrs  %0, primask" : "=r" (result) );
   return result & 3;
}


static inline uint32_t
get_PSP (void)
{
uint32_t  result = 0;

   __asm__ volatile (" mrs  %0, psp" : "=r" (result) );
   return result;
}


static inline void
set_CONTROL (int bits)
{
   __asm__ volatile (" msr  control, %0" : : "r" (bits) );
   __asm__ (" isb");
}


static inline void
set_MSP (uint32_t ads)
{
   __asm__ volatile (" msr  msp, %0" : : "r" (ads) );
   __asm__ (" isb");
}


static inline void
set_PSP (uint32_t ads)
{
   __asm__ volatile (" msr  psp, %0" : : "r" (ads) );
   __asm__ (" isb");
}


static inline void
WFI (void)
{
   __asm__ (" wfi");
}

#if 0
static inline int
SVC (int function)
{
int  retval;

   __asm__ volatile (" svc  %[imm]"
                     : "=r" (retval)
                     : [imm] "I" (function)
                     : "cc", "memory");
   return retval;
}

static inline int
SVC (int func)
{
int  result;
   __asm__ volatile (" svc  %[imm]" : [r0] "=r" (result) : [imm] "I" (func));
   return result;
}
#endif

#if 0
static inline void
save_context (void)
{
uint32_t scratch;

   __asm__ volatile ("mrs    %0, psp\n\t"
                     "stmdb  %0!, {r4-r11}\n\t"
                     "msr    psp, %0\n\t" : "=r" (scratch) );
}


static inline void
load_context (void)
{
uint32_t scratch;

   __asm__ volatile ("mrs    %0, psp\n\t"
                     "ldmfd  %0!, {r4-r11}\n\t"
                     "msr    psp, %0\n\t" : "=r" (scratch) );
}
#endif

