// gpio-init.h
#pragma once
#include <stdint.h>
#include "F205/rcc.h"

typedef struct /* gpio_init_struct */ {
   union {
      struct {
         uint32_t  function : 24; // af, pin_mode, pin, port, rccbit, rccport
         uint32_t  otype    :  1; // see $(controller)/gpio.h
         uint32_t  odata    :  1; //
         uint32_t  ospeed   :  2; //
         uint32_t  pupd     :  2; //
         uint32_t  lock     :  1; //
         uint32_t           :  1;
      } i;
      struct {
         uint32_t  af       :  4; // 0=default .. 0xF=EVENTOUT
         uint32_t  pin_mode :  4; // see typedef pin_mode_enum
         uint32_t  pin      :  4;
         uint32_t  port     :  4; // 'X'-'@' -> A = 1, ... N = 0xE, O = 0xF
         uint32_t  rccbit   :  5; // see typedef rcc_bits_enum
         uint32_t  rccport  :  3; //
         uint32_t  otype    :  1; // see $(controller)/gpio.h
         uint32_t  odata    :  1; //
         uint32_t  ospeed   :  2; //
         uint32_t  pupd     :  2; //
         uint32_t  lock     :  1; //
         uint32_t           :  1;
      } o;
   };
} gpio_init_struct;

typedef enum pin_mode_enum {
   GM_IN     = 0x00,            // = MODE_INPUT
   GM_OUT    = 0x10,            // = MODE_OUTPUT
   GM_AF     = 0x20,            // = MODE_AF
   GM_ANA    = 0x30,            // = MODE_ANALOG
   GM_NC     = 0x40 + 0x00,     // not bonded/connected, defined by gpio_init
   GM_GND    = 0x40 + 0x10,     // PUSH_PULL OUTPUT LOW
   GM_RSVD60 = 0x40 + 0x20,     // unusable ???
   GM_RSVD70 = 0x40 + 0x30      // ADC, DAC, CMP... ???
} pin_mode_enum;

typedef enum rcc_bits_enum {
   RCC_0,
   RCC_CAN1   = APB1_CAN1   * 0x10000 + 0x30000 * 32, // special case:
   RCC_CAN2   = APB1_CAN2   * 0x10000 + 0x30000 * 32, // CAN2 needs CAN1
   RCC_I2C1   = APB1_I2C1   * 0x10000 + 0x10000 * 32,
   RCC_I2C2   = APB1_I2C2   * 0x10000 + 0x10000 * 32,
   RCC_I2C3   = APB1_I2C3   * 0x10000 + 0x10000 * 32,
   RCC_SPI1   = APB2_SPI1   * 0x10000 + 0x20000 * 32,
   RCC_SPI2   = APB1_SPI2   * 0x10000 + 0x10000 * 32,
   RCC_SPI3   = APB1_SPI3   * 0x10000 + 0x10000 * 32,
   RCC_TIM1   = APB2_TIM1   * 0x10000 + 0x20000 * 32,
   RCC_TIM2   = APB1_TIM2   * 0x10000 + 0x10000 * 32,
   RCC_TIM3   = APB1_TIM3   * 0x10000 + 0x10000 * 32,
   RCC_TIM4   = APB1_TIM4   * 0x10000 + 0x10000 * 32,
   RCC_TIM5   = APB1_TIM5   * 0x10000 + 0x10000 * 32,
   RCC_TIM6   = APB1_TIM6   * 0x10000 + 0x10000 * 32,
   RCC_TIM7   = APB1_TIM7   * 0x10000 + 0x10000 * 32,
   RCC_TIM8   = APB2_TIM8   * 0x10000 + 0x20000 * 32,
   RCC_TIM9   = APB2_TIM9   * 0x10000 + 0x20000 * 32,
   RCC_TIM10  = APB2_TIM10  * 0x10000 + 0x20000 * 32,
   RCC_TIM11  = APB2_TIM11  * 0x10000 + 0x20000 * 32,
   RCC_TIM12  = APB1_TIM12  * 0x10000 + 0x10000 * 32,
   RCC_TIM13  = APB1_TIM13  * 0x10000 + 0x10000 * 32,
   RCC_TIM14  = APB1_TIM14  * 0x10000 + 0x10000 * 32,
   RCC_USART1 = APB2_USART1 * 0x10000 + 0x20000 * 32,
   RCC_USART2 = APB1_USART2 * 0x10000 + 0x10000 * 32,
   RCC_USART3 = APB1_USART3 * 0x10000 + 0x10000 * 32,
   RCC_UART4  = APB1_UART4  * 0x10000 + 0x10000 * 32,
   RCC_UART5  = APB1_UART5  * 0x10000 + 0x10000 * 32,
   RCC_USART6 = APB2_USART6 * 0x10000 + 0x20000 * 32,
} rcc_bits_enum;

typedef enum af_pa0_enum {
   AF_PA0_TIM2_CH1_ETR =  1 | GM_AF  | 0x1000 | RCC_TIM2,
   AF_PA0_TIM5_CH1     =  2 | GM_AF  | 0x1000 | RCC_TIM5,
   AF_PA0_TIM8_ETR     =  3 | GM_AF  | 0x1000 | RCC_TIM8,
   AF_PA0_UART4_TX     =  8 | GM_AF  | 0x1000 | RCC_UART4,
   AF_PA0_USART2_CTS   =  7 | GM_AF  | 0x1000 | RCC_USART2,
   AF_PA0_INPUT        =  0 | GM_IN  | 0x1000 | RCC_0,
   AF_PA0_OUTPUT       =  0 | GM_OUT | 0x1000 | RCC_0,
   AF_PA0_ANALOG       =  0 | GM_ANA | 0x1000 | RCC_0,
   AF_PA0_SOFT_GND     =  0 | GM_GND | 0x1000 | RCC_0,
   AF_PA0_NC           =  0 | GM_NC  | 0x1000 | RCC_0
} pa0_enum;

typedef enum af_pa1_enum {
   AF_PA1_TIM2_CH2     =  1 | GM_AF  | 0x1100 | RCC_TIM2,
   AF_PA1_TIM5_CH2     =  2 | GM_AF  | 0x1100 | RCC_TIM5,
   AF_PA1_UART4_RX     =  8 | GM_AF  | 0x1100 | RCC_UART4,
   AF_PA1_USART2_RTS   =  7 | GM_AF  | 0x1100 | RCC_USART2,
   AF_PA1_INPUT        =  0 | GM_IN  | 0x1100 | RCC_0,
   AF_PA1_OUTPUT       =  0 | GM_OUT | 0x1100 | RCC_0,
   AF_PA1_ANALOG       =  0 | GM_ANA | 0x1100 | RCC_0,
   AF_PA1_SOFT_GND     =  0 | GM_GND | 0x1100 | RCC_0,
   AF_PA1_NC           =  0 | GM_NC  | 0x1100 | RCC_0
} pa1_enum;

typedef enum af_pa2_enum {
   AF_PA2_TIM2_CH3     =  1 | GM_AF  | 0x1200 | RCC_TIM2,
   AF_PA2_TIM5_CH3     =  2 | GM_AF  | 0x1200 | RCC_TIM5,
   AF_PA2_TIM9_CH1     =  3 | GM_AF  | 0x1200 | RCC_TIM9,
   AF_PA2_USART2_TX    =  7 | GM_AF  | 0x1200 | RCC_USART2,
   AF_PA2_INPUT        =  0 | GM_IN  | 0x1200 | RCC_0,
   AF_PA2_OUTPUT       =  0 | GM_OUT | 0x1200 | RCC_0,
   AF_PA2_ANALOG       =  0 | GM_ANA | 0x1200 | RCC_0,
   AF_PA2_SOFT_GND     =  0 | GM_GND | 0x1200 | RCC_0,
   AF_PA2_NC           =  0 | GM_NC  | 0x1200 | RCC_0
} pa2_enum;

typedef enum af_pa3_enum {
   AF_PA3_TIM2_CH4     =  1 | GM_AF  | 0x1300 | RCC_TIM2,
   AF_PA3_TIM5_CH4     =  2 | GM_AF  | 0x1300 | RCC_TIM5,
   AF_PA3_TIM9_CH2     =  3 | GM_AF  | 0x1300 | RCC_TIM9,
   AF_PA3_USART2_RX    =  7 | GM_AF  | 0x1300 | RCC_USART2,
   AF_PA3_INPUT        =  0 | GM_IN  | 0x1300 | RCC_0,
   AF_PA3_OUTPUT       =  0 | GM_OUT | 0x1300 | RCC_0,
   AF_PA3_ANALOG       =  0 | GM_ANA | 0x1300 | RCC_0,
   AF_PA3_SOFT_GND     =  0 | GM_GND | 0x1300 | RCC_0,
   AF_PA3_NC           =  0 | GM_NC  | 0x1300 | RCC_0
} pa3_enum;

typedef enum af_pa4_enum {
// AF_PA4_I2S3_WS      =  6 | GM_AF  | 0x1400 | RCC_???,
   AF_PA4_SPI1_NSS     =  5 | GM_AF  | 0x1400 | RCC_SPI1,
   AF_PA4_SPI3_NSS     =  6 | GM_AF  | 0x1400 | RCC_SPI3,
   AF_PA4_USART2_CK    =  7 | GM_AF  | 0x1400 | RCC_USART2,
   AF_PA4_INPUT        =  0 | GM_IN  | 0x1400 | RCC_0,
   AF_PA4_OUTPUT       =  0 | GM_OUT | 0x1400 | RCC_0,
   AF_PA4_ANALOG       =  0 | GM_ANA | 0x1400 | RCC_0,
   AF_PA4_SOFT_GND     =  0 | GM_GND | 0x1400 | RCC_0,
   AF_PA4_NC           =  0 | GM_NC  | 0x1400 | RCC_0
} pa4_enum;

typedef enum af_pa5_enum {
   AF_PA5_SPI1_SCK     =  5 | GM_AF  | 0x1500 | RCC_SPI1,
   AF_PA5_TIM2_CH1_ETR =  1 | GM_AF  | 0x1500 | RCC_TIM2,
   AF_PA5_TIM8_CH1N    =  3 | GM_AF  | 0x1500 | RCC_TIM8,
   AF_PA5_INPUT        =  0 | GM_IN  | 0x1500 | RCC_0,
   AF_PA5_OUTPUT       =  0 | GM_OUT | 0x1500 | RCC_0,
   AF_PA5_ANALOG       =  0 | GM_ANA | 0x1500 | RCC_0,
   AF_PA5_SOFT_GND     =  0 | GM_GND | 0x1500 | RCC_0,
   AF_PA5_NC           =  0 | GM_NC  | 0x1500 | RCC_0
} pa5_enum;

typedef enum af_pa6_enum {
   AF_PA6_SPI1_MISO    =  5 | GM_AF  | 0x1600 | RCC_SPI1,
   AF_PA6_TIM1_BKIN    =  1 | GM_AF  | 0x1600 | RCC_TIM1,
   AF_PA6_TIM3_CH1     =  2 | GM_AF  | 0x1600 | RCC_TIM3,
   AF_PA6_TIM8_BKIN    =  3 | GM_AF  | 0x1600 | RCC_TIM8,
   AF_PA6_INPUT        =  0 | GM_IN  | 0x1600 | RCC_0,
   AF_PA6_OUTPUT       =  0 | GM_OUT | 0x1600 | RCC_0,
   AF_PA6_ANALOG       =  0 | GM_ANA | 0x1600 | RCC_0,
   AF_PA6_SOFT_GND     =  0 | GM_GND | 0x1600 | RCC_0,
   AF_PA6_NC           =  0 | GM_NC  | 0x1600 | RCC_0
} pa6_enum;

typedef enum af_pa7_enum {
   AF_PA7_SPI1_MOSI    =  5 | GM_AF  | 0x1700 | RCC_SPI1,
   AF_PA7_TIM1_CH1N    =  1 | GM_AF  | 0x1700 | RCC_TIM1,
   AF_PA7_TIM3_CH2     =  2 | GM_AF  | 0x1700 | RCC_TIM3,
   AF_PA7_INPUT        =  0 | GM_IN  | 0x1700 | RCC_0,
   AF_PA7_OUTPUT       =  0 | GM_OUT | 0x1700 | RCC_0,
   AF_PA7_ANALOG       =  0 | GM_ANA | 0x1700 | RCC_0,
   AF_PA7_SOFT_GND     =  0 | GM_GND | 0x1700 | RCC_0,
   AF_PA7_NC           =  0 | GM_NC  | 0x1700 | RCC_0
} pa7_enum;

typedef enum af_pa8_enum {
   AF_PA8_I2C3_SCL     =  4 | GM_AF  | 0x1800 | RCC_I2C3,
   AF_PA8_MCO1         =  0 | GM_AF  | 0x1800 | RCC_0,
   AF_PA8_TIM1_CH1     =  1 | GM_AF  | 0x1800 | RCC_TIM1,
   AF_PA8_USART1_CK    =  7 | GM_AF  | 0x1800 | RCC_USART1,
   AF_PA8_INPUT        =  0 | GM_IN  | 0x1800 | RCC_0,
   AF_PA8_OUTPUT       =  0 | GM_OUT | 0x1800 | RCC_0,
   AF_PA8_ANALOG       =  0 | GM_ANA | 0x1800 | RCC_0,
   AF_PA8_SOFT_GND     =  0 | GM_GND | 0x1800 | RCC_0,
   AF_PA8_NC           =  0 | GM_NC  | 0x1800 | RCC_0
} pa8_enum;

typedef enum af_pa9_enum {
   AF_PA9_I2C3_SMBA    =  4 | GM_AF  | 0x1900 | RCC_I2C3,
   AF_PA9_TIM1_CH2     =  1 | GM_AF  | 0x1900 | RCC_TIM1,
   AF_PA9_USART1_TX    =  7 | GM_AF  | 0x1900 | RCC_USART1,
   AF_PA9_INPUT        =  0 | GM_IN  | 0x1900 | RCC_0,
   AF_PA9_OUTPUT       =  0 | GM_OUT | 0x1900 | RCC_0,
   AF_PA9_ANALOG       =  0 | GM_ANA | 0x1900 | RCC_0,
   AF_PA9_SOFT_GND     =  0 | GM_GND | 0x1900 | RCC_0,
   AF_PA9_NC           =  0 | GM_NC  | 0x1900 | RCC_0
} pa9_enum;

typedef enum af_pa10_enum {
   AF_PA10_TIM1_CH3    =  1 | GM_AF  | 0x1A00 | RCC_TIM1,
   AF_PA10_USART1_RX   =  7 | GM_AF  | 0x1A00 | RCC_USART1,
   AF_PA10_INPUT       =  0 | GM_IN  | 0x1A00 | RCC_0,
   AF_PA10_OUTPUT      =  0 | GM_OUT | 0x1A00 | RCC_0,
   AF_PA10_ANALOG      =  0 | GM_ANA | 0x1A00 | RCC_0,
   AF_PA10_SOFT_GND    =  0 | GM_GND | 0x1A00 | RCC_0,
   AF_PA10_NC          =  0 | GM_NC  | 0x1A00 | RCC_0
} pa10_enum;

typedef enum af_pa11_enum {
   AF_PA11_TIM1_CH4    =  1 | GM_AF  | 0x1B00 | RCC_TIM1,
   AF_PA11_USART1_CTS  =  7 | GM_AF  | 0x1B00 | RCC_USART1,
   AF_PA11_INPUT       =  0 | GM_IN  | 0x1B00 | RCC_0,
   AF_PA11_OUTPUT      =  0 | GM_OUT | 0x1B00 | RCC_0,
   AF_PA11_ANALOG      =  0 | GM_ANA | 0x1B00 | RCC_0,
   AF_PA11_SOFT_GND    =  0 | GM_GND | 0x1B00 | RCC_0,
   AF_PA11_NC          =  0 | GM_NC  | 0x1B00 | RCC_0
} pa11_enum;

typedef enum af_pa12_enum {
   AF_PA12_TIM1_ETR    =  1 | GM_AF  | 0x1C00 | RCC_TIM1,
   AF_PA12_USART1_RTS  =  7 | GM_AF  | 0x1C00 | RCC_USART1,
   AF_PA12_INPUT       =  0 | GM_IN  | 0x1C00 | RCC_0,
   AF_PA12_OUTPUT      =  0 | GM_OUT | 0x1C00 | RCC_0,
   AF_PA12_ANALOG      =  0 | GM_ANA | 0x1C00 | RCC_0,
   AF_PA12_SOFT_GND    =  0 | GM_GND | 0x1C00 | RCC_0,
   AF_PA12_NC          =  0 | GM_NC  | 0x1C00 | RCC_0
} pa12_enum;

typedef enum af_pa13_enum {
   AF_PA13_SWDIO       =  0 | GM_AF  | 0x1D00 | RCC_0,
   AF_PA13_INPUT       =  0 | GM_IN  | 0x1D00 | RCC_0,
   AF_PA13_OUTPUT      =  0 | GM_OUT | 0x1D00 | RCC_0,
   AF_PA13_ANALOG      =  0 | GM_ANA | 0x1D00 | RCC_0,
   AF_PA13_SOFT_GND    =  0 | GM_GND | 0x1D00 | RCC_0,
   AF_PA13_NC          =  0 | GM_NC  | 0x1D00 | RCC_0
} pa13_enum;

typedef enum af_pa14_enum {
   AF_PA14_SWCLK       =  0 | GM_AF  | 0x1E00 | RCC_0,
   AF_PA14_INPUT       =  0 | GM_IN  | 0x1E00 | RCC_0,
   AF_PA14_OUTPUT      =  0 | GM_OUT | 0x1E00 | RCC_0,
   AF_PA14_ANALOG      =  0 | GM_ANA | 0x1E00 | RCC_0,
   AF_PA14_SOFT_GND    =  0 | GM_GND | 0x1E00 | RCC_0,
   AF_PA14_NC          =  0 | GM_NC  | 0x1E00 | RCC_0
} pa14_enum;

typedef enum af_pa15_enum {
// AF_PA15_I2S3_WS     =  6 | GM_AF  | 0x1F00 | RCC_???,
   AF_PA15_SPI1_NSS    =  5 | GM_AF  | 0x1F00 | RCC_SPI1,
   AF_PA15_SPI3_NSS    =  6 | GM_AF  | 0x1F00 | RCC_SPI3,
   AF_PA15_TIM2_CH1    =  1 | GM_AF  | 0x1F00 | RCC_TIM2,
   AF_PA15_TIM2_ETR    =  1 | GM_AF  | 0x1F00 | RCC_TIM2,
   AF_PA15_INPUT       =  0 | GM_IN  | 0x1F00 | RCC_0,
   AF_PA15_OUTPUT      =  0 | GM_OUT | 0x1F00 | RCC_0,
   AF_PA15_ANALOG      =  0 | GM_ANA | 0x1F00 | RCC_0,
   AF_PA15_SOFT_GND    =  0 | GM_GND | 0x1F00 | RCC_0,
   AF_PA15_NC          =  0 | GM_NC  | 0x1F00 | RCC_0
} pa15_enum;

typedef enum af_pb0_enum {
   AF_PB0_TIM1_CH2N    =  1 | GM_AF  | 0x2000 | RCC_TIM1,
   AF_PB0_TIM3_CH3     =  2 | GM_AF  | 0x2000 | RCC_TIM3,
   AF_PB0_TIM8_CH2N    =  3 | GM_AF  | 0x2000 | RCC_TIM8,
   AF_PB0_INPUT        =  0 | GM_IN  | 0x2000 | RCC_0,
   AF_PB0_OUTPUT       =  0 | GM_OUT | 0x2000 | RCC_0,
   AF_PB0_ANALOG       =  0 | GM_ANA | 0x2000 | RCC_0,
   AF_PB0_SOFT_GND     =  0 | GM_GND | 0x2000 | RCC_0,
   AF_PB0_NC           =  0 | GM_NC  | 0x2000 | RCC_0
} pb0_enum;

typedef enum af_pb1_enum {
   AF_PB1_TIM1_CH3N    =  1 | GM_AF  | 0x2100 | RCC_TIM1,
   AF_PB1_TIM3_CH4     =  2 | GM_AF  | 0x2100 | RCC_TIM3,
   AF_PB1_TIM8_CH3N    =  3 | GM_AF  | 0x2100 | RCC_TIM8,
   AF_PB1_INPUT        =  0 | GM_IN  | 0x2100 | RCC_0,
   AF_PB1_OUTPUT       =  0 | GM_OUT | 0x2100 | RCC_0,
   AF_PB1_ANALOG       =  0 | GM_ANA | 0x2100 | RCC_0,
   AF_PB1_SOFT_GND     =  0 | GM_GND | 0x2100 | RCC_0,
   AF_PB1_NC           =  0 | GM_NC  | 0x2100 | RCC_0
} pb1_enum;

typedef enum af_pb2_enum {
   AF_PB2_INPUT        =  0 | GM_IN  | 0x2200 | RCC_0,
   AF_PB2_OUTPUT       =  0 | GM_OUT | 0x2200 | RCC_0,
   AF_PB2_ANALOG       =  0 | GM_ANA | 0x2200 | RCC_0,
   AF_PB2_SOFT_GND     =  0 | GM_GND | 0x2200 | RCC_0,
   AF_PB2_NC           =  0 | GM_NC  | 0x2200 | RCC_0
} pb2_enum;

typedef enum af_pb3_enum {
// AF_PB3_I2S3_SCK     =  6 | GM_AF  | 0x2300 | RCC_???,
   AF_PB3_SPI1_SCK     =  5 | GM_AF  | 0x2300 | RCC_SPI1,
   AF_PB3_SPI3_SCK     =  6 | GM_AF  | 0x2300 | RCC_SPI3,
   AF_PB3_TIM2_CH2     =  1 | GM_AF  | 0x2300 | RCC_TIM2,
   AF_PB3_TRACESWO     =  0 | GM_AF  | 0x2300 | RCC_0,
   AF_PB3_INPUT        =  0 | GM_IN  | 0x2300 | RCC_0,
   AF_PB3_OUTPUT       =  0 | GM_OUT | 0x2300 | RCC_0,
   AF_PB3_ANALOG       =  0 | GM_ANA | 0x2300 | RCC_0,
   AF_PB3_SOFT_GND     =  0 | GM_GND | 0x2300 | RCC_0,
   AF_PB3_NC           =  0 | GM_NC  | 0x2300 | RCC_0
} pb3_enum;

typedef enum af_pb4_enum {
   AF_PB4_SPI1_MISO    =  5 | GM_AF  | 0x2400 | RCC_SPI1,
   AF_PB4_SPI3_MISO    =  6 | GM_AF  | 0x2400 | RCC_SPI3,
   AF_PB4_TIM3_CH1     =  2 | GM_AF  | 0x2400 | RCC_TIM3,
   AF_PB4_INPUT        =  0 | GM_IN  | 0x2400 | RCC_0,
   AF_PB4_OUTPUT       =  0 | GM_OUT | 0x2400 | RCC_0,
   AF_PB4_ANALOG       =  0 | GM_ANA | 0x2400 | RCC_0,
   AF_PB4_SOFT_GND     =  0 | GM_GND | 0x2400 | RCC_0,
   AF_PB4_NC           =  0 | GM_NC  | 0x2400 | RCC_0
} pb4_enum;

typedef enum af_pb5_enum {
   AF_PB5_CAN2_RX      =  9 | GM_AF  | 0x2500 | RCC_CAN2,
   AF_PB5_I2C1_SMBA    =  4 | GM_AF  | 0x2500 | RCC_I2C1,
// AF_PB5_I2S3_SD      =  6 | GM_AF  | 0x2500 | RCC_???,
   AF_PB5_SPI1_MOSI    =  5 | GM_AF  | 0x2500 | RCC_SPI1,
   AF_PB5_SPI3_MOSI    =  6 | GM_AF  | 0x2500 | RCC_SPI3,
   AF_PB5_TIM3_CH2     =  2 | GM_AF  | 0x2500 | RCC_TIM3,
   AF_PB5_INPUT        =  0 | GM_IN  | 0x2500 | RCC_0,
   AF_PB5_OUTPUT       =  0 | GM_OUT | 0x2500 | RCC_0,
   AF_PB5_ANALOG       =  0 | GM_ANA | 0x2500 | RCC_0,
   AF_PB5_SOFT_GND     =  0 | GM_GND | 0x2500 | RCC_0,
   AF_PB5_NC           =  0 | GM_NC  | 0x2500 | RCC_0
} pb5_enum;

typedef enum af_pb6_enum {
   AF_PB6_CAN2_TX      =  9 | GM_AF  | 0x2600 | RCC_CAN2,
   AF_PB6_I2C1_SCL     =  4 | GM_AF  | 0x2600 | RCC_I2C1,
   AF_PB6_TIM4_CH1     =  2 | GM_AF  | 0x2600 | RCC_TIM4,
   AF_PB6_USART1_TX    =  7 | GM_AF  | 0x2600 | RCC_USART1,
   AF_PB6_INPUT        =  0 | GM_IN  | 0x2600 | RCC_0,
   AF_PB6_OUTPUT       =  0 | GM_OUT | 0x2600 | RCC_0,
   AF_PB6_ANALOG       =  0 | GM_ANA | 0x2600 | RCC_0,
   AF_PB6_SOFT_GND     =  0 | GM_GND | 0x2600 | RCC_0,
   AF_PB6_NC           =  0 | GM_NC  | 0x2600 | RCC_0
} pb6_enum;

typedef enum af_pb7_enum {
   AF_PB7_I2C1_SDA     =  4 | GM_AF  | 0x2700 | RCC_I2C1,
   AF_PB7_TIM4_CH2     =  2 | GM_AF  | 0x2700 | RCC_TIM4,
   AF_PB7_USART1_RX    =  7 | GM_AF  | 0x2700 | RCC_USART1,
   AF_PB7_INPUT        =  0 | GM_IN  | 0x2700 | RCC_0,
   AF_PB7_OUTPUT       =  0 | GM_OUT | 0x2700 | RCC_0,
   AF_PB7_ANALOG       =  0 | GM_ANA | 0x2700 | RCC_0,
   AF_PB7_SOFT_GND     =  0 | GM_GND | 0x2700 | RCC_0,
   AF_PB7_NC           =  0 | GM_NC  | 0x2700 | RCC_0
} pb7_enum;

typedef enum af_pb8_enum {
   AF_PB8_CAN1_RX      =  9 | GM_AF  | 0x2800 | RCC_CAN1,
   AF_PB8_I2C1_SCL     =  4 | GM_AF  | 0x2800 | RCC_I2C1,
   AF_PB8_TIM10_CH1    =  3 | GM_AF  | 0x2800 | RCC_TIM10,
   AF_PB8_TIM4_CH3     =  2 | GM_AF  | 0x2800 | RCC_TIM4,
   AF_PB8_INPUT        =  0 | GM_IN  | 0x2800 | RCC_0,
   AF_PB8_OUTPUT       =  0 | GM_OUT | 0x2800 | RCC_0,
   AF_PB8_ANALOG       =  0 | GM_ANA | 0x2800 | RCC_0,
   AF_PB8_SOFT_GND     =  0 | GM_GND | 0x2800 | RCC_0,
   AF_PB8_NC           =  0 | GM_NC  | 0x2800 | RCC_0
} pb8_enum;

typedef enum af_pb9_enum {
   AF_PB9_CAN1_TX      =  9 | GM_AF  | 0x2900 | RCC_CAN1,
   AF_PB9_I2C1_SDA     =  4 | GM_AF  | 0x2900 | RCC_I2C1,
// AF_PB9_I2S2_WS      =  5 | GM_AF  | 0x2900 | RCC_???,
   AF_PB9_SPI2_NSS     =  5 | GM_AF  | 0x2900 | RCC_SPI2,
   AF_PB9_TIM11_CH1    =  3 | GM_AF  | 0x2900 | RCC_TIM11,
   AF_PB9_TIM4_CH4     =  2 | GM_AF  | 0x2900 | RCC_TIM4,
   AF_PB9_INPUT        =  0 | GM_IN  | 0x2900 | RCC_0,
   AF_PB9_OUTPUT       =  0 | GM_OUT | 0x2900 | RCC_0,
   AF_PB9_ANALOG       =  0 | GM_ANA | 0x2900 | RCC_0,
   AF_PB9_SOFT_GND     =  0 | GM_GND | 0x2900 | RCC_0,
   AF_PB9_NC           =  0 | GM_NC  | 0x2900 | RCC_0
} pb9_enum;

typedef enum af_pb10_enum {
   AF_PB10_I2C2_SCL    =  4 | GM_AF  | 0x2A00 | RCC_I2C2,
// AF_PB10_I2S2_SCK    =  5 | GM_AF  | 0x2A00 | RCC_???,
   AF_PB10_SPI2_SCK    =  5 | GM_AF  | 0x2A00 | RCC_SPI2,
   AF_PB10_TIM2_CH3    =  1 | GM_AF  | 0x2A00 | RCC_TIM2,
   AF_PB10_USART3_TX   =  7 | GM_AF  | 0x2A00 | RCC_USART3,
   AF_PB10_INPUT       =  0 | GM_IN  | 0x2A00 | RCC_0,
   AF_PB10_OUTPUT      =  0 | GM_OUT | 0x2A00 | RCC_0,
   AF_PB10_ANALOG      =  0 | GM_ANA | 0x2A00 | RCC_0,
   AF_PB10_SOFT_GND    =  0 | GM_GND | 0x2A00 | RCC_0,
   AF_PB10_NC          =  0 | GM_NC  | 0x2A00 | RCC_0
} pb10_enum;

typedef enum af_pb11_enum {
   AF_PB11_I2C2_SDA    =  4 | GM_AF  | 0x2B00 | RCC_I2C2,
   AF_PB11_TIM2_CH4    =  1 | GM_AF  | 0x2B00 | RCC_TIM2,
   AF_PB11_USART3_RX   =  7 | GM_AF  | 0x2B00 | RCC_USART3,
   AF_PB11_INPUT       =  0 | GM_IN  | 0x2B00 | RCC_0,
   AF_PB11_OUTPUT      =  0 | GM_OUT | 0x2B00 | RCC_0,
   AF_PB11_ANALOG      =  0 | GM_ANA | 0x2B00 | RCC_0,
   AF_PB11_SOFT_GND    =  0 | GM_GND | 0x2B00 | RCC_0,
   AF_PB11_NC          =  0 | GM_NC  | 0x2B00 | RCC_0
} pb11_enum;

typedef enum af_pb12_enum {
   AF_PB12_CAN2_RX     =  9 | GM_AF  | 0x2C00 | RCC_CAN2,
   AF_PB12_I2C2_SMBA   =  4 | GM_AF  | 0x2C00 | RCC_I2C2,
// AF_PB12_I2S2_WS     =  5 | GM_AF  | 0x2C00 | RCC_???,
   AF_PB12_SPI2_NSS    =  5 | GM_AF  | 0x2C00 | RCC_SPI2,
   AF_PB12_TIM1_BKIN   =  1 | GM_AF  | 0x2C00 | RCC_TIM1,
   AF_PB12_USART3_CK   =  7 | GM_AF  | 0x2C00 | RCC_USART3,
   AF_PB12_INPUT       =  0 | GM_IN  | 0x2C00 | RCC_0,
   AF_PB12_OUTPUT      =  0 | GM_OUT | 0x2C00 | RCC_0,
   AF_PB12_ANALOG      =  0 | GM_ANA | 0x2C00 | RCC_0,
   AF_PB12_SOFT_GND    =  0 | GM_GND | 0x2C00 | RCC_0,
   AF_PB12_NC          =  0 | GM_NC  | 0x2C00 | RCC_0
} pb12_enum;

typedef enum af_pb13_enum {
   AF_PB13_CAN2_TX     =  9 | GM_AF  | 0x2D00 | RCC_CAN2,
// AF_PB13_I2S2_SCK    =  5 | GM_AF  | 0x2D00 | RCC_???,
   AF_PB13_SPI2_SCK    =  5 | GM_AF  | 0x2D00 | RCC_SPI2,
   AF_PB13_TIM1_CH1N   =  1 | GM_AF  | 0x2D00 | RCC_TIM1,
   AF_PB13_USART3_CTS  =  7 | GM_AF  | 0x2D00 | RCC_USART3,
   AF_PB13_INPUT       =  0 | GM_IN  | 0x2D00 | RCC_0,
   AF_PB13_OUTPUT      =  0 | GM_OUT | 0x2D00 | RCC_0,
   AF_PB13_ANALOG      =  0 | GM_ANA | 0x2D00 | RCC_0,
   AF_PB13_SOFT_GND    =  0 | GM_GND | 0x2D00 | RCC_0,
   AF_PB13_NC          =  0 | GM_NC  | 0x2D00 | RCC_0
} pb13_enum;

typedef enum af_pb14_enum {
   AF_PB14_SPI2_MISO   =  5 | GM_AF  | 0x2E00 | RCC_SPI2,
   AF_PB14_TIM12_CH1   =  9 | GM_AF  | 0x2E00 | RCC_TIM12,
   AF_PB14_TIM1_CH2N   =  1 | GM_AF  | 0x2E00 | RCC_TIM1,
   AF_PB14_TIM8_CH2N   =  3 | GM_AF  | 0x2E00 | RCC_TIM8,
   AF_PB14_USART3_RTS  =  7 | GM_AF  | 0x2E00 | RCC_USART3,
   AF_PB14_INPUT       =  0 | GM_IN  | 0x2E00 | RCC_0,
   AF_PB14_OUTPUT      =  0 | GM_OUT | 0x2E00 | RCC_0,
   AF_PB14_ANALOG      =  0 | GM_ANA | 0x2E00 | RCC_0,
   AF_PB14_SOFT_GND    =  0 | GM_GND | 0x2E00 | RCC_0,
   AF_PB14_NC          =  0 | GM_NC  | 0x2E00 | RCC_0
} pb14_enum;

typedef enum af_pb15_enum {
// AF_PB15_I2S2_SD     =  5 | GM_AF  | 0x2F00 | RCC_???,
   AF_PB15_RTC_50Hz    =  0 | GM_AF  | 0x2F00 | RCC_0,
   AF_PB15_SPI2_MOSI   =  5 | GM_AF  | 0x2F00 | RCC_SPI2,
   AF_PB15_TIM12_CH2   =  9 | GM_AF  | 0x2F00 | RCC_TIM12,
   AF_PB15_TIM1_CH3N   =  1 | GM_AF  | 0x2F00 | RCC_TIM1,
   AF_PB15_TIM8_CH3N   =  3 | GM_AF  | 0x2F00 | RCC_TIM8,
   AF_PB15_INPUT       =  0 | GM_IN  | 0x2F00 | RCC_0,
   AF_PB15_OUTPUT      =  0 | GM_OUT | 0x2F00 | RCC_0,
   AF_PB15_ANALOG      =  0 | GM_ANA | 0x2F00 | RCC_0,
   AF_PB15_SOFT_GND    =  0 | GM_GND | 0x2F00 | RCC_0,
   AF_PB15_NC          =  0 | GM_NC  | 0x2F00 | RCC_0
} pb15_enum;

typedef enum af_pc0_enum {
   AF_PC0_INPUT        =  0 | GM_IN  | 0x3000 | RCC_0,
   AF_PC0_OUTPUT       =  0 | GM_OUT | 0x3000 | RCC_0,
   AF_PC0_ANALOG       =  0 | GM_ANA | 0x3000 | RCC_0,
   AF_PC0_SOFT_GND     =  0 | GM_GND | 0x3000 | RCC_0,
   AF_PC0_NC           =  0 | GM_NC  | 0x3000 | RCC_0
} pc0_enum;

typedef enum af_pc1_enum {
   AF_PC1_INPUT        =  0 | GM_IN  | 0x3100 | RCC_0,
   AF_PC1_OUTPUT       =  0 | GM_OUT | 0x3100 | RCC_0,
   AF_PC1_ANALOG       =  0 | GM_ANA | 0x3100 | RCC_0,
   AF_PC1_SOFT_GND     =  0 | GM_GND | 0x3100 | RCC_0,
   AF_PC1_NC           =  0 | GM_NC  | 0x3100 | RCC_0
} pc1_enum;

typedef enum af_pc2_enum {
   AF_PC2_INPUT        =  0 | GM_IN  | 0x3200 | RCC_0,
   AF_PC2_OUTPUT       =  0 | GM_OUT | 0x3200 | RCC_0,
   AF_PC2_ANALOG       =  0 | GM_ANA | 0x3200 | RCC_0,
   AF_PC2_SOFT_GND     =  0 | GM_GND | 0x3200 | RCC_0,
   AF_PC2_NC           =  0 | GM_NC  | 0x3200 | RCC_0
} pc2_enum;

typedef enum af_pc3_enum {
   AF_PC3_SPI2_MOSI    =  5 | GM_AF  | 0x3300 | RCC_SPI2,
   AF_PC3_INPUT        =  0 | GM_IN  | 0x3300 | RCC_0,
   AF_PC3_OUTPUT       =  0 | GM_OUT | 0x3300 | RCC_0,
   AF_PC3_ANALOG       =  0 | GM_ANA | 0x3300 | RCC_0,
   AF_PC3_SOFT_GND     =  0 | GM_GND | 0x3300 | RCC_0,
   AF_PC3_NC           =  0 | GM_NC  | 0x3300 | RCC_0
} pc3_enum;

typedef enum af_pc4_enum {
   AF_PC4_INPUT        =  0 | GM_IN  | 0x3400 | RCC_0,
   AF_PC4_OUTPUT       =  0 | GM_OUT | 0x3400 | RCC_0,
   AF_PC4_ANALOG       =  0 | GM_ANA | 0x3400 | RCC_0,
   AF_PC4_SOFT_GND     =  0 | GM_GND | 0x3400 | RCC_0,
   AF_PC4_NC           =  0 | GM_NC  | 0x3400 | RCC_0
} pc4_enum;

typedef enum af_pc5_enum {
   AF_PC5_INPUT        =  0 | GM_IN  | 0x3500 | RCC_0,
   AF_PC5_OUTPUT       =  0 | GM_OUT | 0x3500 | RCC_0,
   AF_PC5_ANALOG       =  0 | GM_ANA | 0x3500 | RCC_0,
   AF_PC5_SOFT_GND     =  0 | GM_GND | 0x3500 | RCC_0,
   AF_PC5_NC           =  0 | GM_NC  | 0x3500 | RCC_0
} pc5_enum;

typedef enum af_pc6_enum {
// AF_PC6_I2S2_MCK     =  5 | GM_AF  | 0x3600 | RCC_???,
   AF_PC6_TIM13_CH1    =  9 | GM_AF  | 0x3600 | RCC_TIM13,
   AF_PC6_TIM3_CH1     =  2 | GM_AF  | 0x3600 | RCC_TIM3,
   AF_PC6_TIM8_CH1     =  3 | GM_AF  | 0x3600 | RCC_TIM8,
   AF_PC6_USART6_TX    =  8 | GM_AF  | 0x3600 | RCC_USART6,
   AF_PC6_INPUT        =  0 | GM_IN  | 0x3600 | RCC_0,
   AF_PC6_OUTPUT       =  0 | GM_OUT | 0x3600 | RCC_0,
   AF_PC6_ANALOG       =  0 | GM_ANA | 0x3600 | RCC_0,
   AF_PC6_SOFT_GND     =  0 | GM_GND | 0x3600 | RCC_0,
   AF_PC6_NC           =  0 | GM_NC  | 0x3600 | RCC_0
} pc6_enum;

typedef enum af_pc7_enum {
// AF_PC7_I2S3_MCK     =  6 | GM_AF  | 0x3700 | RCC_???,
   AF_PC7_TIM14_CH1    =  9 | GM_AF  | 0x3700 | RCC_TIM14,
   AF_PC7_TIM3_CH2     =  2 | GM_AF  | 0x3700 | RCC_TIM3,
   AF_PC7_TIM8_CH2     =  3 | GM_AF  | 0x3700 | RCC_TIM8,
   AF_PC7_USART6_RX    =  8 | GM_AF  | 0x3700 | RCC_USART6,
   AF_PC7_INPUT        =  0 | GM_IN  | 0x3700 | RCC_0,
   AF_PC7_OUTPUT       =  0 | GM_OUT | 0x3700 | RCC_0,
   AF_PC7_ANALOG       =  0 | GM_ANA | 0x3700 | RCC_0,
   AF_PC7_SOFT_GND     =  0 | GM_GND | 0x3700 | RCC_0,
   AF_PC7_NC           =  0 | GM_NC  | 0x3700 | RCC_0
} pc7_enum;

typedef enum af_pc8_enum {
   AF_PC8_TIM3_CH3     =  2 | GM_AF  | 0x3800 | RCC_TIM3,
   AF_PC8_TIM8_CH3     =  3 | GM_AF  | 0x3800 | RCC_TIM8,
   AF_PC8_USART6_CK    =  8 | GM_AF  | 0x3800 | RCC_USART6,
   AF_PC8_INPUT        =  0 | GM_IN  | 0x3800 | RCC_0,
   AF_PC8_OUTPUT       =  0 | GM_OUT | 0x3800 | RCC_0,
   AF_PC8_ANALOG       =  0 | GM_ANA | 0x3800 | RCC_0,
   AF_PC8_SOFT_GND     =  0 | GM_GND | 0x3800 | RCC_0,
   AF_PC8_NC           =  0 | GM_NC  | 0x3800 | RCC_0
} pc8_enum;

typedef enum af_pc9_enum {
   AF_PC9_I2C3_SDA     =  4 | GM_AF  | 0x3900 | RCC_I2C3,
// AF_PC9_I2S2_CKIN    =  5 | GM_AF  | 0x3900 | RCC_???,
// AF_PC9_I2S3_CKIN    =  6 | GM_AF  | 0x3900 | RCC_???,
   AF_PC9_MCO2         =  0 | GM_AF  | 0x3900 | RCC_0,
   AF_PC9_TIM3_CH4     =  2 | GM_AF  | 0x3900 | RCC_TIM3,
   AF_PC9_TIM8_CH4     =  3 | GM_AF  | 0x3900 | RCC_TIM8,
   AF_PC9_INPUT        =  0 | GM_IN  | 0x3900 | RCC_0,
   AF_PC9_OUTPUT       =  0 | GM_OUT | 0x3900 | RCC_0,
   AF_PC9_ANALOG       =  0 | GM_ANA | 0x3900 | RCC_0,
   AF_PC9_SOFT_GND     =  0 | GM_GND | 0x3900 | RCC_0,
   AF_PC9_NC           =  0 | GM_NC  | 0x3900 | RCC_0
} pc9_enum;

typedef enum af_pc10_enum {
// AF_PC10_I2S3_SCK    =  6 | GM_AF  | 0x3A00 | RCC_???,
   AF_PC10_SPI3_SCK    =  6 | GM_AF  | 0x3A00 | RCC_SPI3,
   AF_PC10_UART4_TX    =  8 | GM_AF  | 0x3A00 | RCC_UART4,
   AF_PC10_USART3_TX   =  7 | GM_AF  | 0x3A00 | RCC_USART3,
   AF_PC10_INPUT       =  0 | GM_IN  | 0x3A00 | RCC_0,
   AF_PC10_OUTPUT      =  0 | GM_OUT | 0x3A00 | RCC_0,
   AF_PC10_ANALOG      =  0 | GM_ANA | 0x3A00 | RCC_0,
   AF_PC10_SOFT_GND    =  0 | GM_GND | 0x3A00 | RCC_0,
   AF_PC10_NC          =  0 | GM_NC  | 0x3A00 | RCC_0
} pc10_enum;

typedef enum af_pc11_enum {
   AF_PC11_CAN1_RX     =  9 | GM_AF  | 0x3B00 | RCC_CAN1,
   AF_PC11_SPI3_MISO   =  6 | GM_AF  | 0x3B00 | RCC_SPI3,
   AF_PC11_UART4_RX    =  8 | GM_AF  | 0x3B00 | RCC_UART4,
   AF_PC11_USART3_RX   =  7 | GM_AF  | 0x3B00 | RCC_USART3,
   AF_PC11_INPUT       =  0 | GM_IN  | 0x3B00 | RCC_0,
   AF_PC11_OUTPUT      =  0 | GM_OUT | 0x3B00 | RCC_0,
   AF_PC11_ANALOG      =  0 | GM_ANA | 0x3B00 | RCC_0,
   AF_PC11_SOFT_GND    =  0 | GM_GND | 0x3B00 | RCC_0,
   AF_PC11_NC          =  0 | GM_NC  | 0x3B00 | RCC_0
} pc11_enum;

typedef enum af_pc12_enum {
   AF_PC12_CAN1_TX     =  9 | GM_AF  | 0x3C00 | RCC_CAN1,
// AF_PC12_I2S3_SD     =  6 | GM_AF  | 0x3C00 | RCC_???,
   AF_PC12_SPI3_MOSI   =  6 | GM_AF  | 0x3C00 | RCC_SPI3,
   AF_PC12_UART5_TX    =  8 | GM_AF  | 0x3C00 | RCC_UART5,
   AF_PC12_USART3_CK   =  7 | GM_AF  | 0x3C00 | RCC_USART3,
   AF_PC12_INPUT       =  0 | GM_IN  | 0x3C00 | RCC_0,
   AF_PC12_OUTPUT      =  0 | GM_OUT | 0x3C00 | RCC_0,
   AF_PC12_ANALOG      =  0 | GM_ANA | 0x3C00 | RCC_0,
   AF_PC12_SOFT_GND    =  0 | GM_GND | 0x3C00 | RCC_0,
   AF_PC12_NC          =  0 | GM_NC  | 0x3C00 | RCC_0
} pc12_enum;

typedef enum af_pc13_enum {
   AF_PC13_INPUT       =  0 | GM_IN  | 0x3D00 | RCC_0,
   AF_PC13_OUTPUT      =  0 | GM_OUT | 0x3D00 | RCC_0,
   AF_PC13_ANALOG      =  0 | GM_ANA | 0x3D00 | RCC_0,
   AF_PC13_SOFT_GND    =  0 | GM_GND | 0x3D00 | RCC_0,
   AF_PC13_NC          =  0 | GM_NC  | 0x3D00 | RCC_0
} pc13_enum;

typedef enum af_pc14_enum {
   AF_PC14_INPUT       =  0 | GM_IN  | 0x3E00 | RCC_0,
   AF_PC14_OUTPUT      =  0 | GM_OUT | 0x3E00 | RCC_0,
   AF_PC14_ANALOG      =  0 | GM_ANA | 0x3E00 | RCC_0,
   AF_PC14_SOFT_GND    =  0 | GM_GND | 0x3E00 | RCC_0,
   AF_PC14_NC          =  0 | GM_NC  | 0x3E00 | RCC_0
} pc14_enum;

typedef enum af_pc15_enum {
   AF_PC15_INPUT       =  0 | GM_IN  | 0x3F00 | RCC_0,
   AF_PC15_OUTPUT      =  0 | GM_OUT | 0x3F00 | RCC_0,
   AF_PC15_ANALOG      =  0 | GM_ANA | 0x3F00 | RCC_0,
   AF_PC15_SOFT_GND    =  0 | GM_GND | 0x3F00 | RCC_0,
   AF_PC15_NC          =  0 | GM_NC  | 0x3F00 | RCC_0
} pc15_enum;

typedef enum af_pd2_enum {
   AF_PD2_TIM3_ETR     =  2 | GM_AF  | 0x4200 | RCC_TIM3,
   AF_PD2_UART5_RX     =  8 | GM_AF  | 0x4200 | RCC_UART5,
   AF_PD2_INPUT        =  0 | GM_IN  | 0x4200 | RCC_0,
   AF_PD2_OUTPUT       =  0 | GM_OUT | 0x4200 | RCC_0,
   AF_PD2_ANALOG       =  0 | GM_ANA | 0x4200 | RCC_0,
   AF_PD2_SOFT_GND     =  0 | GM_GND | 0x4200 | RCC_0,
   AF_PD2_NC           =  0 | GM_NC  | 0x4200 | RCC_0
} pd2_enum;

typedef enum af_ph0_enum {
   AF_PH0_INPUT        =  0 | GM_IN  | 0x8000 | RCC_0,
   AF_PH0_OUTPUT       =  0 | GM_OUT | 0x8000 | RCC_0,
   AF_PH0_ANALOG       =  0 | GM_ANA | 0x8000 | RCC_0,
   AF_PH0_SOFT_GND     =  0 | GM_GND | 0x8000 | RCC_0,
   AF_PH0_NC           =  0 | GM_NC  | 0x8000 | RCC_0
} ph0_enum;

typedef enum af_ph1_enum {
   AF_PH1_INPUT        =  0 | GM_IN  | 0x8100 | RCC_0,
   AF_PH1_OUTPUT       =  0 | GM_OUT | 0x8100 | RCC_0,
   AF_PH1_ANALOG       =  0 | GM_ANA | 0x8100 | RCC_0,
   AF_PH1_SOFT_GND     =  0 | GM_GND | 0x8100 | RCC_0,
   AF_PH1_NC           =  0 | GM_NC  | 0x8100 | RCC_0
} ph1_enum;

// AF_PD0_CAN1_RX      =  9 | GM_AF  | 0x4000 | RCC_,
// AF_PD1_CAN1_TX      =  9 | GM_AF  | 0x4100 | RCC_,
// AF_PD3_USART2_CTS   =  7 | GM_AF  | 0x4300 | RCC_,
// AF_PD4_USART2_RTS   =  7 | GM_AF  | 0x4400 | RCC_,
// AF_PD5_USART2_TX    =  7 | GM_AF  | 0x4500 | RCC_,
// AF_PD6_USART2_RX    =  7 | GM_AF  | 0x4600 | RCC_,
// AF_PD7_USART2_CK    =  7 | GM_AF  | 0x4700 | RCC_,
// AF_PD8_USART3_TX    =  7 | GM_AF  | 0x4800 | RCC_,
// AF_PD9_USART3_RX    =  7 | GM_AF  | 0x4900 | RCC_,
// AF_PD10_USART3_CK   =  7 | GM_AF  | 0x4A00 | RCC_,
// AF_PD11_USART3_CTS  =  7 | GM_AF  | 0x4B00 | RCC_,
// AF_PD12_TIM4_CH1    =  2 | GM_AF  | 0x4C00 | RCC_,
// AF_PD12_USART3_RTS  =  7 | GM_AF  | 0x4C00 | RCC_,
// AF_PD13_TIM4_CH2    =  2 | GM_AF  | 0x4D00 | RCC_,
// AF_PD14_TIM4_CH3    =  2 | GM_AF  | 0x4E00 | RCC_,
// AF_PD15_TIM4_CH4    =  2 | GM_AF  | 0x4F00 | RCC_,
// AF_PE0_TIM4_ETR     =  2 | GM_AF  | 0x5000 | RCC_,
// AF_PE5_TIM9_CH1     =  3 | GM_AF  | 0x5500 | RCC_,
// AF_PE6_TIM9_CH2     =  3 | GM_AF  | 0x5600 | RCC_,
// AF_PE7_TIM1_ETR     =  1 | GM_AF  | 0x5700 | RCC_,
// AF_PE8_TIM1_CH1N    =  1 | GM_AF  | 0x5800 | RCC_,
// AF_PE9_TIM1_CH1     =  1 | GM_AF  | 0x5900 | RCC_,
// AF_PE10_TIM1_CH2N   =  1 | GM_AF  | 0x5A00 | RCC_,
// AF_PE11_TIM1_CH2    =  1 | GM_AF  | 0x5B00 | RCC_,
// AF_PE12_TIM1_CH3N   =  1 | GM_AF  | 0x5C00 | RCC_,
// AF_PE13_TIM1_CH3    =  1 | GM_AF  | 0x5D00 | RCC_,
// AF_PE14_TIM1_CH4    =  1 | GM_AF  | 0x5E00 | RCC_,
// AF_PE15_TIM1_BKIN   =  1 | GM_AF  | 0x5F00 | RCC_,
// to be continued

