// image-header.h
#pragma once
#include <stddef.h>
#include <stdint.h>

static const uint32_t DEC_MAGIC = 0xDEC11780;

typedef enum __attribute__ ((packed)) image_type_enum {
   IT_UNDEFINED,
   IT_BOOTLOADER,
   IT_SYSLIB,
   IT_APP,
   IT_DATA,
   IT_SHM
} image_type_enum;

typedef void (isr_type)(void);  // interrupt service routine
typedef void (crt0_type)(int);  // startup routines
typedef int  (crt1_type)(int);  //

typedef struct image_struct {
   uint32_t         *top_of_stack;        // 00  0  initial SP
   union {
      isr_type      *crt_boot;            // 04  1  initial PC bootloader
      crt0_type     *crt0;                // 04  1  initial PC syslib
      crt1_type     *crt1;                // 04  1  initial PC programs
   };
   isr_type         *css_nmi;             // 08  2  NMI/CSS
   isr_type         *hard_fault;          // 0c  3  hard fault
   char              name[8];             // 10
   char              version_string[8];   // 18  z.B. "2017za"  "V03.07"
   uint8_t           verno;               // 20  z.B. 3
   uint8_t           revno;               // 21  z.B. 7
   int8_t            arg;                 // 22  poor mans *argv[]
   image_type_enum   image_type;          // 23  shared mem, app, ...
   uint32_t          source_date_epoch;   // 24  }
   uint32_t          syslib_version;      // 28  } display as UTC
   uint32_t          shm_version;         // 2c  }
   void             *end;                 // 30  last byte of text segment
   uint32_t          crc;                 // 34
   const char       *syslog_text;         // 38
   const void       *const_start;         // 3c  (systick)
   uint32_t         *ram_start;           // 40
   void             *bss_end;             // 44
   const void       *text_start;          // 48
   uint16_t          start_delay;         // 4c  ms
   uint8_t           privilege;           // 4e  1 = privileged
   int8_t            prio;                // 4f  priority 1 .. 99
} image_struct;
# ifndef S_SPLINT_S
_Static_assert (sizeof(image_struct) == 80, " Bad struct size");
#endif

typedef struct vector_struct {
  uint32_t        *top_of_stack;        // 00   0  initial SP  } not used, see
  crt0_type       *crt0;                // 04   1  initial PC  } image_struct
  isr_type        *css_nmi;             // 08   2  NMI = clock failure
  isr_type        *hard_fault;          // 0c   3
  isr_type        *mem_fault;           // 10   4
  isr_type        *bus_fault;           // 14   5
  isr_type        *usage_fault;         // 18   6
  isr_type        *rsvd_7;              // 1c   7
  isr_type        *rsvd_8;              // 20   8
  isr_type        *rsvd_9;              // 24   9
  isr_type        *rsvd_10;             // 28  10
  isr_type        *svc;                 // 2c  11
  isr_type        *debug;               // 30  12
  isr_type        *rsvd_13;             // 34  13
  isr_type        *pendsv;              // 38  14
  isr_type        *systick;             // 3c  15
  isr_type        *interrupts[128-16];
} vector_struct;
# ifndef S_SPLINT_S
_Static_assert (sizeof(vector_struct) == 512, " Bad struct size");
#endif

