// mps1401.h
#pragma once

#ifndef PIN
#define PIN(port,bit,signal,function,type,speed,pull,data) \
static const int P##port##_##signal = (1 << bit);
#endif

//  F205 I/O Pins
//
//  *1) PC9 treibt einen FET ohne Pull-Down aber mit Open Drain und
//      internem Pull-Up. Das ergibt mit der FET-Gate-Kapazitt ein
//      runderes Signal am Buzzer. Nach dem Reset ist das erstmal ein
//      GPIO, beep() schaltet dann auf AF_TIM3 um. Deshalb schaltet
//      gpio_init() keinen Takt ein. Das macht aber nichts, beep()
//      macht das sowieso selbst; es soll nmlich mglichst autark
//      sein. Auerdem benutzt es zwei Timer und DMA.
//
//  *2) PB2 ist whrend Reset BOOT1 mit externem Pull-Down,
//      danach Reset-Ausgang zum PCA9538. Der Eingang vom PCA9538
//      ist netterweise aktiv Low.
//
//  *3) I2C geht per Software einfacher als mit /dieser/ Hardware.
//      Der erste Versuch hat gleich funktioniert und was 160 Byte
//      kleiner als der schlecht funktionierende Hardware-Treiber...


//  Port Bit Signal     Function  Type Speed   Pull Data    Pin
//                                                        //  1 VBAT = VDD
PIN ( C, 13, SERVICE_L, INPUT,     pp, 0,      0,   0   ) //  2
PIN ( C, 14, EXTAL32,   INPUT,     pp, 0,      0,   0   ) //  3 1Hz from RTC
//    C, 15                                               //  4 n.c.
PIN ( H,  0, EXTAL,     INPUT,     pp, 0,      DOWN,0   ) //  5
PIN ( H,  1, XTAL,      INPUT,     pp, 0,      DOWN,0   ) //  6
//                                                        //  7 RESET_L
PIN ( C,  0, VS_USV,    ANALOG,    pp, 0,      0,   0   ) //  8 ADC123 Ch.10
PIN ( C,  1, CS_24V,    ANALOG,    pp, 0,      0,   0   ) //  9 ADC123 Ch.11
PIN ( C,  2, VS_B1_24V, ANALOG,    pp, 0,      0,   0   ) // 10 ADC123 Ch.12
PIN ( C,  3, VS_B1_5V,  ANALOG,    pp, 0,      0,   0   ) // 11 ADC123 Ch.13
//                                                        // 12 VSSA = GND
//                                                        // 13 VDDA
PIN ( A,  0, WAKEUP,    INPUT,     pp, 0,      0,   0   ) // 14 Alarm from RTC
PIN ( A,  1, VS_6V,     ANALOG,    pp, 0,      0,   0   ) // 15 ADC123 Ch.1
PIN ( A,  2, U2_TXD,    USART2_TX, pp, 0,      0,   LOW ) // 16
PIN ( A,  3, U2_RXD,    USART2_RX, pp, 0,      0,   0   ) // 17
//                                                        // 18 GND
//                                                        // 19 VDD
PIN ( A,  4, LEDOFF_L,  ANALOG,    pp, 0,      0,   0   ) // 20 DAC1
PIN ( A,  5, M25_SCK,   SPI1_SCK,  pp, MEDIUM, 0,   0   ) // 21 SCK
PIN ( A,  6, M25_Q,     SPI1_MISO, pp, MEDIUM, 0,   0   ) // 22 MISO
PIN ( A,  7, M25_D,     SPI1_MOSI, pp, MEDIUM, 0,   0   ) // 23 MOSI
PIN ( C,  4, AIN_B,     ANALOG,    pp, 0,      0,   0   ) // 24 ADC12 Ch.14
PIN ( C,  5, AIN_A,     ANALOG,    pp, 0,      0,   0   ) // 25 ADC12 Ch.15
PIN ( B,  0, VS_24V,    ANALOG,    pp, 0,      0,   0   ) // 26 ADC12 Ch.8
PIN ( B,  1, VS_MO_24V, ANALOG,    pp, 0,      0,   0   ) // 27 ADC12 Ch.9
PIN ( B,  2, RST9538_L, OUTPUT,    pp, 0,      0,   LOW ) // 28 BOOT1 *2)
PIN ( B, 10, U3_TXD,    USART3_TX, pp, 0,      0,   0   ) // 29
PIN ( B, 11, U3_RXD,    USART3_RX, pp, 0,      0,   0   ) // 30
//                                                        // 31 VCAP1
//                                                        // 32 VDD
PIN ( B, 12, CAN_RX,    CAN2_RX,   pp, 0,      0,   0   ) // 33
PIN ( B, 13, CAN_TX,    CAN2_TX,   pp, 0,      0,   0   ) // 34
PIN ( B, 14, MO_RTS_L,  OUTPUT,    OD, 0,      0,   HIGH) // 35
PIN ( B, 15, MO_CTS_L,  INPUT,     pp, 0,      0,   0   ) // 36
PIN ( C,  6, MO_TXD,    USART6_TX, OD, 0,      0,   HIGH) // 37
PIN ( C,  7, MO_RXD,    USART6_RX, pp, 0,      0,   0   ) // 38
PIN ( C,  8, U4_RTS_H,  OUTPUT,    pp, 0,      0,   LOW ) // 39
PIN ( C,  9, 4KHZ,      OUTPUT,    OD, 0,      UP,  0   ) // 40 buzzer *1)
PIN ( A,  8, U3_RTS_H,  OUTPUT,    pp, 0,      0,   LOW ) // 41
PIN ( A,  9, USB_TXD,   USART1_TX, pp, 0,      0,   0   ) // 42
PIN ( A, 10, USB_RXD,   USART1_RX, pp, 0,      0,   0   ) // 43
PIN ( A, 11, U2_RTS_H,  OUTPUT,    pp, 0,      0,   LOW ) // 44
PIN ( A, 12, U5_RTS_H,  OUTPUT,    pp, 0,      0,   LOW ) // 45
PIN ( A, 13, SWDIO,     INPUT,     pp, FAST,   UP,  0   ) // 46
//                                                        // 47 VCAP2
//                                                        // 48 VDD
PIN ( A, 14, SWCLK,     INPUT,     pp, FAST,   DOWN,0   ) // 49
PIN ( A, 15, M25_SS_L,  SPI1_NSS,  pp, MEDIUM, 0,   0   ) // 50 NSS
PIN ( C, 10, U4_TXD,    UART4_TX,  pp, 0,      0,   0   ) // 51
PIN ( C, 11, U4_RXD,    UART4_RX,  pp, 0,      0,   0   ) // 52
PIN ( C, 12, U5_TXD,    UART5_TX,  pp, 0,      0,   LOW ) // 53
PIN ( D,  2, U5_RXD,    UART5_RX,  pp, 0,      0,   0   ) // 54
PIN ( B,  3, TRACESWO,  INPUT,     pp, FAST,   UP,  0   ) // 55
PIN ( B,  4, M25_SSA_H, OUTPUT,    pp, MEDIUM, 0,   0   ) // 56 chip select
PIN ( B,  5, M25_SSB_H, OUTPUT,    pp, MEDIUM, 0,   0   ) // 57 chip select
PIN ( B,  6, MO_DTR_L,  OUTPUT,    OD, 0,      0,   HIGH) // 58
PIN ( B,  7, MO_DCD_L,  INPUT,     pp, 0,      0,   0   ) // 59
//                                                        // 60 BOOT0
PIN ( B,  8, SCL,       INPUT,     OD, 0,      0,   0   ) // 61 } *3)
PIN ( B,  9, SDA,       INPUT,     OD, 0,      0,   0   ) // 62 }
//                                                        // 63 GND
//                                                        // 64 VDD

