// syslib.h
#pragma once
#include <stdarg.h>
#include <stddef.h>
#include <stdint.h>

typedef enum __attribute__ ((packed)) locale_bits_enum {
   LC_LANG_DE,
   LC_LANG_EN,
   LC_LANG_RU,
   LC_LANG_3,
   LC_LANG_4,
   LC_LANG_5,
   LC_LANG_6,
   LC_LANG_7,
   LC_LANG_MASK = 4 + 2 + 1
} locale_bits_enum;

typedef enum __attribute__ ((packed)) control_char_enum {
   CC_NUL,     //  0  @  nul
   CC_SOH,     //  1  A  soh
   CC_STX,     //  2  B  stx
   CC_CTRLC,   //  3  C  etx
   CC_EOT,     //  4  D  eot
   CC_ENQ,     //  5  E  enq
   CC_ACK,     //  6  F  ack
   CC_BEL,     //  7  G  bel
   CC_BS,      //  8  H  bs
   CC_HT,      //  9  I  ht
   CC_LF,      // 10  J  lf
   CC_VT,      // 11  K  vt
   CC_FF,      // 12  L  ff
   CC_CR,      // 13  M  cr
   CC_SO,      // 14  N  so
   CC_SI,      // 15  O  si
   CC_DLE,     // 16  P  dle
   CC_XON,     // 17  Q  xon
   CC_LED_OFF, // 18  R  dc2
   CC_XOFF,    // 19  S  xoff
   CC_LED_RED, // 20  T  dc4
   CC_NAK,     // 21  U  nak
   CC_SYN,     // 22  V  syn
   CC_ETB,     // 23  W  etb
   CC_CAN,     // 24  X  can
   CC_EM,      // 25  Y  em
   CC_SUB,     // 26  Z  sub
   CC_ESC,     // 27  [  esc
   CC_FS,      // 28  \  fs
   CC_GS,      // 29  ]  gs
   CC_RS,      // 30  ^  rs
   CC_US,      // 31  _  us
   CC_DEL      = 127,
   CC_APC      = 9 * 16 + 15, // Application Command = Anfang der DAZ-Daten
   CC_ST       = 9 * 16 + 12, // String Terminator = Ende der DAZ-Daten
   CC_CUU      = 'A' + 128,   //  esc [ A  Cursor up
   CC_CUD      = 'B' + 128,   //  esc [ B  Cursor down
   CC_CUF      = 'C' + 128,   //  esc [ C  Cursor forward
   CC_CUB      = 'D' + 128,   //  esc [ D  Cursor back
   CC_PF1      = 'P' + 128,   //  esc O P  Bildschirm ein
   CC_PF2      = 'Q' + 128,   //  esc O Q
   CC_PF3      = 'R' + 128,   //  esc O R
   CC_PF4      = 'S' + 128,   //  esc O S  '*', Funktion whlen
   CC_TIMEOUT  = 'T' + 128
} control_char_enum;

typedef enum rtc_flags_enum {
   RTC_BLF            = 2,
   RTC_OSF            = 24,
   RTC_BOOT_FLAGS_SET = 25,
   RTC_ALL_FLAGS      = 26,
   RTC_TRIM           = 27,
   RTC_TIME           = 28
} rtc_flags_enum;

typedef enum reset_flags_enum {
   RESET_RMVF,       // BF_RESET_FIREWALL
   RESET_BROWNOUT,   // BF_RESET_OBL
   RESET_PIN,
   RESET_POWER_ON,   // BF_RESET_BROWNOUT
   RESET_SOFT,
   RESET_I_WATCHDOG,
   RESET_W_WATCHDOG,
   RESET_LOW_POWER
} reset_flags_enum;

typedef enum __attribute__ ((packed)) mode_switch_enum {
   MODE_SWITCH_START,
   MODE_SWITCH_A,
   MODE_SWITCH_B,
   MODE_SWITCH_STANDBY,
   MODE_SWITCH_BHB,
   MODE_SWITCH_SERVICE,
   MODE_SWITCH_DAUERTEST,
   MODE_SWITCH_UPDATE,
   MODE_SWITCH_SHUTDOWN
} mode_switch_enum;

typedef enum __attribute__ ((packed)) task_enum {
   TS_null,
   TS_crc,
   TS_old,
   TS_old_lib,
   TS_old_shm,
   TS_terminated,
   TS_pid_valid,
   TS_loaded,
   TS_ready,
   TS_running,
   TS_timer,
   TS_blocked
} task_enum;

static const char *ts_text[] = {
   [TS_null]       = "NULL  ",
   [TS_crc]        = "CRC   ",
   [TS_old]        = "OLD   ",
   [TS_old_lib]    = "OLDLIB",
   [TS_old_shm]    = "OLDSHM",
   [TS_terminated] = "term. ",
   [TS_pid_valid]  = "PID ? ",
   [TS_loaded]     = "loaded",
   [TS_ready]      = "ready ",
   [TS_running]    = "run   ",
   [TS_timer]      = "timer ",
   [TS_blocked]    = "block "
};
_Static_assert (sizeof ts_text / sizeof ts_text[0] == 12, "ts_text");

typedef struct taskinfo_struct {
   char      name[8];
   char      version[8];
   uint32_t  source_date_epoch;
   uint8_t   stdio;
   task_enum state;
   int16_t   rsvd;
} taskinfo_struct;

typedef struct stdio_struct {
   uint8_t  rxbuf[35];
   uint8_t  get;
   uint8_t  put;
   int8_t   esc_state;
   int8_t   channel;
   int8_t   pid;
   int    (*putchar) (int);
} stdio_struct;
static const int UART_RXBUF_SIZE = sizeof (((stdio_struct *)0)->rxbuf);

typedef struct task_struct {
   struct task_struct  *next;
   struct image_struct *image;
   stdio_struct        *stdio;
   uint32_t  volatile  *sp;
   uint8_t              lc_lang       : 3;
   uint8_t              lc_fahrenheit : 1;
   uint8_t              lc_am_pm      : 1;
   uint8_t              stackalarm    : 1;
   uint8_t              rsvd_bit_a    : 1;
   uint8_t              rsvd_bit_b    : 1;
   uint8_t              tz;         // timezone index
   uint8_t              tpl;        // thread privilege level, 1 = unprivileged
   uint8_t              pid;
   task_enum volatile   state;
   uint8_t              work;       // time slot - ticks running   } duty
   uint8_t              doze;       //           - ticks suspended } cycle
   uint8_t   volatile   slot;       //           - down counter [systicks]
   uint32_t  volatile   timer;      // wakeup time [ms]
   uint32_t             rbar_text;  // address & region no.
   uint32_t             rasr_text;  // size & attributes
   uint32_t             rbar_data;  // address & region no.
   uint32_t             rasr_data;  // size & attributes
   uint32_t             cycles;     // HCLK cycles
} task_struct;
# ifndef S_SPLINT_S
_Static_assert (sizeof (task_struct) == 48, " Bad struct size");
#endif

typedef struct clk_struct {
   int  hse;
   int  hsi;
   int  hclk;
   int  hclk_MHz;
   int  ahb1;
   int  apb1;
   int  apb2;
   int  delta;
   int  hsitrim;
} clk_struct;

typedef struct tm {
   int  tm_sec;
   int  tm_min;
   int  tm_hour;
   int  tm_mday;
   int  tm_mon;
   int  tm_year;
   int  tm_wday;
   int  tm_yday;
   int  tm_isdst;
} tm;

typedef struct syslog_struct {
   uint8_t   id;
   uint8_t   unit;  // wird erst von syslog_get ausgefllt
   uint16_t  msec;
   uint32_t  sec;
   int       p32;
   int       p16;
} syslog_struct;
# ifndef S_SPLINT_S
_Static_assert (sizeof (syslog_struct) == 16, " Bad struct size");
#endif


// id muss immer zweistellig geschrieben werden, also z.B.
// SYSLOG (01, "Foo", 0, 0);

#define SYSLOG(id,txt,p1,p2) \
   while (0); \
   static const char __attribute__((section(".syslog"),used)) \
                     SYSLOG_TXT_##id[] = #id txt; \
   syslog (id, p1, p2)

#define SYSLOG_SYS(id,txt,p1,p2) \
   while (0); \
   static const char __attribute__((section(".syslog"),used)) \
                     SYSLOG_TXT_##id[] = #id txt; \
   do_syslog (id, p1, p2)


// libcdp
//
// OBACHT: die folgenden Zeilen werden von tools/lib_tables verarbeitet

typedef enum __attribute__ ((packed)) svc_enum {
   SVC_NULL,
   SVC_EXIT,
   SVC_GET_TASKINFO,
   SVC_GOD_MODE,
   SVC_KILL,
   SVC_NICE,
   SVC_SET_MODE_SWITCH,
   SVC_SLEEP_MS,
   SVC_SLEEP_UNTIL,
   SVC_SYSLOG,
   SVC_SYS_REBOOT,
   SVC_TASK_CLONE,
   SVC_TASK_SET_LOCALE,
   SVC_TASK_SET_TZ,
   SVC_TASK_WAKEUP,
   SVC_TRACE,
   SVC_UART_INIT,
   SVC_UTC_SLEW,
   SVC_UTC_STEP
} svc_enum;

int            abs (int i);
void           adc_init (void);
int            adc_read (int channel);
int            atoi (const char *text);
void           beep (int mHz, int ms);
void           beep_again (void);
int            boot_flag (int what);
void           clear_screen (void);
void           delay_ms (uint32_t ms);
void           delay_us (uint16_t us);
int            debug_printf (const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
void           exit (int status) __attribute__ ((noreturn));
int            get_24V (void);
clk_struct    *get_clocks (void);
void          *get_const_ptr (const char *taskname);
uint32_t       get_mode_switch (void);
int            get_slew (void);
int            get_taskinfo (int pid, taskinfo_struct *info);
uint32_t       get_task_timer (void);
int            get_ticks (void);
uint32_t       get_uptime (void);
int            getchar (int ms);
int            get_lc_fahrenheit (void);
int            get_lc_lang (void);
int            getpid (void);
int            gettz (void);
void           gmtime (uint32_t t, int *y, int *m, int *d, int *H, int *M, int *S);
int            god_mode (int onoff);
int            i2c_init (void);
int            i2c_output (int magic, int signals, int on);
int            i2c_receive (int address, uint8_t *rxbuf, int rxcount);
int            i2c_send_1 (int address, uint8_t tx1);
int            i2c_send_2 (int address, uint8_t tx1, uint8_t tx2);
int            i2c_send_x (int address, uint8_t *txbuf, int txcount);
int            kill (int pid);
void           led (int brightness);
uint32_t       local_realtime (void);
void          *memset (void *s, int c, int n);
uint32_t       mktime (struct tm *tm);
void           mpu_show (void);
int            nice (int inc);
int            one_wire_read_eeprom (int row, uint8_t *data, int rowsize);
char          *one_wire_serialno (uint8_t *rxbuf);
int            one_wire_write_eeprom (int row, uint8_t *data, int rowsize);
int            pidof (const char *taskname);
void           power_init (void);
void           print_ascii (int width, const char *text);
int            print_cpuid (void);
int            print_u_id (void);
int            printf (const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
int            printftime (int tz, uint32_t utc, const char *fmt);
int            putchar (int c);
int            puts (const char *text);
int            rtc_print_flags (int flags);
uint32_t       rtc_read (int what);
int            rtc_set (uint32_t utc);
int            rtc_trim (int aging_offset);
int            set_mode_switch (uint32_t request);
void           show_tasks (int pid);
int            sleep_ms (int ms);
int            sleep_until (uint32_t ms);
int            strcmp (const char *s1, const char *s2);
int            strlen (const char *s);
int            syslog (int event_id, int p32, int p16);
int            syslog_get (syslog_struct *event, syslog_struct **index);
syslog_struct *syslog_get_first (int direction);
char          *syslog_get_text (size_t id);
int            sys_reboot (void);
int            task_clone (struct image_struct *child);
int            task_set_locale (int lang, int fahrenheit, int am_pm);
int            task_set_tz (int timezone);
int            task_wakeup (int pid);
int            timezone_offset (uint32_t utc);
int            toupper (int c);
void           trace (void);
int            tz_num_zones (void);
int            tz_match_id (int tz_id);
int            uart_init (int channel, int baud, stdio_struct *buf);
uint32_t       utc_monotonic (void);
int            utc_slew (uint32_t utc);
int            utc_step (uint32_t utc);
int            vprintf (const char *fmt, va_list ap);

