/* memory-ram.ld -- linker script F205RC ( 80 + 16K RAM, 256K Flash)
                                  F205RE (112 + 16K RAM, 512K Flash)

// static const uint32_t  sectorlist_F205[] = {
//    0x08000000,  //  0   16K = application boot loader (flash)
//    0x08004000,  //  1   16K
//    0x08008000,  //  2   16K
//    0x0800C000,  //  3   16K
//    0x08010000,  //  4   64K __ F205B, 128K (48+16K RAM = zu wenig)
//    0x08020000,  //  5  128K ___ F205C, 256K (80+16K RAM)
//    0x08040000,  //  6  128K ____ F205D, 384K (den gibt es nicht)
//    0x08060000,  //  7  128K _____ F205E, 512K (128K RAM)
//    0x08080000,  //  8  128K
//    0x080A0000,  //  9  128K ______ F205F, 768K
//    0x080C0000,  // 10  128K
//    0x080E0000,  // 11  128K _______ F205G, 1024K
//    0x20000000,  //       8K = system memory boot loader (ram)
//    0x20002000,  //       4K }
//    0x20003000,  //       4K } = 72K application
//        ...                  }
//    0x20013000,  //       4K }
//    0x20014000,  //      32K  ???
//    0x2001C000,  //      16K RAM

   Die Reihenfolge im Flash sollte so sortiert werden, dass die MPU Regions
   nicht zu gross werden. Fr den Bereich 2000f000, 8K ist die Region 128K
   gross und kann per SRD oben und unten nur um Vielfache von 16K verkleinert
   werden. Das ergibt dan 2000c000..20014000.

   Eine Region mit 32K kann per SRD um 4K verkleinert werden, das passt gut,
   weil Flash nicht feiner als 4K unterteilt werden sollte.

   Die Region 1 (ro data) muss control, data und tzdata abdecken. Dafr gibt
   es die Symbole shm_ro und shm_end. Fr ein gemischtes Flash/RAM-System
   muss die Region dann von 08000000 bis 20020000 gehen.
*/

MEMORY
{
/* st_reserved   (r)  : ORIGIN = 0x20000000, LENGTH = 8K */
   flash_boot    (rx) : ORIGIN = 0x08000000, LENGTH = 16K
   flash_tzdata  (rx) : ORIGIN = 0x20000000, LENGTH = 8K
   flash_vdrive  (rx) : ORIGIN = 0x20002000, LENGTH = 24K
   flash_syslib  (rx) : ORIGIN = 0x20008000, LENGTH = 20K
/*   flash_display (rx) : ORIGIN = 0x20008000, LENGTH = 40K */
   flash_control (rx) : ORIGIN = 0x20012000, LENGTH = 4K
   flash_init    (rx) : ORIGIN = 0x20013000, LENGTH = 12K

   no_go_area    (r)  : ORIGIN = 0x20016000, LENGTH = 24K

   ram_boot      (rw) : ORIGIN = 0x2001c000, LENGTH = 1K
   ram_syslib    (rw) : ORIGIN = 0x2001c800, LENGTH = 4K
   ram_init      (rw) : ORIGIN = 0x2001d800, LENGTH = 2K
   ram_control   (rw) : ORIGIN = 0x2001e000, LENGTH = 1K
   ram_modem     (rw) : ORIGIN = 0x2001e400, LENGTH = 1K
   ram_display   (rw) : ORIGIN = 0x2001e800, LENGTH = 5K
   ram_data      (rw) : ORIGIN = 0x2001fc00, LENGTH = 1K
}

   shm           = ORIGIN(ram_data)     + 0;
   tos_idle_task = ORIGIN(ram_boot)     + 512;

/* shm_ro        = ORIGIN(flash_init)   + 0;
   shm_end       = ORIGIN(flash_tzdata) + LENGTH(flash_tzdata);
*/

