// find-ftdi.c

#define _POSIX_C_SOURCE 200809L // readlink, symlink
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "fail.h"

static const char *const TTY_CLASS = "/sys/class/tty";
static const char *const VISASCOPE = "/dev/serial/by-id/"
                                     "usb-FTDI_Visatronic_-_VISASCOPE";

int
main (void)
{
   struct stat    driver_stat;
   struct dirent *link;
   DIR           *tty_dir;
   char          *s1, path[512], dev_dir[512], product[88];
   int            pfd, plen;
   int            debug = 0;

   if (stat ("/sys/bus/usb-serial/drivers/ftdi_sio", &driver_stat)) {
      system ("/sbin/modprobe ftdi_sio");
      sleep (1);
   }
   if (chdir (TTY_CLASS)) {
      fail (errno, "chdir %s", TTY_CLASS);
   }
   tty_dir = opendir (".");
   if (tty_dir == NULL) {
      fail (errno, "opendir %s", TTY_CLASS);
   }

   while ((link = readdir (tty_dir))) {
      if (strncmp (link->d_name, "ttyUSB", 6)) {
         continue;
      }
      if (readlink (link->d_name, dev_dir, sizeof dev_dir - 1) < 7) {
         continue;
      }
      while ((s1 = strrchr (dev_dir, '/'))) {
         *s1 = '\0';
         snprintf (path, sizeof path, "%s/product", dev_dir);
         if (debug) {
            printf ("%s\n", path);
         }
         if ((pfd = open (path, O_RDONLY)) == -1) {
            continue;
         }
         plen = read (pfd, product, sizeof product);
         if (plen != -1) {
            close (pfd);
            product[plen] = '\0';
            s1 = strchr (product, '\n');
            if (s1) {
               *s1 = '\0';
            }
            if (debug) {
               printf ("*** %s ***\n", product);
            }
            if (strncmp (product, "Visatronic - VISASCOPE", 22) == 0) {
               snprintf (path, sizeof path, "/dev/%s", link->d_name);
               printf ("%s\n", path);
               if (chown (path, 0, 20)) {
                  fail (errno, "chown %s", path);
               }
               if (chmod (path, 0660)) {
                  fail (errno, "chmod %s", path);
               }
               mkdir ("/dev/serial", 0555);
               mkdir ("/dev/serial/by-id", 0555);
               if (symlink (path, VISASCOPE)) {
                  fail (errno, "symlink %s %s", path, VISASCOPE);
               }
            }
            if (strncmp (product, "FT232R USB UART", 15) == 0) {
               snprintf (path, sizeof path, "/dev/%s", link->d_name);
               if (chown (path, 0, 20)) {
                  fail (errno, "chown %s", path);
               }
               if (chmod (path, 0660)) {
                  fail (errno, "chmod %s", path);
               }
               printf ("%s\n", path);
            }
         }
      }
   }
   closedir (tty_dir);
   return 0;
}

