// libubl.h

#include <stdint.h>

static const int    DL_BAUD = 115200;
static char * const DL_FORMAT = "8E1";
static const int    TIME_OUT = 680; // ms

typedef enum ubl_command_enum {
  UBL_GET,
  UBL_GET_VERSION,
  UBL_GET_CHIP_ID,
  UBL_READ         = 0x11,
  UBL_GO           = 0x21,
  UBL_WRITE        = 0x31,
  UBL_ERASE        = 0x43,
  UBL_EXT_ERASE    = 0x44,
  UBL_WR_PROTECT   = 0x63,
  UBL_WR_UNPROTECT = 0x73,
  UBL_RD_PROTECT   = 0x82,
  UBL_RD_UNPROTECT = 0x92
} ubl_command_enum;

static char * const ubl_cmd_text[256] = {
  [UBL_GET]          = "get",
  [UBL_GET_VERSION]  = "get_version",
  [UBL_GET_CHIP_ID]  = "get_chip_id",
  [UBL_READ]         = "read",
  [UBL_GO]           = "go",
  [UBL_WRITE]        = "write",
  [UBL_ERASE]        = "erase",
  [UBL_EXT_ERASE]    = "extended_erase",
  [UBL_WR_PROTECT]   = "write_protect",
  [UBL_WR_UNPROTECT] = "write_unprotect",
  [UBL_RD_PROTECT]   = "readout_protect",
  [UBL_RD_UNPROTECT] = "readout_unprotect"
};

int   ubl_connect (int reset);
void  ubl_logc (int character);
void  ubl_log_open (void);
void  ubl_log_printf (const char *format, ...);
int   ubl_simple_cmd (int cmd);
int   ubl_pid (void);
int   ubl_read_memory (int s19, uint32_t start, int count);
int   ubl_read_row (uint32_t start, int buf[], int size);
int   ubl_write_page (uint32_t start, uint8_t *data);
int   ubl_write_word (uint32_t start, uint32_t data);
int   ubl_go (uint32_t start);

