// misc-tools.c

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include "misc-tools.h"
#include "fail.h"


void
html_print_iso5 (const char *iso)
{
   int  c1;

   for (/**/; *iso; iso++) {
      c1 = (unsigned char)*iso;
      if (c1 >= 127) {
         if (c1 < 160) {
            fail (0, "illegal character %d", c1);
         } else {
            printf ("&#%d;", (c1 - 160) + 1024);
         }
      } else {
         putchar (c1);
      }
   }
}


void
html_print_iso1 (const char *iso)
{
   int  c1;

   for (/**/; *iso; iso++) {
      c1 = (unsigned char)*iso;
      if (c1 >= 127) {
         if (c1 < 160) {
            fail (0, "illegal character %d", c1);
         } else {
            printf ("&#%d;", (c1 - 160) + 160);
         }
      } else {
         putchar (c1);
      }
   }
}


char *
chdir_version (const char *arg)
{
   static char  version[8];

   if (strlen (arg) > 6) {
      fail (0, "Version string too long");
   }
   strcpy (version, arg);
   version[0] = tolower (version[0]);
   if (chdir (version)) {
      chdir ("..");
      if (chdir (version)) {
         chdir ("..");
         if (chdir (version)) {
            fail (errno, "chdir %s:", version);
         }
      }
   }
   version[7] = '\0';
   version[6] = version[5];
   version[5] = version[4];
   version[4] = version[3];
   version[3] = '.';
   version[2] = version[2];
   version[1] = version[1];
   version[0] = toupper (version[0]);
   return version;
}


char *
iso_time (uint32_t utc)
{
   static char  date[32];
   time_t  tmp;

   if (utc) {
      tmp = utc;
   } else {
      tmp = time (NULL);
   }
   strftime (date, sizeof date, "%F %T UTC", gmtime (&tmp));
   return date;
}


char *
pp_mtime (const char *source)
{
   struct stat  buf;

   if (stat (source, &buf)) {
      fail (errno, "stat %s:", source);
   }
   return iso_time ((uint32_t)buf.st_mtime);
}


char
utf8_to_iso (const char *text, int *inc)
{
   uint32_t  u1, u2;
   char     *bp = (char *)text;
   int       bytes;

   u1 = (unsigned char)*bp++;
   if (u1 < 0x7f) {
      bytes = 1;
   } else {
      if (u1 > 0xf0) {
         u1 &= 7;
         bytes = 4;
      } else {
         if (u1 > 0xe0) {
            u1 &= 15;
            bytes = 3;
         } else {
            if (u1 > 0xc0) {
               u1 &= 31;
               bytes = 2;
            } else {
               return 0;
            }
         }
      }
   }
   *inc = bytes - 1;
   for (/**/; bytes > 1; bytes--) {
      u2 = (unsigned char)*bp++;
      if (u2 < 0x80 || u2 >= 0xc0) { return 0; }
      u1 = (u1 << 6) | u2;
   }
   switch (u1) {
   case 0xae:
      return '';
   case 0xc4:
      return '';
   case 0xd6:
      return '';
   case 0xdc:
      return '';
   case 0xe4:
      return '';
   case 0xf6:
      return '';
   case 0xfc:
      return '';
   case 0xdf:
      return '';
   default:
      fail (0, "unbekanntes utf-8 0x%x (bytes = %d)", u1, bytes);
   }
}


void
edit_line (char comment, char *buf)
{
   char  *s1, *s2;
   int    eol = 0;

   if (comment) {
      if (buf[0] == comment) {
         buf[0] = '\0';
         return;
      }
   }
   if (buf[0] == '\r') {
      for (s1 = buf; *s1; s1++) {
         s1[0] = s1[1];
      }
      s1[0] = '\0';
   }

   for (s1 = buf, s2 = buf; *s1 && eol == 0; s1++) {
      if (*s1 >= ' ' && *s1 <= '~') {
         *s2++ = *s1;
      } else {
         switch (*s1) {
         case '\t':
            *s2 = ' ';
            break;
         case '\r':
            *s2 = '\0';
            eol = 1;
            break;
         case '\n':
            *s2 = '\0';
            eol = 1;
            break;
         default:
//          utf = utf8_to_iso (s1, &inc);
//          if (utf) {
//             *s2++ = utf;
//             s1 += inc;
//             break;
//          }
            if ((unsigned char)*s1 >= 127 && (unsigned char)*s1 <= 160) {
               printf ("%s", buf);
               fail (0, "unerlaubtes Zeichen %d in Spalte %d",
                     (unsigned char)*s1, s1 - buf + 1);
            }
            *s2++ = *s1;
            break;
         }
      }
   }

   if (eol == 0) {
      fail (0, "kein '\\n'; Zeile zu lang?");
   }
   return;
}


void
html_print_css (void)
{
   printf ("<style TYPE='text/css'>\n");
   printf ("<!--\n"
           "body {\n"
           "  margin-top:0px;\n"
           "  padding:0;\n"
           "  background-color:#DDB;\n"
           "  font-family:sans-serif;\n"
           "  font-size:12px;\n"
           "  color:black;\n"
           "}\n"
           "img {\n"
           "  border:none;\n"
           "}\n"
           "div.box {\n"
           "  position:static;\n"
           "  margin:4px;\n"
           "  padding:6px;\n"
           "  border:solid thin black;\n"
           "  background-color:#CCCCCC;\n"
           "}\n"
           "h2 {\n"
           "  margin-top:7px;\n"
           "  margin-bottom:7px;\n"
           "  font-size:18px;\n"
           "}\n"
           "h3 {\n"
           "  margin-top:6px;\n"
           "  margin-bottom:6px;\n"
           "  font-size:16px;\n"
           "  color:black;\n"
           "}\n"
           "h4 {\n"
           "  margin-top:4px;\n"
           "  margin-bottom:4px;\n"
           "  font-size:14px;\n"
           "  color:black;\n"
           "}\n"
           "h5 {\n"
           "  margin-top:0px;\n"
           "  margin-bottom:0px;\n"
           "  font-size:12px;\n"
           "  color:#003399;\n"
           "}\n"
           "table.tabelle {\n"
           "  empty-cells:show;\n"
           "  background-color:#777777;\n"
           "  border:thin solid white;\n"
           "}\n"
           "tr.alt0 {\n"
           "  background-color:#CCCCCC;\n"
           "}\n"
           "tr.alt1 {\n"
           "  background-color:#E0E0E0;\n"
           "}\n"
           "tr.altw {\n"
           "  background-color:yellow;\n"
           "}\n"
           "th {\n"
           "  font-family:sans-serif;\n"
           "  font-size:12px;\n"
           "  padding-left:4px;\n"
           "  padding-right:4px;\n"
           "  background-color:#BBBBBB;\n"
           "}\n"
           "td {\n"
           "  font-family:sans-serif;\n"
           "  font-size:12px;\n"
           "  padding-left:4px;\n"
           "  padding-right:4px;\n"
           "  background-color:#EEEEEE;\n"
           "}\n"
           "td.char {\n"
           "  text-align:center;\n"
           "}\n"
           "td.int {\n"
           "  text-align:right;\n"
           "}\n"
           "span.hi {\n"
           "  background-color:#EEEE88;\n"
           "}\n"
           "-->\n"
           "</style>\n");
   return;
}


char *
http_date (time_t utc)
{
// Tue, 15 Nov 1994 08:12:31 GMT
   static char  *wday[7] = { "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun" };
   static char  *mon[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
                           };
   static char  buf[32];
   struct tm   *gmt;

   if (utc < 1000000000) {
      utc = time (NULL) + utc;
   }
   gmt = gmtime (&utc);
   snprintf (buf, sizeof buf, "%s, %02d %s %04d %02d:%02d:%02d GMT",
             wday[gmt->tm_wday], gmt->tm_mday, mon[gmt->tm_mon],
             gmt->tm_year + 1900, gmt->tm_hour, gmt->tm_min, gmt->tm_sec);
   return buf;
}


void
html_print_head (const char *title)
{
   char  *request, *server;

   server = getenv ("SERVER_SOFTWARE");
   request = getenv ("REQUEST_METHOD");
   if (request == NULL) {
      printf ("HTTP/1.0 500 Internal Server Error\r\n");
      printf ("Connection: close\r\n");
      exit (EXIT_SUCCESS);
   }
   if (strcmp (request, "GET") && strcmp (request, ("HEAD"))) {
      printf ("HTTP/1.0 501 Not Implemented\r\n");
      printf ("Connection: close\r\n");
      exit (EXIT_SUCCESS);
   }
   printf ("HTTP/1.0 200 OK\r\n");
   printf ("Connection: close\r\n");
   printf ("Content-Type: text/html\r\n");
   printf ("Date: %s\r\n", http_date (0));
   printf ("Expires: %s\r\n", http_date (7));
   if (server) {
      printf ("Server: %s\r\n", server);
   }
   printf ("\r\n");

   if (request && strcmp (request, "HEAD") == 0) {
      exit (EXIT_SUCCESS);
   }
   printf ("<!DOCTYPE html>\n");
   printf ("<html>\n");
   printf ("<head>\n");
   printf ("<meta charset='UTF-8'>\n");
// printf ("<meta charset='ISO-8859-1'>\n");
   printf ("<title>");
   html_print_iso1 (title);
   printf ("</title>\n");
   printf ("<link rel='stylesheet' media='screen' href='/html/screen.css'>\n");
   printf ("</head>\n");
   return;
}


void
ps_header (void)
{
   printf ("%%!PS-Adobe-3.0\n");
   printf ("%%BoundingBox: 0 0 595 842\n");
   printf ("\n");
   printf ("/Helvetica findfont\n");
   printf ("dup length dict begin\n");
   printf ("   { 1 index /FID ne\n");
   printf ("                {def}\n");
   printf ("                {pop pop}\n");
   printf ("            ifelse\n");
   printf ("   } forall\n");
   printf ("   /Encoding ISOLatin1Encoding def\n");
   printf ("   currentdict\n");
   printf ("end\n");
   printf ("/Helvetica-ISOLatin1 exch definefont pop\n");
   printf ("\n");
   printf ("/Helvetica-Bold findfont\n");
   printf ("dup length dict begin\n");
   printf ("   { 1 index /FID ne\n");
   printf ("                {def}\n");
   printf ("                {pop pop}\n");
   printf ("            ifelse\n");
   printf ("   } forall\n");
   printf ("   /Encoding ISOLatin1Encoding def\n");
   printf ("   currentdict\n");
   printf ("end\n");
   printf ("/Helvetica-ISOLatin1-Bold exch definefont pop\n");
   printf ("\n");
   return;
}

