// source-data-epoch.c

#define _POSIX_C_SOURCE 200809L  // strdup
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include "fail.h"

static char *const BLDDIR = "BUILD";
static char        buf[4096];
static char       *files[128];


uint32_t
source_date_epoch (const char *memory_ld)
{
   DIR            *blddir;
   struct dirent  *dfile;
   struct stat     dstat;
   size_t          bytes;
   time_t          latest;
   int             dep, i1;
   char           *text, *s1, *file;

   files[0] = strdup (memory_ld);
// files[1] = "../makefile.mk";

   if (chdir (BLDDIR)) {
      fail (errno, "chdir %s", BLDDIR);
   }
   blddir = opendir (".");
   if (blddir == NULL) {
      fail (errno, "opendir %s", BLDDIR);
   }

   dep = -1;
   while ((dfile = readdir (blddir))) {
      if (*dfile->d_name == '.') {
         continue;
      }
      if (strcmp (dfile->d_name + strlen (dfile->d_name) - 2, ".d")) {
         continue;
      }
//    printf ("---- %s\n", dfile->d_name);
      dep = open (dfile->d_name, O_RDONLY);
      if (dep == -1) {
         fail (errno, "open %s", dfile->d_name);
      }
      bytes = read (dep, buf, sizeof buf - 1);
      close (dep);
      if (bytes <= 0 || bytes >= sizeof buf) {
         fail (errno, "read %s", dfile->d_name);
      }
      buf[bytes] = '\0';

      for (text = buf; *text; text++) {
         if (*text == '\\' || *text == '\n') {
            *text = ' ';
         }
      }

      text = strchr (buf, ':');
      if (text == NULL) {
         fail (0, "%s: ':' not found", dfile->d_name);
      }
      text += 1;
      while ((s1 = strtok (text, " "))) {
         text = NULL;
         if (*s1 == '\0') { continue; }
         if (strcmp (s1, "lib-table.c") == 0)   { continue; }
         if (strcmp (s1, "lib-vectors.c") == 0) { continue; }
         if (strcmp (s1, "shm-tables.c") == 0)  { continue; }
//       printf ("'%s'\n", s1);
         for (i1 = 0; files[i1]; i1++) {
            if (strcmp (s1, files[i1]) == 0) { break; }
         }
         if (files[i1] == NULL) {
            files[i1] = strdup (s1);
         }
      }
   }
   if (dep == -1) {
      fail (0, "no files, %s is clean", BLDDIR);
   }
   closedir (blddir);
// printf ("========================\n");

   if (chdir ("..")) {
      fail (errno, "chdir ..");
   }
   latest = 0;
   file = "???";
   for (i1 = 0; files[i1]; i1++) {
      if (stat (files[i1], &dstat)) {
         fail (errno, "stat %s", files[i1]);
      }
      if (strncmp (files[i1], "../", 3) == 0) {
         if (strncmp (files[i1], "../include/", 11) == 0) {
            files[i1] += 11;
         } else {
            files[i1] += 3;
         }
      }
//    printf ("%ld  %s\n", dstat.st_mtime, files[i1]);
      if (dstat.st_mtime > latest) {
         latest = dstat.st_mtime;
         file = files[i1];
      }
   }

   strftime (buf, sizeof buf, "%F %T", localtime (&latest));
   printf ("%ld  %X  %s  %s\n", latest, (uint32_t)latest, buf, file);
   return latest;
}


// int
// main (void)
// {
//    source_date_epoch ();
//    return 0;
// }

