// srec-to-update.c

#define _POSIX_C_SOURCE 200809L  // strdup
#include <ctype.h>
#include <errno.h>
#include <libgen.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "srecord.h"
#include "fail.h"
#include "misc-tools.h"
#include "../include/image-header.h"

static image_struct dummy;
//static const size_t OFF_PRG = offsetof (image_struct, name);
//static const size_t SIZ_PRG = sizeof dummy.name;
//static const size_t OFF_VER = offsetof (image_struct, version_string);
static const size_t SIZ_VER = sizeof dummy.version_string;
//static const size_t OFF_VNO = offsetof (image_struct, verno);
//static const size_t OFF_RNO = offsetof (image_struct, revno);
static const size_t OFF_END = offsetof (image_struct, end);
//static const size_t OFF_CRC = offsetof (image_struct, crc);
//static const size_t OFF_SDE = offsetof (image_struct, source_date_epoch);
static const size_t OFF_TXT = offsetof (image_struct, text_start);

//uint32_t  source_date_epoch (const char *memory_ld);


static uint32_t
header (img_st *image, size_t offset)
{
   size_t   ads;
   uint32_t value;

   ads = image->hw_ads & image->imask;
   ads += offset;
   value  = (uint32_t)image->data[ads + 0] <<  0;
   value |= (uint32_t)image->data[ads + 1] <<  8;
   value |= (uint32_t)image->data[ads + 2] << 16;
   value |= (uint32_t)image->data[ads + 3] << 24;
   return value;
}


static int
check_header (img_st *image)
{
   uint32_t  end, txt;

   txt = header (image, OFF_TXT);
   end = header (image, OFF_END);
   if ((uint32_t)image->hw_ads != txt) {
      fail (0, "\n   text_start = 0x%08x, image->hw_ads = 0x%08x",
            txt, image->hw_ads);
   }
   if (((uint32_t)image->end - end) > 12) {
      fail (0, "\n   end = 0x%08x, image->end = 0x%08x", end, image->end);
   }
   return image->end - image->hw_ads + 1;
}


char *
version (char *local_version, int *vno, int *rno)
{
   static const char *valid_chars =
      "0123456789.-ABCDEFGHIJKLMNOPQRSTUVWXYZ-abcdefghijklmnopqrstuvwxyz";
   static char  buf[88];
   FILE        *verh;
   char        *s1, *logl, *version = NULL;

   if (local_version && *local_version) {
      version = local_version;
      logl = "local version";
   } else {
      logl = "global version";
      verh = fail_or_open_ro ("version");
      while (fgets (buf, sizeof buf, verh)) {
         fail_line += 1;
         if (*buf == '/') { continue; }
         s1 = strchr (buf, '\n');
         if (s1 == NULL) {
            fail (0, "Line too long");
         }
         *s1 = '\0';
         s1 = strchr (buf, '\r');
         if (s1) {
            *s1 = '\0';
         }
         if (strlen (buf) == 0) { continue; }
         if (version != NULL) {
            fail (0, "Multiple version strings '%s', '%s'", version, buf);
         }
         version = strdup (buf);
      }
      fclose (verh);
   }

   if (version == NULL) {
      fail (0, "Version string not found");
   }
   if (strlen (version) >= SIZ_VER) {
      fail (0, "%s too long '%*.*s'", logl, SIZ_VER, SIZ_VER, version);
   }
   for (s1 = version; *s1; s1++) {
      if (strchr (valid_chars, *s1) == NULL) {
         fail (0, "Illegal character '%c' in %s '%s'", *s1, logl, version);
      }
   }
   if (version[0] == 'V' && version[3] == '.'
       && isdigit (version[1]) && isdigit (version[2])
       && isdigit (version[4]) && isdigit (version[5])) {
      *vno = atoi (version + 1);
      *rno = atoi (version + 4);
      if (*vno >= 94 || *rno >= 94) {
         fail (0, "'%s' >= 94 (DAZ limit)", version);
      }
   } else {
      *vno = 0;
      *rno = 0;
   }
   return version;
}


int
main (int argc, char *argv[])
{
   static char  s19name[256], txtname[256], header[256];
   img_st      *img[7];
   uint32_t     source_date_epoch;
   int          vno, rno, ax, imgcnt, total;
   char        *arg;
   int          sectors = sizeof img / sizeof img[0];

   if (argc < 2 || argc > sectors) {
      fail_usage ("image.s19... (max. %d images)", sectors - 1);
   }
   snprintf (txtname, sizeof txtname, "LTPFLASH.ROM");
   total = 0;
   imgcnt = 0;
   source_date_epoch = 0;

   for (ax = 1; ax < argc; ax++) {
      arg = argv[ax];
//    snprintf (elfname, sizeof elfname, "%s.elf", arg);
      snprintf (s19name, sizeof s19name, "%s/BUILD/%s.s19", arg, arg);
//    snprintf (tmpname, sizeof tmpname, "%s-s19.tmp", arg);

//    elf2srec (elfname, tmpname);
      img[ax] = srec_load (s19name);
      total += check_header (img[ax]);
//    srec_print (txtname, img[ax]);
//    srec_print_bin (txtname, img[ax]);
      if (img[ax]->time > source_date_epoch) {
         source_date_epoch = img[ax]->time;
      }
      imgcnt += 1;
   }

   printf ("total = %d = 0x%x\n", total, total);

   version (NULL, &vno, &rno);
   snprintf (header, sizeof header, "LTP V%02d.%02d %s",
             vno, rno, iso_time (source_date_epoch));
   unlink (txtname);
   for (ax = 1; ax <= imgcnt; ax++) {
      srec_print_base85 (txtname, img[ax], 48, header);
      *header = '\0';
   }
   return 0;
}

