// srecord.c

#include <errno.h>
#include <libgen.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "srecord.h"
#include "fail.h"
#include "misc-tools.h"
#include "../include/image-header.h"

//static image_struct dummy;
//static const size_t OFF_PRG = offsetof (image_struct, name);
//static const size_t SIZ_PRG = sizeof dummy.name;
//static const size_t OFF_VER = offsetof (image_struct, version_string);
//static const size_t SIZ_VER = sizeof dummy.version_string;
static const size_t OFF_VNO = offsetof (image_struct, verno);
static const size_t OFF_RNO = offsetof (image_struct, revno);
//static const size_t OFF_END = offsetof (image_struct, end);
//static const size_t OFF_CRC = offsetof (image_struct, crc);
static const size_t OFF_SDE = offsetof (image_struct, source_date_epoch);
// static const size_t OFF_LIB = offsetof (image_struct, syslib_version);

static const int  flash_size = 1024 * 1024;
static const int  hextab[256] = {
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   0,   1,  2,  3,    4,  5,  6,  7,    8,  9, -1, -1,   -1, -1, -1, -1,
   -1, 10, 11, 12,   13, 14, 15, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,
   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1,   -1, -1, -1, -1
};


static uint32_t
hex2int (const char *text, int chars)
{
   uint32_t  value;
   uint8_t   ascii;
   int       nibble;

   for (value = 0; chars > 0; chars--) {
      ascii = (uint8_t) (*text++);
      nibble = hextab[ascii];
      if (nibble == -1) {
         fail (0, "invalid hex digit '0x%2.2x'", *--text & 255);
      }
      value = (value << 4) | nibble;
   }
   return value;
}


static void
check_checksum (const char *buf, int count)
{
   int  i1, checksum = 0;

   for (i1 = 1; i1 <= count; i1++) {
      checksum += hex2int (buf + i1 * 2, 2);
   }
   checksum = (~checksum) & 255;
   i1 = hex2int (buf + 2 + count * 2, 2);
   if (checksum != i1) {
      fail (0, "checksum error 0x%x", checksum);
   }
   return;
}


static void
check_eol (char *eol)
{
   if ((eol[0] == '\n' && eol[1] == '\0')
       || (eol[0] == '\r' && eol[1] == '\0')
       || (eol[0] == '\n' && eol[1] == '\r' && eol[2] == '\0')
       || (eol[0] == '\r' && eol[1] == '\n' && eol[2] == '\0')) {
      eol[0] = '\0';
      return;
   }
   fail (0, "junk at end of line");
}


uint32_t
srec_header (img_st *image, size_t offset)
{
   size_t   ads;
   uint32_t value;

   ads = image->hw_ads & image->imask;
   ads += offset;
   value  = (uint32_t)image->data[ads + 0] <<  0;
   value |= (uint32_t)image->data[ads + 1] <<  8;
   value |= (uint32_t)image->data[ads + 2] << 16;
   value |= (uint32_t)image->data[ads + 3] << 24;
   return value;
}


img_st *
srec_load (const char *s19name)
{
   static img_st *image;
   FILE          *s19;
   char           buf[512];
   int            ads, hw_bottom, hw_top = 0;
   int            i1, ads_chars, count, byte_count;

   s19 = fail_or_open_ro (s19name);

   image = malloc (sizeof (img_st));
   image->hw_ads = 0x7fffffff;
   image->end = 0;
   image->imask = 0xfffff;
   image->data = malloc (flash_size);
   memset (image->data, 255, flash_size);

   errno = 0;
   while (fgets (buf, sizeof buf, s19)) {
      fail_line += 1;
      if (buf[0] != 'S') {
         fail (0, "not in s-record format");
      }
      count = hex2int (buf + 2, 2);
      check_checksum (buf, count);
      check_eol (buf + 2 + count * 2 + 2);
      switch (buf[1]) {
      case '0':
         for (i1 = 3; i1 < count; i1++) {
            image->comment[i1 - 3] = hex2int (buf + 2 + i1 * 2, 2);
         }
         image->comment[i1 - 3] = '\0';
         continue;
      case '1':
         ads_chars = 4;
         break;
      case '2':
         ads_chars = 6;
         break;
      case '3':
         ads_chars = 8;
         break;
      case '5':
         continue;
      case '6':
         continue;
      case '7':
         ads_chars = 8;
         break;
      case '8':
         ads_chars = 6;
         break;
      case '9':
         ads_chars = 4;
         break;
      default:
         fail (0, "unknown s-record format");
      }

      byte_count = count - (ads_chars / 2 + 1);
      if (byte_count < 0) {
         fail (0, "byte count = %d", count);
      }
      if (byte_count == 0) { continue; }

      ads = hex2int (buf + 4, ads_chars);
      if (ads < image->hw_ads) {
         image->hw_ads = ads;
      }
      if (hw_top == 0) {
//       if (image->hw_ads < 0x20002000) {
         if (image->hw_ads < 0x20000000) {
            hw_bottom = 0x08000000;
            hw_top    = hw_bottom + 256 * 1024;
         } else {
//          hw_bottom = 0x20002000;
            hw_bottom = 0x20000000;
//          hw_top    = hw_bottom + (128 - (8 + 16)) * 1024;
            hw_top    = hw_bottom + (128 - (0 + 16)) * 1024;
         }
      }
      for (i1 = 0; i1 < byte_count; i1++) {
         if (ads < hw_bottom || ads >= hw_top) {
            fail (0, "illegal address 0x%x", ads);
         }
         if (ads > image->end) {
            image->end = ads;
         }
         image->data[ads++ & image->imask] =
            hex2int (buf + 4 + ads_chars + i1 * 2, 2);
      }
   }
   if (errno) {
      fail (errno, "fgets loop");
   }
   fclose (s19);

   if (image->hw_ads & (2048 - 1)) {
      fail (0, "illegal hardware address 0x%x", image->hw_ads);
   }
   image->start_ads = image->hw_ads;
   image->verno = image->data[OFF_VNO];
   image->revno = image->data[OFF_RNO];
   image->time = srec_header (image, OFF_SDE);
   printf ("hardware_ads: 0x%.8x .. 0x%.8x\n", image->hw_ads, image->end);
   return image;
}


void
srec_print (const char *filename, img_st *image, int bpl, int pic_offset)
{
   FILE  *s19;
   int    i1, i2, checksum, comlen;
   int    ads = image->hw_ads;

   printf ("srec_print: %s at 0x%x\n", image->comment, ads);

   s19 = fail_or_append (filename);

   comlen = strlen (image->comment);
   if (comlen) {
      fprintf (s19, "S0%2.2X0000", comlen + 3);
      checksum = comlen + 3;
      for (i1 = 0; i1 < comlen; i1++) {
         checksum += image->comment[i1];
         fprintf (s19, "%2.2X", image->comment[i1]);
      }
      fprintf (s19, "%2.2X\r\n", (~checksum) & 255);
   }

   for (i2 = 0; i2 <= image->imask; i2 += bpl) {
      checksum = 0;
      for (i1 = 0; i1 < bpl; i1++) {
         checksum += image->data[ads++ & image->imask];
      }
      if (checksum == bpl * 255) { continue; }

      ads -= bpl;
      checksum += 5 + bpl;
      checksum += ((ads + pic_offset) >> 24) & 255;
      checksum += ((ads + pic_offset) >> 16) & 255;
      checksum += ((ads + pic_offset) >>  8) & 255;
      checksum += ((ads + pic_offset) >>  0) & 255;
      fprintf (s19, "S3%2.2X%8.8X", 5 + bpl, ads + pic_offset);
      for (i1 = 0; i1 < bpl; i1++) {
         fprintf (s19, "%2.2X", image->data[ads++ & image->imask]);
      }
      fprintf (s19, "%2.2X\r\n", (~checksum) & 255);
   }
   ads = image->hw_ads;
   checksum = 0;
   checksum += 5;
   checksum += (ads >> 24) & 255;
   checksum += (ads >> 16) & 255;
   checksum += (ads >>  8) & 255;
   checksum += (ads >>  0) & 255;
   fprintf (s19, "S7%2.2X%8.8X", 5, ads);
   fprintf (s19, "%2.2X\r\n", (~checksum) & 255);
   fclose (s19);
   return;
}


void
srec_print_base64 (const char *filename, img_st *image)
{
   static const char  bc[64] =
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
   FILE     *txt;
   uint32_t  bits;
   int       ads, i1, bpl = 48, c1, c2, c3, c4;

   printf ("srec_print: %s at 0x%x\n", image->comment, image->hw_ads);

   txt = fail_or_create (filename);

   for (ads = image->hw_ads; ads <= image->end; /**/) {
      for (i1 = 0; i1 < bpl; i1 += 3) {
         bits  = image->data[ads++ & image->imask] << 16;
         bits |= image->data[ads++ & image->imask] << 8;
         bits |= image->data[ads++ & image->imask];
         c1 = (bits >> 18) & 63;
         c2 = (bits >> 12) & 63;
         c3 = (bits >>  6) & 63;
         c4 = (bits >>  0) & 63;
         fprintf (txt, "%c%c%c%c", bc[c1], bc[c2], bc[c3], bc[c4]);
      }
      fprintf (txt, "\r\n");
   }
   fclose (txt);
   return;
}


static char *
base85 (uint32_t bits)
{
   static int8_t  bc85[85] = {
//    ' ',
      '!',
//    '"',
//    '#',
      '$',
      '%',
//    '&',
//    ''',
      '(',
      ')',
      '*',
      '+',
      ',',
      '-',
      '.',
      '/',
      '0',
      '1',
      '2',
      '3',
      '4',
      '5',
      '6',
      '7',
      '8',
      '9',
      ':',
      ';',
//    '<',
      '=',
//    '>',
      '?',
      '@',
      'A',
      'B',
      'C',
      'D',
      'E',
      'F',
      'G',
      'H',
      'I',
      'J',
      'K',
      'L',
      'M',
      'N',
      'O',
      'P',
      'Q',
      'R',
      'S',
      'T',
      'U',
      'V',
      'W',
      'X',
      'Y',
      'Z',
      '[',
//    '\'",
      ']',
      '^',
      '_',
//    '`',
      'a',
      'b',
      'c',
      'd',
      'e',
      'f',
      'g',
      'h',
      'i',
      'j',
      'k',
      'l',
      'm',
      'n',
      'o',
      'p',
      'q',
      'r',
      's',
      't',
      'u',
      'v',
      'w',
      'x',
      'y',
      'z',
      '{',
      '|',
      '}'
//    '~'
//    DEL
      };
   static char  text[8];

   text[4] = bc85[bits % 85];
   bits = bits / 85;
   text[3] = bc85[bits % 85];
   bits = bits / 85;
   text[2] = bc85[bits % 85];
   bits = bits / 85;
   text[1] = bc85[bits % 85];
   bits = bits / 85;
   text[0] = bc85[bits % 85];
   text[5] = '\0';
   return text;
}


void
srec_print_base85 (const char *filename, img_st *image, int bpl, char *header)
{
   FILE     *txt;
   uint32_t  bits;
   int       ads, i1, size;

   printf ("srec_print: %s at 0x%x\n", image->comment, image->hw_ads);
   size = image->end - image->hw_ads + 1;

   txt = fail_or_append (filename);
   if (header && *header) {
      fprintf (txt, "# %s\r\n#\r\n", header);
      fprintf (txt, "# What is the .ROM file type?\r\n"
               "# \r\n"
               "# The .rom filename extension chiefly serves to denote\r\n"
               "# generic Read-Only Memory (ROM) images saved as files.\r\n"
               "# A .rom file can hold PC BIOS firmware or contents of\r\n"
               "# an electronic device's ROM. It contains identical\r\n"
               "# representation of a binary program code placed into\r\n"
               "# the ROM (flash) memory for persistent storage.\r\n"
               "#\r\n"
               "# ROM files usually have specific sizes that are multiple\r\n"
               "# of 8 (256, 512, and so on), although the exact format\r\n"
               "# of a given .rom file depends on its origin and purpose.\r\n"
               "# ROM files are commonly used for distributing and\r\n"
               "# updating firmware and may have several other extensions\r\n"
               "# (.bin, .ami) or no extension at all.\r\n"
               "# (http://www.filetypeadvisor.com/extension/rom)\r\n"
               "#\r\n");
   }
   fprintf (txt, "# 0x%08x %s %s %d\r\n",
            image->hw_ads, image->comment, iso_time (image->time), size);

   for (ads = image->hw_ads; ads <= image->end; /**/) {
      for (i1 = 0; i1 < bpl; i1 += 4) {
         bits  = image->data[ads++ & image->imask] << 24;
         bits |= image->data[ads++ & image->imask] << 16;
         bits |= image->data[ads++ & image->imask] << 8;
         bits |= image->data[ads++ & image->imask];
         fprintf (txt, "%s", base85 (bits));
      }
      fprintf (txt, "\r\n");
   }
   fclose (txt);
   return;
}


void
srec_print_bin (const char *filename, img_st *image)
{
   FILE  *txt;
   int    ads, i1, bpl = 16;

   printf ("srec_print: %s at 0x%x\n", image->comment, image->hw_ads);

   txt = fail_or_append (filename);
   for (ads = image->hw_ads; ads <= image->end; /**/) {
      for (i1 = 0; i1 < bpl; i1 += 1) {
         fputc (image->data[ads++ & image->imask], txt);
      }
   }
   fclose (txt);
   return;
}

