//  crash.c

// Replacement for _unhandled_exception()
// from Chibios/os/ports/GCC/ARMCMx/STM32F[024]xx/vectors.c
// http://community.arm.com/message/16111 (dead)

#include <stddef.h>
#include "F205/undocumented.h"
#include "F205/scb.h"
#include "asm.h"
#include "exec.h"

// extern task_struct  *current;

void print_debug_event (const char *delim);


void __attribute__ ((naked))
crash (void)
{
   __asm__ ("cpsid i");
   __asm__ ("mrs    r3,MSP");
   __asm__ ("mrs    r2,PSP");
   __asm__ ("tst    lr,#4");
   __asm__ ("ite    eq");
   __asm__ ("mrseq  r1,MSP");
   __asm__ ("mrsne  r1,PSP");

   __asm__ ("mov    r0,#0");
   __asm__ ("movt   r0,#0");
   __asm__ ("mov    sp,r0");
   __asm__ ("mov    r0,#0xED94");
   __asm__ ("movt   r0,#0xE000");
   __asm__ ("str    sp,[r0,0]");

   __asm__ ("mov    r0,#0x400");
   __asm__ ("movt   r0,#0x2000");
   __asm__ ("mov    sp,r0");
   __asm__ (                      "push  {lr}"); // 20
   __asm__ (                      "push  {r3}"); // 19  MSP
   __asm__ (                      "push  {r2}"); // 18  PSP
   __asm__ ("mrs    r0,IPSR\n"    "push  {r0}"); // 17
   __asm__ ("ldr    r0,[r1,28]\n" "push  {r0}"); // 16  PSR
   __asm__ ("ldr    r0,[r1,24]\n" "push  {r0}"); // 15  pc
   __asm__ ("ldr    r0,[r1,20]\n" "push  {r0}"); // 14  lr
   __asm__ (                      "push  {r1}"); // 13  sp
   __asm__ ("ldr    r0,[r1,16]\n" "push  {r0}"); // 12  r12
   __asm__ (                      "push  {r11}");
   __asm__ (                      "push  {r10}");
   __asm__ (                      "push  {r9}");
   __asm__ (                      "push  {r8}");
   __asm__ (                      "push  {r7}");
   __asm__ (                      "push  {r6}");
   __asm__ (                      "push  {r5}");
   __asm__ (                      "push  {r4}");
   __asm__ ("ldr    r0,[r1,12]\n" "push  {r0}"); //  3  r3
   __asm__ ("ldr    r0,[r1,8]\n"  "push  {r0}"); //  2  r2
   __asm__ ("ldr    r0,[r1,4]\n"  "push  {r0}"); //  1  r1
   __asm__ ("ldr    r0,[r1,0]\n"  "push  {r0}"); //  0  r0
   __asm__ ("mov    r0,sp");
   __asm__ ("b      crashdump");
}


void
crashdump (uint32_t regs[])
{
   static const char  *exmsg[16] = {
      "Thread mode crashdump", //  0
      "",                      //  1
      "NMI",                   //  2
      "Hard fault",            //  3
      "MPU fault",             //  4
      "Bus fault",             //  5
      "Usage fault",           //  6
      "",                      //  7
      "",                      //  8
      "",                      //  9
      "",                      // 10
      "SVC",                   // 11
      "Debug exception",       // 12
      "",                      // 13
      "PendSV",                // 14
      "SysTick"                // 15
   };
   static const char  *cfsr_msg[] = {
      "Instruction access violation",
      "Data access violation",
      "F2",
      "MM fault on unstacking for a return from exception",
      "MM fault on stacking for exception entry",
      "F5",
      "FP lazy state preservation",
      "MM Fault Address Register valid",
      "Instruction bus error",
      "Precise data bus error",
      "Imprecise data bus error",
      "Bus fault on unstacking for a return from exception",
      "Bus fault on stacking for exception entry",
      "F13",
      "F14",
      "Bus Fault Address Register valid",
      "Undefined instruction",
      "Invalid state",
      "Invalid PC load",
      "No coprocessor",
      "F20",
      "F21",
      "F22",
      "F23",
      "Unaligned access",
      "Divide by zero",
      "CFS26",
      "F27",
      "F28",
      "Vector read fault",
      "Forced",
      "Debug event"
   };
// task_struct  crash_tcb, *victim;
// stdio_struct uart1;
   uint32_t    *msp, *psp, *ads, at;
   int          i1, i2, cfsr, isrno;


// DWT->F0_FUNCTION = 0; // OBACHT:
   DEBUG->TRCENA = 0;    // debug module brauchen einen power-on-reset!

// board_shutdown ();

   isrno = regs[17] & 511;
   at = (isrno == 0) ? regs[14] : regs[15];
   psp = (uint32_t *)regs[18];
   msp = (uint32_t *)regs[19];
   cfsr = SCB->CFSR | (SCB->HFSR & 0xc0000000) | (SCB->HFSR << 28);
// victim = current;
// current = &crash_tcb;
// do_uart_init (1, 115200, &uart1);

   printf ("\n");
   ads = (uint32_t *) ((uint32_t)psp & ~ 511);
   for (i2 = 0; i2 < 32; i2++) {
      printf ("%08lx", (uint32_t)ads);
      for (i1 = 0; i1 < 8; i1++) {
         if (ads == psp) {
            printf ("\033[36m" " %08lx" "\033[m", *ads++);
         } else {
            printf (" %08lx", *ads++);
         }
      }
      printf ("\n");
   }

   if (isrno > 15 || *exmsg[isrno] == '\0') {
      printf ("\nException #%d", isrno);
   } else {
      if (isrno == 12) {
//    if ((SCB->DFSR & DFSR_WATCHPOINT) && DWT->F0_MATCHED) {
//      printf ("Stack overflow");
//    } else {
         print_debug_event ("\n");
//    }
      } else {
         if (cfsr == 0) {
            printf ("\n\033[33m" "%s" "\033[m", exmsg[isrno]);
         } else {
            printf ("\n%s", exmsg[isrno]);
         }
      }
   }
   printf (" at %lx, ", at);
   printf ("%s, ", (regs[14] & 4) ? "PSP" : "MSP");
   printf ("%s, ", (get_CONTROL () & 1) ? "USER" : "SYS");
   printf ("PSR: %c%c%c%c%c %s ISR #%ld\n",
           regs[16] & (1 << 31) ? 'N' : '-',
           regs[16] & (1 << 30) ? 'Z' : '-',
           regs[16] & (1 << 29) ? 'C' : '-',
           regs[16] & (1 << 28) ? 'V' : '-',
           regs[16] & (1 << 27) ? 'Q' : '-',
           regs[16] & (1 << 24) ? "T" : "\033[31;1m" "T=0" "\033[m",
           regs[16] & 511);

   for (i1 = 31; i1 >= 0; i1--) {
      if (cfsr & (1 << i1)) {
         switch (i1) {
         case 31:
            printf ("%s" "\033[33m", cfsr_msg[i1]);
            print_debug_event (", ");
            printf ("\033[m\n");
            break;
         case 15:
            printf ("%s \033[36m0x%.8lx\033[m\n", cfsr_msg[i1], SCB->BFAR);
            break;
         case 7:
            printf ("%s \033[36m0x%.8lx\033[m\n", cfsr_msg[i1], SCB->MMFAR);
            break;
         default:
            printf ("\033[33m" "%s" "\033[m\n", cfsr_msg[i1]);
         }
      }
   }

   printf ("\nPSP,MSP %8lx %8lx\n", (uint32_t)psp, (uint32_t)msp);

   printf ("  r0-r7 %8lx %8lx %8lx %8lx %8lx %8lx %8lx %8lx\n",
           regs[0], regs[1], regs[2], regs[3],
           regs[4], regs[5], regs[6], regs[7]);
   printf (" r8-r15 %8lx %8lx %8lx %8lx %8lx %8lx %8lx %8lx\n",
           regs[8],  regs[9],  regs[10], regs[11],
           regs[12], regs[13], regs[14], regs[15]);

#if 0
   msp = (uint32_t *) (regs[19] + 80);
   psp = (uint32_t *) (regs[18] + 80);
   for (i1 = 0; i1 < 24; i1++) {
      printf ("%.8x  %.8x", (int)msp, *msp);
      printf (" %s  ", (msp == (int *)regs[19]) ? "<-MSP" : "     ");
      printf ("%.8x  %.8x", (int)psp, *psp);
      printf (" %s\n", (psp == (int *)regs[18]) ? "<-PSP" : "     ");
      msp--;
      psp--;
   }
#endif
   printf ("\n");
// mpu_show ();
// show_tasks (victim->pid);
// trace_show ();
   printf ("../tools/addr-to-line %08lx\n", at);

   while (1) {
      for (i2 = 0; i2 < 42; i2++) {
         for (i1 = 0; i1 < 420000; i1 += 2) {
            led (i1 / 4096);
         }
//       beep (0, 0);
      }
      printf ("Restart\n");
//    board_reboot ();
   }
}


void
print_debug_event (const char *delim)
{
   if (SCB->DFSR & DFSR_HALT)   { printf ("%s" "Debug halt",   delim); }
   if (SCB->DFSR & DFSR_BREAK)  { printf ("%s" "Breakpoint",   delim); }
   if (SCB->DFSR & DFSR_WATCH)  { printf ("%s" "Watchpoint",   delim); }
   if (SCB->DFSR & DFSR_VECTOR) { printf ("%s" "Vector catch", delim); }
   if (SCB->DFSR & DFSR_EXTERN) { printf ("%s" "External",     delim); }
}

