// delay.c

#include <stddef.h>
#include <stdint.h>
#include "asm.h"
#include "boot.h"
#include "F205/rcc.h"
#include "F205/tim67.h"


void
sleep_ms (int ms)
{
   while (ms >= 0) {
      delay_us (1000);
      ms -= 1;
   }
   return;
}


void
delay_us (uint16_t us)
{
// Die maximale Verzgerung ist unbegrenzt wenn eine Task mit hherer
// Prioritt dazwischen kommt (dagegen hilft nice()). Gegen den systick
// Interrupt (beobachtet: ca. 500 Takte) knnte man evt. etwas machen,
// gegen UART-RX Interrupts eher nicht.

// Der minimal beobachtete Overhead war 13 Taktzyklen. Mit dem aktuellen
// Test aus init/show.c waren es minimal 16 Takte.
// Typische Werte bei 16MHz sind
// Soll 3us: 25, 37
//      4us: 21, 33
//      5us: 17, 29, 41
//      6us: 25, 37
//      7us: 21, 33
//      8us: 17, 29, 41
// Im Mittel (ohne Bercksichtigung der Interrupts) sind es 29 Takte.

// Eigentlich sollte man den DWT CYCCNT benutzen, aber darauf kann man im
// User Mode nicht zugreifen. Mit einer besseren MPU knnte man eine extra
// Region einrichten. Mit einem SVC wird whrend der Wartezeit der scheduler
// blockiert. Das hrt sich gut an, ntzt den Aufrufenden aber nicht viel,
// weil er dann eben kurz nach dem delay_us() unterbrochen wird. Ansonsten
// strt es eher. Und der Overhead ist immens. Deshalb spendieren wir einen
// extra Hardware Timer.

   uint16_t  t0;

   t0 = TIM7->CNT;
   while ((uint16_t) (TIM7->CNT - t0) < us) { };
   return;
}


void
delay_init (void)
{
   RCC->APB1ENR |= 1 << APB1_TIM7;              // TIM7 =
   DSB ();                                      // free running
   TIM7->PSC = 16000000 / 1000000 - 1;          // 1us time base
   TIM7->ARR = 65535;                           // needed by
   TIM7->CR1_CEN = 1;                           // delay_us()
   TIM7->EGR_UG = 1;                            //
   return;
}

