// erase-sector.c

#include <stdint.h>
#include "F205/flash.h"
#include "F205/iwdg.h"
#include "F205/scb.h"
#include "boot.h"


int
erase_sector (int sector)
{
   static uint32_t * const  base[12+1] = {
      (uint32_t *)0x08000000,  // 00
      (uint32_t *)0x08004000,  // 01
      (uint32_t *)0x08008000,  // 02
      (uint32_t *)0x0800C000,  // 03
      (uint32_t *)0x08010000,  // 04
      (uint32_t *)0x08020000,  // 05
      (uint32_t *)0x08040000,  // 06
      (uint32_t *)0x08060000,  // 07
      (uint32_t *)0x08080000,  // 08
      (uint32_t *)0x080A0000,  // 09
      (uint32_t *)0x080C0000,  // 10
      (uint32_t *)0x080E0000,  // 11
      (uint32_t *)0x08100000,  // --
   };
   uint32_t  *test;

   if (sector < 0 || sector > 11) {
      printf ("Bad sector %d\n", sector);
      return -1;
   }

   for (test = base[sector]; test < base[sector + 1] - 1; test++) {
      if (*test != 0xFFFFFFFF) { break; }
   }
   if (test == base[sector + 1] - 1) {
      printf ("Sector %d is empty\n", sector);
      return 0;
   }
   FLASH->KEY = FLASH_KEY1;
   FLASH->KEY = FLASH_KEY2;
   if (FLASH->LOCK) {
      printf ("Flash is locked\n");
      return -1;
   }
   if (FLASH->BSY) {
      printf ("Flash is busy\n");
      return -1;
   }
   printf ("Erasing sector %d ", sector);
   FLASH->PSIZE = FLASH_PSIZE_32_BIT;
   FLASH->SER = 1;
   FLASH->SNB = sector;
   FLASH->STRT = 1;
   while (FLASH->BSY) {
      putchar ('.');
   }
   FLASH->LOCK = 1;
   printf ("\ndone\n");
   return 0;
}

