// find-syslib.c

#include <stddef.h>
#include <stdint.h>
#include "boot.h"
#include "image-header.h"


static int
is_taskname (const char *text, size_t size)
{
   const char *s1;

   if (*text < 'a' || *text > 'z') {
      return 0;
   }
   for (s1 = text + 1; s1 < text + size - 1; s1++) {
      if (*s1 == '\0') { break; }
      if (*s1 < '0' || *s1 > 'z') { return 0; }
      if (*s1 > '9' && *s1 < 'a') { return 0; }
   }
   for (/**/; s1 < text + size; s1++) {
      if (*s1 != '\0') { return 0; }
   }
   return 1;
}


image_struct *
find_syslib (void)
{
static const uint32_t  sectorlist_F205[] = {
// 0x20000000,  //       8K = system memory boot loader (ram)
// 0x20002000,  //       4K = vdrive
// 0x20003000,  //       4K } = 72K application
   0x20004000, 0x20005000, 0x20006000, 0x20007000,
   0x20008000, 0x20009000, 0x2000A000, 0x2000B000,
// 0x2000C000, 0x2000D000, 0x2000E000, 0x2000F000,
   0x20010000, 0x20011000, 0x20012000, 0x20013000,
// 0x20014000,  //      16K  ???
// 0x20018000,  //      16K  ???
// 0x2001C000,  //      16K RAM
// 0x08000000,  //  0   16K = application boot loader (flash)
// 0x08004000,  //  1   16K
// 0x08008000,  //  2   16K
// 0x0800C000,  //  3   16K
// 0x08010000,  //  4   64K __ F205B, 128K (48+16K RAM = zu wenig)
// 0x08020000,  //  5  128K ___ F205C, 256K (80+16K RAM)
   0x08040000,  //  6  128K ____ F205D, 384K (den gibt es nicht)
   0x08060000,  //  7  128K _____ F205E, 512K (128K RAM)
   0x08080000,  //  8  128K
   0x080A0000,  //  9  128K ______ F205F, 768K
   0x080C0000,  // 10  128K
   0x080E0000,  // 11  128K _______ F205G, 1024K
   0
   };
   image_struct  *image;
   uint32_t       sector, top;
   int            sx;

   for (sx = 0; sectorlist_F205[sx] != 0; sx++) {
      sector = sectorlist_F205[sx];
      image = (image_struct *)sector;
      top = (uint32_t)image->end;
      printf ("0x%08lx  0x%08lx  %3d  0x%08lx  0x%08lx  0x%08lx\n",
              sector, top, image->image_type,
              sector + 2 * 1024, (uint32_t)image->crt0, top - 128);
      if (top < sector +  2 * 1024)             { continue; }
      if (top > sector + 64 * 1024)             { continue; }
      if ((uint32_t)image->crt0 < sector + 128) { continue; }
      if ((uint32_t)image->crt0 > top - 128)    { continue; }
      if (image->image_type != IT_SYSLIB)       { continue; }
      if (crc_check (image))                    { continue; }
      if (is_taskname (image->name, sizeof image->name)) {
         return image;
      }
   }
   return NULL;
}

