//  i2c-soft.c
//
//   S       7     6     5     4     3     2     1     0     A      P
//~~~\__/~~~~~\_____/~~~~~\_____/~~~~~\_____/~~~~~\_____/~~~~~\_____/~~~~~~~~~~
//~~~~~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~~~~~~~~~~~~
//|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
//-  0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5  6  7  8  9  0  1
//
//   S       7     6     5     4     3     2     1     0     A     7     6
//~~~\__/~~~~~\_____/~~~~~\_____/~~~~~\_____/~~~~~\_____/~~~~~\_____/~~~~~\____
//~~~~~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~~\__/~
//|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
//-  0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5  6  7  8  9  0  1  2  3  4
//      0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5  6  7

#include "F205/gpio.h"
#include "boot.h"
#include "mps1401.h"
//#include "syslib.h"

extern volatile uint32_t  task_timer;

static const int WRITE = 0;
static const int READ  = 1;

static int  i2c_data (int bits);
static int  i2c_ack (int ack);
static void i2c_stop (void);


static int
i2c_start (int ads, int rw)
{
   int  bus;

   bus = GPIOB->IDR & (PB_SDA | PB_SCL);
   if (bus != (PB_SCL | PB_SDA)) {
//    SYSLOG (55, "c" "I2C %d %c: BUSY", ads, rw ? 'R' : 'W');
      return -1;
   }
   GPIOB->BSR = PB_SDA | PB_SCL;
   delay_us (5);         // tSU-STA, tBUF

   GPIOB->BRR = PB_SDA;
   delay_us (5);         // tHD-STA
   GPIOB->BRR = PB_SCL;

   i2c_data ((ads << 1) | rw);
   if (i2c_ack (1)) {
//    SYSLOG (56, "c" "I2C %d %c: NAK", ads, rw ? 'R' : 'W');
      i2c_stop ();
      return -1;
   }
   return 0;
}


static int
i2c_data (int bits)
{
   int  bi, rdat;

   rdat = 0;
   for (bi = 128; bi >= 1; bi = bi >> 1) {
      if (bits & bi) {    // tHD-DAT
         GPIOB->BSR = PB_SDA;
      } else {
         GPIOB->BRR = PB_SDA;
      }
      delay_us (5);       // tLOW
      GPIOB->BSR = PB_SCL;
      delay_us (5);       // tHIGH
      rdat = rdat << 1;
      if (GPIOB->IDR & PB_SDA) {
         rdat |= 1;
      }
      GPIOB->BRR = PB_SCL;
   }
   return rdat;
}


static int
i2c_ack (int ack)
{
   if (ack == 0) {
      GPIOB->BRR = PB_SDA;
   } else {
      GPIOB->BSR = PB_SDA;
   }
   delay_us (5);       // tLOW
   GPIOB->BSR = PB_SCL;
   delay_us (5);       // tHIGH
   if ((GPIOB->IDR & PB_SDA) == 0) {
      ack = 0;
   }
   GPIOB->BRR = PB_SCL;
   delay_us (5);       // tLOW ?
   return ack;
}


static void
i2c_stop (void)
{
   GPIOB->BRR = PB_SCL;
   delay_us (5);
   GPIOB->BRR = PB_SDA;

   delay_us (5);       // tLOW
   GPIOB->BSR = PB_SCL;
   delay_us (5);       // tSU-STOP
   GPIOB->BSR = PB_SDA;
   delay_us (5);       // tBUF
   return;
}


int
i2c_send_x (int address, uint8_t *txbuf, int txcount)
{
   int  i1;

   if (txcount < 1 || txcount > 256) { return -1; }

// nice (-20);
   if (i2c_start (address, WRITE)) {
//    nice (0);
      return -1;
   }

   for (i1 = 0; i1 < txcount; i1++) {
      i2c_data (txbuf[i1]);
      i2c_ack (1);
   }
   i2c_stop ();
// nice (0);
   return 0;
}


int
i2c_send_2 (int address, uint8_t tx1, uint8_t tx2)
{
   uint8_t  txbuf[4];

   txbuf[0] = tx1;
   txbuf[1] = tx2;
   return i2c_send_x (address, txbuf, 2);
}


int
i2c_send_1 (int address, uint8_t tx1)
{
   uint8_t  txbuf[4];

   txbuf[0] = tx1;
   return i2c_send_x (address, txbuf, 1);
}


int
i2c_receive (int address, uint8_t *rxbuf, int rxcount)
{
   int  ri;

   rxcount -= 1;
   if (rxcount < 0 || rxcount > 15) { return -1; }

// nice (-20);
   if (i2c_start (address, READ)) {
//    nice (0);
      return -1;
   }

   for (ri = 0; ri <= rxcount; ri++) {
      rxbuf[ri] = i2c_data (255);
      if (ri < rxcount) {
         i2c_ack (0);
      } else {
         i2c_ack (-1);
      }
   }
   i2c_stop ();
// nice (0);
   return 0;
}


int
i2c_init (void)
{
   int  scl;

   GPIOB->BSR = PB_SCL | PB_SDA;
   GPIOB->MODER8 = MODE_OUTPUT;
   GPIOB->MODER9 = MODE_OUTPUT;
   delay_us (7);

   for (scl = 0; scl < 9; scl++) {
      if (GPIOB->IDR & PB_SDA) { break; }
      GPIOB->BRR = PB_SCL;
      delay_us (7);
      GPIOB->BSR = PB_SCL;
      delay_us (7);
   }
   if (scl > 0) {
//    SYSLOG (54, "i" "I2C init: %d sync pulses", scl, 0);
   }
   return 0;
}

