// uart1.c

#include <stddef.h>
#include "asm.h"
#include "F205/rcc.h"
#include "F205/uart.h"


int
getchar1 (void)
{
   while (UART1->SR_RXNE == 0) { }  // time-out per watchdog
   return UART1->DR;
}


int
putchar1 (int c1)
{
   while (UART1->SR_TXE == 0) {  }
   UART1->DR = c1;
   return c1;
}


int
putchar (int c1)
{
   if (c1 == '\n' || c1 == '\r') {
      putchar1 ('\r');
      putchar1 ('\n');
      return '\n';
   }
   return putchar1 (c1);
}


int
puts1 (const char *text)
{
   while (*text) {
      putchar1 (*text++);
   }
   putchar1 ('\n');
   return 0;
}

#if 0
int
dprintf (const char *fmt, ...)
{
   va_list  ap;

   va_start (ap, fmt);
   return vprintf (fmt, ap);
   va_end (ap);
}
#endif

void
uart1_init (int baud)
{
   static const int  br_clock = 16000000;
   uart_struct  *uart;

   RCC->APB2RSTR |= (1 << APB2_USART1);
   uart = UART1;
   ISB ();
   DSB ();
   RCC->APB2RSTR &= ~ (1 << APB2_USART1);

// br_clock ist genau genommen = (clocks->apbx / 16) * 16
// wegen 16 Clocks/Bit und 4 Bit fractional baud rate
   uart->BRR = (br_clock + baud / 2) / baud;

   uart->CR2_STOP2 = 1;
   uart->CR1_M   = 1;
   uart->CR1_PCE = 1;
   uart->CR1_TE  = 1;
   uart->CR1_RE  = 1;
   uart->CR1_UE  = 1;
   return;
}

